# Copyright 2022 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


from airflow import DAG
from airflow.providers.google.cloud.transfers import gcs_to_bigquery

default_args = {
    "owner": "Google",
    "depends_on_past": False,
    "start_date": "2022-08-20",
}


with DAG(
    dag_id="merfish.cell_by_gene",
    default_args=default_args,
    max_active_runs=1,
    schedule_interval="@weekly",
    catchup=False,
    default_view="graph",
) as dag:

    # Task to load CSV data to a BigQuery table
    load_cell_by_gene_to_bq = gcs_to_bigquery.GCSToBigQueryOperator(
        task_id="load_cell_by_gene_to_bq",
        bucket="public-datasets-vizgen-merfish",
        source_objects=[
            "datasets/mouse_brain_map/BrainReceptorShowcase/Slice1/Replicate1/cell_by_gene_S1R1.csv",
            "datasets/mouse_brain_map/BrainReceptorShowcase/Slice1/Replicate2/cell_by_gene_S1R2.csv",
            "datasets/mouse_brain_map/BrainReceptorShowcase/Slice1/Replicate3/cell_by_gene_S1R3.csv",
            "datasets/mouse_brain_map/BrainReceptorShowcase/Slice2/Replicate1/cell_by_gene_S2R1.csv",
            "datasets/mouse_brain_map/BrainReceptorShowcase/Slice2/Replicate2/cell_by_gene_S2R2.csv",
            "datasets/mouse_brain_map/BrainReceptorShowcase/Slice2/Replicate3/cell_by_gene_S2R3.csv",
            "datasets/mouse_brain_map/BrainReceptorShowcase/Slice3/Replicate1/cell_by_gene_S3R1.csv",
            "datasets/mouse_brain_map/BrainReceptorShowcase/Slice3/Replicate2/cell_by_gene_S3R2.csv",
            "datasets/mouse_brain_map/BrainReceptorShowcase/Slice3/Replicate3/cell_by_gene_S3R3.csv",
        ],
        source_format="CSV",
        destination_project_dataset_table="bigquery-public-data-dev.merfish.cell_by_gene",
        skip_leading_rows=1,
        allow_quoted_newlines=True,
        write_disposition="WRITE_TRUNCATE",
        schema_fields=[
            {"name": "Oxgr1", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Htr1a", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Htr1b", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Htr1d", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Htr1f", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Htr2a", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Htr2b", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Htr2c", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Htr4", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Htr5a", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Htr5b", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Htr6", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Htr7", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Adora1", "type": "string", "description": "", "mode": "nullable"},
            {
                "name": "Adora2a",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Adora2b",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {"name": "Adora3", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Adgra1", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Adgra2", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Adgra3", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Adgrb1", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Adgrb3", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Adgre1", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Adgre4", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Adgrf4", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Adgrf5", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Adgrl1", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Adgrl2", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Adgrl3", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Adgrl4", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Adgrd1", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Adgrf1", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Adgrf2", "type": "string", "description": "", "mode": "nullable"},
            {"name": "ADGRF3", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Adgrg1", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Adgrg2", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Adgrg4", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Adgrg5", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Adgrg6", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Adgrg7", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Adgrv1", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Mc2r", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Adra1a", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Adra1b", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Adra1d", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Adra2a", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Adra2b", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Adra2c", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Aplnr", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Ackr1", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Ackr2", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Ackr3", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Ackr4", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Bdkrb1", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Bdkrb2", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Adrb1", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Adrb2", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Adrb3", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Brs3", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Ccr1l1", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Ccr1", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Ccr10", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Ccr2", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Ccr4", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Ccr6", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Ccr7", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Ccr8", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Ccr9", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Ccrl2", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Cxcr1", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Cxcr2", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Cxcr3", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Cxcr4", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Cxcr5", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Cxcr6", "type": "string", "description": "", "mode": "nullable"},
            {"name": "C3ar1", "type": "string", "description": "", "mode": "nullable"},
            {"name": "C5ar1", "type": "string", "description": "", "mode": "nullable"},
            {"name": "C5ar2", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Celsr1", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Celsr2", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Celsr3", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Calcrl", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Calcr", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Cnr1", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Cnr2", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Xcr1", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Cmklr1", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Cckar", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Crhr1", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Crhr2", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Cx3cr1", "type": "string", "description": "", "mode": "nullable"},
            {
                "name": "Cysltr1",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Cysltr2",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {"name": "Drd1", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Drd5", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Drd2", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Drd3", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Drd4", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Oprd1", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Ednrb", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Ednra", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Casr", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Fpr1", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Fshr", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Fpr_s1", "type": "string", "description": "", "mode": "nullable"},
            {
                "name": "Fpr_rs3",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {"name": "Ffar1", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Ffar3", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Ffar4", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Fzd1", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Fzd10", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Fzd2", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Fzd3", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Fzd4", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Fzd5", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Fzd6", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Fzd7", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Fzd8", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Fzd9", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Gpbar1", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Gper1", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Gpr1", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Gpr135", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Gpr143", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Gpr15", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Gpr157", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Gpr161", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Gpr176", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Gpr182", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Gpr183", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Gpr20", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Gpr22", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Gpr26", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Gpr3", "type": "string", "description": "", "mode": "nullable"},
            {
                "name": "Gpr37l1",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {"name": "Gpr39", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Gpr4", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Gpr52", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Gpr55", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Gpr6", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Gpr61", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Gpr62", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Gpr84", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Gpr87", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Gprc5b", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Gprc5c", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Gprc5d", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Gprc6a", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Galr1", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Galr2", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Gabbr1", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Gabbr2", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Gipr", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Grpr", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Cckbr", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Gcgr", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Glp1r", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Glp2r", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Gpr119", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Gnrhr", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Ghsr", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Ghrhr", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Hrh1", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Hrh2", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Hrh3", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Hrh4", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Hcar1", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Hcar2", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Oprk1", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Kiss1r", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Lgr4", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Lgr5", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Lgr6", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Ltb4r", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Ltb4r2", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Lhcgr", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Lpar1", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Lpar2", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Lpar3", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Lpar4", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Lpar6", "type": "string", "description": "", "mode": "nullable"},
            {
                "name": "Mrgpra3",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Mrgprb1",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Mrgprb2",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Mrgprb4",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Mrgprb5",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Mrgprb8",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {"name": "Mrgprd", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Mrgpre", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Mrgprf", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Mrgprg", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Mrgprh", "type": "string", "description": "", "mode": "nullable"},
            {
                "name": "Mrgprx2",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {"name": "Mchr1", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Mc3r", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Mc4r", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Mc5r", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Opn4", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Mtnr1a", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Mtnr1b", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Gpr50", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Grm1", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Grm2", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Grm3", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Grm4", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Grm5", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Grm6", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Grm7", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Oprm1", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Chrm1", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Chrm2", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Chrm3", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Chrm4", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Chrm5", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Gpr18", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Nmbr", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Tacr3", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Nmur2", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Npffr2", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Npsr1", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Npy1r", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Npy2r", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Npy4r", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Npy6r", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Npbwr1", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Ntsr1", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Ntsr2", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Opn3", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Opn5", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Hcrtr1", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Hcrtr2", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Oxtr", "type": "string", "description": "", "mode": "nullable"},
            {"name": "P2ry1", "type": "string", "description": "", "mode": "nullable"},
            {"name": "P2ry12", "type": "string", "description": "", "mode": "nullable"},
            {"name": "P2yr13", "type": "string", "description": "", "mode": "nullable"},
            {"name": "P2ry14", "type": "string", "description": "", "mode": "nullable"},
            {"name": "P2ry2", "type": "string", "description": "", "mode": "nullable"},
            {"name": "P2ry4", "type": "string", "description": "", "mode": "nullable"},
            {"name": "P2ry6", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Pth2r", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Pth1r", "type": "string", "description": "", "mode": "nullable"},
            {
                "name": "Adcyap1r1",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {"name": "Ptafr", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Ccr3", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Gpr101", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Gpr132", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Gpr139", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Gpr141", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Gpr142", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Gpr146", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Gpr149", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Gpr150", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Gpr151", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Gpr152", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Gpr153", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Gpr156", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Gpr158", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Gpr160", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Gpr162", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Gpr171", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Gpr173", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Gpr174", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Gpr21", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Gpr27", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Gpr33", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Gpr34", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Gpr45", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Gpr63", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Gpr75", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Gpr82", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Gpr83", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Gpr85", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Gpr88", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Prokr1", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Prokr2", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Prlhr", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Gpr37", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Ptgir", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Ptgdr", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Ptgdr2", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Ptger1", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Ptger2", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Ptger3", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Ptger4", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Ptgfr", "type": "string", "description": "", "mode": "nullable"},
            {"name": "F2r", "type": "string", "description": "", "mode": "nullable"},
            {"name": "F2rl1", "type": "string", "description": "", "mode": "nullable"},
            {"name": "F2rl2", "type": "string", "description": "", "mode": "nullable"},
            {"name": "F2rl3", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Mas1", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Gpr65", "type": "string", "description": "", "mode": "nullable"},
            {"name": "P2ry10", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Qrfpr", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Rxfp1", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Rxfp2", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Rxfp3", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Rxfp4", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Gprc5a", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Rho", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Sctr", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Opn1sw", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Smo", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Sstr1", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Sstr2", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Sstr3", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Sstr4", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Sstr5", "type": "string", "description": "", "mode": "nullable"},
            {"name": "S1pr1", "type": "string", "description": "", "mode": "nullable"},
            {"name": "S1pr2", "type": "string", "description": "", "mode": "nullable"},
            {"name": "S1pr3", "type": "string", "description": "", "mode": "nullable"},
            {"name": "S1pr4", "type": "string", "description": "", "mode": "nullable"},
            {"name": "S1pr5", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Tacr2", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Tacr1", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Sucnr1", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Tas1r1", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Tas1r2", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Tas1r3", "type": "string", "description": "", "mode": "nullable"},
            {
                "name": "Tas2r135",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {"name": "Tas2r7", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Tbxa2r", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Tshr", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Trhr", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Taar1", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Taar2", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Taar3", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Taar4", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Taar6", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Taar7a", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Taar7b", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Taar7d", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Taar7e", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Taar7f", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Taar8a", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Taar8b", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Taar8c", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Taar9", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Agtr1a", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Agtr1b", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Agtr2", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Gpr17", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Uts2r", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Vipr1", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Vipr2", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Avpr1a", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Avpr1b", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Avpr2", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Rrh", "type": "string", "description": "", "mode": "nullable"},
            {
                "name": "Vmn1r40",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Vmn1r42",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Vmn1r43",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Vmn1r44",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Vmn1r45",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Vmn1r46",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Vmn1r47",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Vmn1r49",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Vmn1r50",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Vmn1r51",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Vmn1r52",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Vmn1r53",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Vmn1r54",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {"name": "V1ra8", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Vmn2r1", "type": "string", "description": "", "mode": "nullable"},
            {
                "name": "Vmn2r26",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {"name": "Gramd3", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Plxnb3", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Tjap1", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Cspg5", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Mki67", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Cenpe", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Man1a", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Sema4d", "type": "string", "description": "", "mode": "nullable"},
            {
                "name": "Tmem108",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {"name": "Traf4", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Gjc3", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Sox8", "type": "string", "description": "", "mode": "nullable"},
            {
                "name": "Aldh1l1",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {"name": "Olig1", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Pcdh15", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Gad1", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Syt4", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Baiap2", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Amigo2", "type": "string", "description": "", "mode": "nullable"},
            {
                "name": "Slc17a6",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Slc17a7",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Slc17a8",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {"name": "Aqp4", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Mlc1", "type": "string", "description": "", "mode": "nullable"},
            {
                "name": "Slc15a3",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {"name": "Selplg", "type": "string", "description": "", "mode": "nullable"},
            {
                "name": "Slco1a4",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {"name": "Fn1", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Cldn5", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Emcn", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Myh11", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Lmod1", "type": "string", "description": "", "mode": "nullable"},
            {
                "name": "Atp13a5",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {"name": "Kcnj8", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Abcc9", "type": "string", "description": "", "mode": "nullable"},
            {
                "name": "Cd300c2",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {"name": "C1qa", "type": "string", "description": "", "mode": "nullable"},
            {"name": "C1qb", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Cxcl12", "type": "string", "description": "", "mode": "nullable"},
            {
                "name": "Arhgap29",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {"name": "Sox9", "type": "string", "description": "", "mode": "nullable"},
            {
                "name": "Slc25a18",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {"name": "Dlk1", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Peg10", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Dgkk", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Chat", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Grin2b", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Cx3cl1", "type": "string", "description": "", "mode": "nullable"},
            {
                "name": "Igfbpl1",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {"name": "Eomes", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Ascl1", "type": "string", "description": "", "mode": "nullable"},
            {
                "name": "Slc32a1",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {"name": "Th", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Top2a", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Acte1", "type": "string", "description": "", "mode": "nullable"},
            {
                "name": "Ppp1r3g",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {"name": "Gfap", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Timp4", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Anxa11", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Cldn19", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Rspo3", "type": "string", "description": "", "mode": "nullable"},
            {
                "name": "Slc47a1",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {"name": "Pln", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Foxd3", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Alk", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Axl", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Csf1r", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Ddr1", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Ddr2", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Efemp1", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Egfr", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Epha1", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Epha2", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Epha3", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Epha4", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Epha5", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Epha6", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Epha7", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Epha8", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Epha10", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Ephb1", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Ephb2", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Ephb3", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Ephb6", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Erbb2", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Erbb3", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Erbb4", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Fgfr2", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Fgfr3", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Fgfr4", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Fgfrl1", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Flt1", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Flt3", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Flt4", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Igf1r", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Insr", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Insrr", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Kdr", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Kit", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Lmtk2", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Mertk", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Met", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Musk", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Nrp1", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Nrp2", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Ntrk1", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Ntrk2", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Ntrk3", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Pdgfra", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Pdgfrb", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Ptk7", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Ret", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Ror1", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Ror2", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Ros1", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Ryk", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Tek", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Tie1", "type": "string", "description": "", "mode": "nullable"},
            {"name": "Tyro3", "type": "string", "description": "", "mode": "nullable"},
            {
                "name": "Blank_0",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_1",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_2",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_3",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_4",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_5",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_6",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_7",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_8",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_9",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_10",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_11",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_12",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_13",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_14",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_15",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_16",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_17",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_18",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_19",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_20",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_21",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_22",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_23",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_24",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_25",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_26",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_27",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_28",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_29",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_30",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_31",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_32",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_33",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_34",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_35",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_36",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_37",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_38",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_39",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_40",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_41",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_42",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_43",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_44",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_45",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_46",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_47",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_48",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_49",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_50",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_51",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_52",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_53",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_54",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_55",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_56",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_57",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_58",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_59",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_60",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_61",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_62",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_63",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_64",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_65",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_66",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_67",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_68",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_69",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_70",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_71",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_72",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_73",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_74",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_75",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_76",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_77",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_78",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_79",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_80",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_81",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_82",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_83",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_84",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_85",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_86",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_87",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_88",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_89",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_90",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_91",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_92",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_93",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_94",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_95",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_96",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_97",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_98",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_99",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_100",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_101",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_102",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_103",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_104",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_105",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_106",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_107",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_108",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_109",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_110",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_111",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_112",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_113",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_114",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_115",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_116",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_117",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_118",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_119",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_120",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_121",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_122",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_123",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_124",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_125",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_126",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_127",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_128",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_129",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_130",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_131",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_132",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_133",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_134",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_135",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_136",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_137",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_138",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_139",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_140",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_141",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_142",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_143",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_144",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_145",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_146",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_147",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_148",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_149",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_150",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_151",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_152",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_153",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_154",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_155",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_156",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_157",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_158",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_159",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_160",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_161",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_162",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_163",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_164",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
            {
                "name": "Blank_165",
                "type": "string",
                "description": "",
                "mode": "nullable",
            },
        ],
    )

    load_cell_by_gene_to_bq
