# Copyright 2022 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


from airflow import DAG
from airflow.providers.cncf.kubernetes.operators import kubernetes_pod
from airflow.providers.google.cloud.operators import kubernetes_engine

default_args = {
    "owner": "Google",
    "depends_on_past": False,
    "start_date": "2022-03-01",
}


with DAG(
    dag_id="nhtsa_traffic_fatalities.nhtsa_traffic_fatalities",
    default_args=default_args,
    max_active_runs=1,
    schedule_interval="@daily",
    catchup=False,
    default_view="graph",
) as dag:
    create_cluster = kubernetes_engine.GKECreateClusterOperator(
        task_id="create_cluster",
        project_id="{{ var.value.gcp_project }}",
        location="us-central1-c",
        body={
            "name": "nhtsa-traffic-fatalities",
            "initial_node_count": 2,
            "network": "{{ var.value.vpc_network }}",
            "node_config": {
                "machine_type": "e2-standard-16",
                "oauth_scopes": [
                    "https://www.googleapis.com/auth/devstorage.read_write",
                    "https://www.googleapis.com/auth/cloud-platform",
                ],
            },
        },
    )

    # Run CSV transform within kubernetes pod for accident pipeline
    accident_2015_transform_csv = kubernetes_pod.KubernetesPodOperator(
        task_id="accident_2015_transform_csv",
        startup_timeout_seconds=600,
        name="accident_2015",
        namespace="composer",
        service_account_name="datasets",
        image_pull_policy="Always",
        image="{{ var.json.nhtsa_traffic_fatalities.container_registry.run_csv_transform_kub }}",
        env_vars={
            "PIPELINE_NAME": "{{ var.json.nhtsa_traffic_fatalities.accident_2015.pipeline_name }}",
            "SOURCE_URL": "{{ var.json.nhtsa_traffic_fatalities.accident_2015.source_url }}",
            "CHUNKSIZE": "{{ var.json.nhtsa_traffic_fatalities.accident_2015.chunksize }}",
            "SOURCE_ZIPFILE_EXTRACTED": "accident.csv",
            "SOURCE_FILE": "{{ var.json.nhtsa_traffic_fatalities.accident_2015.source_file }}",
            "PROJECT_ID": "{{ var.value.gcp_project }}",
            "DATASET_ID": "{{ var.json.nhtsa_traffic_fatalities.accident_2015.dataset_id }}",
            "TABLE_ID": "{{ var.json.nhtsa_traffic_fatalities.accident_2015.destination_table }}",
            "START_YEAR": "{{ var.json.nhtsa_traffic_fatalities.accident_2015.start_year }}",
            "END_YEAR": "{{ var.json.nhtsa_traffic_fatalities.accident_2015.end_year }}",
            "DROP_DEST_TABLE": "{{ var.json.nhtsa_traffic_fatalities.accident_2015.drop_dest_table }}",
            "TARGET_GCS_BUCKET": "{{ var.value.composer_bucket }}",
            "TARGET_GCS_PATH": "{{ var.json.nhtsa_traffic_fatalities.accident_2015.target_gcs_path }}",
            "SCHEMA_PATH": "{{ var.json.nhtsa_traffic_fatalities.accident_2015.schema_path }}",
            "INPUT_CSV_HEADERS": '[\n  "state_number",\n  "state_name",\n  "consecutive_number",\n  "number_of_vehicle_forms_submitted_all",\n  "number_of_motor_vehicles_in_transport_mvit",\n  "number_of_parked_working_vehicles",\n  "number_of_forms_submitted_for_persons_not_in_motor_vehicles",\n  "number_of_forms_submitted_for_persons_in_motor_vehicles",\n  "number_of_persons_in_motor_vehicles_in_transport_mvit",\n  "number_of_persons_not_in_motor_vehicles_in_transport_mvit",\n  "county",\n  "city",\n  "day_of_crash",\n  "day_name",\n  "month_of_crash",\n  "month_of_crash_name",\n  "year_of_crash",\n  "day_of_week",\n  "day_of_week_name",\n  "hour_of_crash",\n  "hour_of_crash_name",\n  "minute_of_crash",\n  "minute_of_crash_name",\n  "national_highway_system",\n  "national_highway_system_name",\n  "route_signing",\n  "route_signing_name",\n  "trafficway_identifier",\n  "trafficway_identifier_2",\n  "land_use",\n  "land_use_name",\n  "functional_system",\n  "functional_system_name",\n  "ownership",\n  "ownership_name",\n  "milepoint",\n  "milepoint_name",\n  "latitude",\n  "latitude_name",\n  "longitude",\n  "longitude_name",\n  "special_jurisdiction",\n  "special_jurisdiction_name",\n  "first_harmful_event",\n  "first_harmful_event_name",\n  "manner_of_collision",\n  "manner_of_collision_name",\n  "relation_to_junction_within_interchange_area",\n  "relation_to_junction_within_interchange_area_name",\n  "relation_to_junction_specific_location",\n  "relation_to_junction_specific_location_name",\n  "type_of_intersection",\n  "type_of_intersection_name",\n  "work_zone",\n  "work_zone_name",\n  "relation_to_trafficway",\n  "relation_to_trafficway_name",\n  "light_condition",\n  "light_condition_name",\n  "atmospheric_conditions_1",\n  "atmospheric_conditions_1_name",\n  "atmospheric_conditions_2",\n  "atmospheric_conditions_2_name",\n  "atmospheric_conditions",\n  "atmospheric_conditions_name",\n  "school_bus_related",\n  "school_bus_related_name",\n  "rail_grade_crossing_identifier",\n  "rail_grade_crossing_identifier_name",\n  "hour_of_notification",\n  "hour_of_notification_name",\n  "minute_of_notification",\n  "minute_of_notification_name",\n  "hour_of_arrival_at_scene",\n  "hour_of_arrival_at_scene_name",\n  "minute_of_arrival_at_scene",\n  "minute_of_arrival_at_scene_name",\n  "hour_of_ems_arrival_at_hospital",\n  "hour_of_ems_arrival_at_hospital_name",\n  "minute_of_ems_arrival_at_hospital",\n  "minute_of_ems_arrival_at_hospital_name",\n  "related_factors_crash_level_1",\n  "related_factors_crash_level_1_name",\n  "related_factors_crash_level_2",\n  "related_factors_crash_level_2_name",\n  "related_factors_crash_level_3",\n  "related_factors_crash_level_3_name",\n  "number_of_fatalities",\n  "number_of_drunk_drivers"\n]',
            "INPUT_DTYPES": '{\n  "state_number": "str",\n  "state_name": "str",\n  "consecutive_number": "str",\n  "number_of_vehicle_forms_submitted_all": "str",\n  "number_of_motor_vehicles_in_transport_mvit": "str",\n  "number_of_parked_working_vehicles": "str",\n  "number_of_forms_submitted_for_persons_not_in_motor_vehicles": "str",\n  "number_of_forms_submitted_for_persons_in_motor_vehicles": "str",\n  "number_of_persons_in_motor_vehicles_in_transport_mvit": "str",\n  "number_of_persons_not_in_motor_vehicles_in_transport_mvit": "str",\n  "county": "str",\n  "city": "str",\n  "day_of_crash": "str",\n  "day_name": "str",\n  "month_of_crash": "str",\n  "month_of_crash_name": "str",\n  "year_of_crash": "str",\n  "day_of_week": "str",\n  "day_of_week_name": "str",\n  "hour_of_crash": "str",\n  "hour_of_crash_name": "str",\n  "minute_of_crash": "str",\n  "minute_of_crash_name": "str",\n  "national_highway_system": "str",\n  "national_highway_system_name": "str",\n  "route_signing": "str",\n  "route_signing_name": "str",\n  "trafficway_identifier": "str",\n  "trafficway_identifier_2": "str",\n  "land_use": "str",\n  "land_use_name": "str",\n  "functional_system": "str",\n  "functional_system_name": "str",\n  "ownership": "str",\n  "ownership_name": "str",\n  "milepoint": "str",\n  "milepoint_name": "str",\n  "latitude": "str",\n  "latitude_name": "str",\n  "longitude": "str",\n  "longitude_name": "str",\n  "special_jurisdiction": "str",\n  "special_jurisdiction_name": "str",\n  "first_harmful_event": "str",\n  "first_harmful_event_name": "str",\n  "manner_of_collision": "str",\n  "manner_of_collision_name": "str",\n  "relation_to_junction_within_interchange_area": "str",\n  "relation_to_junction_within_interchange_area_name": "str",\n  "relation_to_junction_specific_location": "str",\n  "relation_to_junction_specific_location_name": "str",\n  "type_of_intersection": "str",\n  "type_of_intersection_name": "str",\n  "work_zone": "str",\n  "work_zone_name": "str",\n  "relation_to_trafficway": "str",\n  "relation_to_trafficway_name": "str",\n  "light_condition": "str",\n  "light_condition_name": "str",\n  "atmospheric_conditions_1": "str",\n  "atmospheric_conditions_1_name": "str",\n  "atmospheric_conditions_2": "str",\n  "atmospheric_conditions_2_name": "str",\n  "atmospheric_conditions": "str",\n  "atmospheric_conditions_name": "str",\n  "school_bus_related": "str",\n  "school_bus_related_name": "str",\n  "rail_grade_crossing_identifier": "str",\n  "rail_grade_crossing_identifier_name": "str",\n  "hour_of_notification": "str",\n  "hour_of_notification_name": "str",\n  "minute_of_notification": "str",\n  "minute_of_notification_name": "str",\n  "hour_of_arrival_at_scene": "str",\n  "hour_of_arrival_at_scene_name": "str",\n  "minute_of_arrival_at_scene": "str",\n  "minute_of_arrival_at_scene_name": "str",\n  "hour_of_ems_arrival_at_hospital": "str",\n  "hour_of_ems_arrival_at_hospital_name": "str",\n  "minute_of_ems_arrival_at_hospital": "str",\n  "minute_of_ems_arrival_at_hospital_name": "str",\n  "related_factors_crash_level_1": "str",\n  "related_factors_crash_level_1_name": "str",\n  "related_factors_crash_level_2": "str",\n  "related_factors_crash_level_2_name": "str",\n  "related_factors_crash_level_3": "str",\n  "related_factors_crash_level_3_name": "str",\n  "number_of_fatalities": "str",\n  "number_of_drunk_drivers": "str"\n}',
            "RENAME_MAPPINGS_LIST": '{\n  "STATE": "state_number",\n  "STATENAME": "state_name",\n  "ST_CASE": "consecutive_number",\n  "VE_TOTAL": "number_of_vehicle_forms_submitted_all",\n  "VE_FORMS": "number_of_motor_vehicles_in_transport_mvit",\n  "PVH_INVL": "number_of_parked_working_vehicles",\n  "PEDS": "number_of_forms_submitted_for_persons_not_in_motor_vehicles",\n  "PERSONS": "number_of_forms_submitted_for_persons_in_motor_vehicles",\n  "PERMVIT": "number_of_persons_in_motor_vehicles_in_transport_mvit",\n  "PERNOTMVIT": "number_of_persons_not_in_motor_vehicles_in_transport_mvit",\n  "COUNTY": "county",\n  "CITY": "city",\n  "DAY": "day_of_crash",\n  "DAYNAME": "day_name",\n  "MONTH": "month_of_crash",\n  "MONTHNAME": "month_of_crash_name",\n  "YEAR": "year_of_crash",\n  "DAY_WEEK": "day_of_week",\n  "DAY_WEEKNAME": "day_of_week_name",\n  "HOUR": "hour_of_crash",\n  "HOURNAME": "hour_of_crash_name",\n  "MINUTE": "minute_of_crash",\n  "MINUTENAME": "minute_of_crash_name",\n  "NHS": "national_highway_system",\n  "NHSNAME": "national_highway_system_name",\n  "ROUTE": "route_signing",\n  "ROUTENAME": "route_signing_name",\n  "TWAY_ID": "trafficway_identifier",\n  "TWAY_ID2": "trafficway_identifier_2",\n  "RUR_URB": "land_use",\n  "RUR_URBNAME": "land_use_name",\n  "FUNC_SYS": "functional_system",\n  "FUNC_SYSNAME": "functional_system_name",\n  "RD_OWNER": "ownership",\n  "RD_OWNERNAME": "ownership_name",\n  "MILEPT": "milepoint",\n  "MILEPTNAME": "milepoint_name",\n  "LATITUDE": "latitude",\n  "LATITUDENAME": "latitude_name",\n  "LONGITUD": "longitude",\n  "LONGITUDNAME": "longitude_name",\n  "SP_JUR": "special_jurisdiction",\n  "SP_JURNAME": "special_jurisdiction_name",\n  "HARM_EV": "first_harmful_event",\n  "HARM_EVNAME": "first_harmful_event_name",\n  "MAN_COLL": "manner_of_collision",\n  "MAN_COLLNAME": "manner_of_collision_name",\n  "RELJCT1": "relation_to_junction_within_interchange_area",\n  "RELJCT1NAME": "relation_to_junction_within_interchange_area_name",\n  "RELJCT2": "relation_to_junction_specific_location",\n  "RELJCT2NAME": "relation_to_junction_specific_location_name",\n  "TYP_INT" : "type_of_intersection",\n  "TYP_INTNAME": "type_of_intersection_name",\n  "WRK_ZONE": "work_zone",\n  "WRK_ZONENAME": "work_zone_name",\n  "REL_ROAD": "relation_to_trafficway",\n  "REL_ROADNAME": "relation_to_trafficway_name",\n  "LGT_COND": "light_condition",\n  "LGT_CONDNAME": "light_condition_name",\n  "WEATHER1": "atmospheric_conditions_1",\n  "WEATHER1NAME": "atmospheric_conditions_1_name",\n  "WEATHER2": "atmospheric_conditions_2",\n  "WEATHER2NAME": "atmospheric_conditions_2_name",\n  "WEATHER": "atmospheric_conditions",\n  "WEATHERNAME": "atmospheric_conditions_name",\n  "SCH_BUS": "school_bus_related",\n  "SCH_BUSNAME": "school_bus_related_name",\n  "RAIL": "rail_grade_crossing_identifier",\n  "RAILNAME": "rail_grade_crossing_identifier_name",\n  "NOT_HOUR": "hour_of_notification",\n  "NOT_HOURNAME": "hour_of_notification_name",\n  "NOT_MIN": "minute_of_notification",\n  "NOT_MINNAME": "minute_of_notification_name",\n  "ARR_HOUR": "hour_of_arrival_at_scene",\n  "ARR_HOURNAME": "hour_of_arrival_at_scene_name",\n  "ARR_MIN": "minute_of_arrival_at_scene",\n  "ARR_MINNAME": "minute_of_arrival_at_scene_name",\n  "HOSP_HR": "hour_of_ems_arrival_at_hospital",\n  "HOSP_HRNAME": "hour_of_ems_arrival_at_hospital_name",\n  "HOSP_MN": "minute_of_ems_arrival_at_hospital",\n  "HOSP_MNNAME": "minute_of_ems_arrival_at_hospital_name",\n  "CF1": "related_factors_crash_level_1",\n  "CF1NAME": "related_factors_crash_level_1_name",\n  "CF2": "related_factors_crash_level_2",\n  "CF2NAME": "related_factors_crash_level_2_name",\n  "CF3": "related_factors_crash_level_3",\n  "CF3NAME": "related_factors_crash_level_3_name",\n  "FATALS": "number_of_fatalities",\n  "DRUNK_DR": "number_of_drunk_drivers"\n}',
        },
        resources={
            "request_ephemeral_storage": "4G",
            "request_cpu": "1",
            "request_memory": "4G",
        },
    )

    # Run CSV transform within kubernetes pod for accident pipeline
    accident_2016_2019_transform_csv = kubernetes_pod.KubernetesPodOperator(
        task_id="accident_2016_2019_transform_csv",
        startup_timeout_seconds=600,
        name="accident_2016_2019",
        namespace="composer",
        service_account_name="datasets",
        image_pull_policy="Always",
        image="{{ var.json.nhtsa_traffic_fatalities.container_registry.run_csv_transform_kub }}",
        env_vars={
            "PIPELINE_NAME": "{{ var.json.nhtsa_traffic_fatalities.accident_2016_2019.pipeline_name }}",
            "SOURCE_URL": "{{ var.json.nhtsa_traffic_fatalities.accident_2016_2019.source_url }}",
            "CHUNKSIZE": "{{ var.json.nhtsa_traffic_fatalities.accident_2016_2019.chunksize }}",
            "SOURCE_ZIPFILE_EXTRACTED": "accident.csv",
            "SOURCE_FILE": "{{ var.json.nhtsa_traffic_fatalities.accident_2016_2019.source_file }}",
            "PROJECT_ID": "{{ var.value.gcp_project }}",
            "DATASET_ID": "{{ var.json.nhtsa_traffic_fatalities.accident_2016_2019.dataset_id }}",
            "TABLE_ID": "{{ var.json.nhtsa_traffic_fatalities.accident_2016_2019.destination_table }}",
            "START_YEAR": "{{ var.json.nhtsa_traffic_fatalities.accident_2016_2019.start_year }}",
            "END_YEAR": "{{ var.json.nhtsa_traffic_fatalities.accident_2016_2019.end_year }}",
            "DROP_DEST_TABLE": "{{ var.json.nhtsa_traffic_fatalities.accident_2016_2019.drop_dest_table }}",
            "TARGET_GCS_BUCKET": "{{ var.value.composer_bucket }}",
            "TARGET_GCS_PATH": "{{ var.json.nhtsa_traffic_fatalities.accident_2016_2019.target_gcs_path }}",
            "SCHEMA_PATH": "{{ var.json.nhtsa_traffic_fatalities.accident_2016_2019.schema_path }}",
            "INPUT_CSV_HEADERS": '[\n  "state_number",\n  "state_name",\n  "consecutive_number",\n  "number_of_vehicle_forms_submitted_all",\n  "number_of_motor_vehicles_in_transport_mvit",\n  "number_of_parked_working_vehicles",\n  "number_of_forms_submitted_for_persons_not_in_motor_vehicles",\n  "number_of_forms_submitted_for_persons_in_motor_vehicles",\n  "number_of_persons_in_motor_vehicles_in_transport_mvit",\n  "number_of_persons_not_in_motor_vehicles_in_transport_mvit",\n  "county",\n  "county_name",\n  "city",\n  "city_name",\n  "day_of_crash",\n  "day_name",\n  "month_of_crash",\n  "month_of_crash_name",\n  "year_of_crash",\n  "day_of_week",\n  "day_of_week_name",\n  "hour_of_crash",\n  "hour_of_crash_name",\n  "minute_of_crash",\n  "minute_of_crash_name",\n  "national_highway_system",\n  "national_highway_system_name",\n  "route_signing",\n  "route_signing_name",\n  "trafficway_identifier",\n  "trafficway_identifier_2",\n  "land_use",\n  "land_use_name",\n  "functional_system",\n  "functional_system_name",\n  "ownership",\n  "ownership_name",\n  "milepoint",\n  "milepoint_name",\n  "latitude",\n  "latitude_name",\n  "longitude",\n  "longitude_name",\n  "special_jurisdiction",\n  "special_jurisdiction_name",\n  "first_harmful_event",\n  "first_harmful_event_name",\n  "manner_of_collision",\n  "manner_of_collision_name",\n  "relation_to_junction_within_interchange_area",\n  "relation_to_junction_within_interchange_area_name",\n  "relation_to_junction_specific_location",\n  "relation_to_junction_specific_location_name",\n  "type_of_intersection",\n  "type_of_intersection_name",\n  "work_zone",\n  "work_zone_name",\n  "relation_to_trafficway",\n  "relation_to_trafficway_name",\n  "light_condition",\n  "light_condition_name",\n  "atmospheric_conditions_1",\n  "atmospheric_conditions_1_name",\n  "atmospheric_conditions_2",\n  "atmospheric_conditions_2_name",\n  "atmospheric_conditions",\n  "atmospheric_conditions_name",\n  "school_bus_related",\n  "school_bus_related_name",\n  "rail_grade_crossing_identifier",\n  "rail_grade_crossing_identifier_name",\n  "hour_of_notification",\n  "hour_of_notification_name",\n  "minute_of_notification",\n  "minute_of_notification_name",\n  "hour_of_arrival_at_scene",\n  "hour_of_arrival_at_scene_name",\n  "minute_of_arrival_at_scene",\n  "minute_of_arrival_at_scene_name",\n  "hour_of_ems_arrival_at_hospital",\n  "hour_of_ems_arrival_at_hospital_name",\n  "minute_of_ems_arrival_at_hospital",\n  "minute_of_ems_arrival_at_hospital_name",\n  "related_factors_crash_level_1",\n  "related_factors_crash_level_1_name",\n  "related_factors_crash_level_2",\n  "related_factors_crash_level_2_name",\n  "related_factors_crash_level_3",\n  "related_factors_crash_level_3_name",\n  "number_of_fatalities",\n  "number_of_drunk_drivers"\n]',
            "INPUT_DTYPES": '{\n  "state_number": "str",\n  "state_name": "str",\n  "consecutive_number": "str",\n  "number_of_vehicle_forms_submitted_all": "str",\n  "number_of_motor_vehicles_in_transport_mvit": "str",\n  "number_of_parked_working_vehicles": "str",\n  "number_of_forms_submitted_for_persons_not_in_motor_vehicles": "str",\n  "number_of_forms_submitted_for_persons_in_motor_vehicles": "str",\n  "number_of_persons_in_motor_vehicles_in_transport_mvit": "str",\n  "number_of_persons_not_in_motor_vehicles_in_transport_mvit": "str",\n  "county": "str",\n  "county_name": "str",\n  "city": "str",\n  "city_name": "str",\n  "day_of_crash": "str",\n  "day_name": "str",\n  "month_of_crash": "str",\n  "month_of_crash_name": "str",\n  "year_of_crash": "str",\n  "day_of_week": "str",\n  "day_of_week_name": "str",\n  "hour_of_crash": "str",\n  "hour_of_crash_name": "str",\n  "minute_of_crash": "str",\n  "minute_of_crash_name": "str",\n  "national_highway_system": "str",\n  "national_highway_system_name": "str",\n  "route_signing": "str",\n  "route_signing_name": "str",\n  "trafficway_identifier": "str",\n  "trafficway_identifier_2": "str",\n  "land_use": "str",\n  "land_use_name": "str",\n  "functional_system": "str",\n  "functional_system_name": "str",\n  "ownership": "str",\n  "ownership_name": "str",\n  "milepoint": "str",\n  "milepoint_name": "str",\n  "latitude": "str",\n  "latitude_name": "str",\n  "longitude": "str",\n  "longitude_name": "str",\n  "special_jurisdiction": "str",\n  "special_jurisdiction_name": "str",\n  "first_harmful_event": "str",\n  "first_harmful_event_name": "str",\n  "manner_of_collision": "str",\n  "manner_of_collision_name": "str",\n  "relation_to_junction_within_interchange_area": "str",\n  "relation_to_junction_within_interchange_area_name": "str",\n  "relation_to_junction_specific_location": "str",\n  "relation_to_junction_specific_location_name": "str",\n  "type_of_intersection": "str",\n  "type_of_intersection_name": "str",\n  "work_zone": "str",\n  "work_zone_name": "str",\n  "relation_to_trafficway": "str",\n  "relation_to_trafficway_name": "str",\n  "light_condition": "str",\n  "light_condition_name": "str",\n  "atmospheric_conditions_1": "str",\n  "atmospheric_conditions_1_name": "str",\n  "atmospheric_conditions_2": "str",\n  "atmospheric_conditions_2_name": "str",\n  "atmospheric_conditions": "str",\n  "atmospheric_conditions_name": "str",\n  "school_bus_related": "str",\n  "school_bus_related_name": "str",\n  "rail_grade_crossing_identifier": "str",\n  "rail_grade_crossing_identifier_name": "str",\n  "hour_of_notification": "str",\n  "hour_of_notification_name": "str",\n  "minute_of_notification": "str",\n  "minute_of_notification_name": "str",\n  "hour_of_arrival_at_scene": "str",\n  "hour_of_arrival_at_scene_name": "str",\n  "minute_of_arrival_at_scene": "str",\n  "minute_of_arrival_at_scene_name": "str",\n  "hour_of_ems_arrival_at_hospital": "str",\n  "hour_of_ems_arrival_at_hospital_name": "str",\n  "minute_of_ems_arrival_at_hospital": "str",\n  "minute_of_ems_arrival_at_hospital_name": "str",\n  "related_factors_crash_level_1": "str",\n  "related_factors_crash_level_1_name": "str",\n  "related_factors_crash_level_2": "str",\n  "related_factors_crash_level_2_name": "str",\n  "related_factors_crash_level_3": "str",\n  "related_factors_crash_level_3_name": "str",\n  "number_of_fatalities": "str",\n  "number_of_drunk_drivers": "str"\n}',
            "RENAME_MAPPINGS_LIST": '{\n  "STATE": "state_number",\n  "STATENAME": "state_name",\n  "ST_CASE": "consecutive_number",\n  "VE_TOTAL": "number_of_vehicle_forms_submitted_all",\n  "VE_FORMS": "number_of_motor_vehicles_in_transport_mvit",\n  "PVH_INVL": "number_of_parked_working_vehicles",\n  "PEDS": "number_of_forms_submitted_for_persons_not_in_motor_vehicles",\n  "PERSONS": "number_of_forms_submitted_for_persons_in_motor_vehicles",\n  "PERMVIT": "number_of_persons_in_motor_vehicles_in_transport_mvit",\n  "PERNOTMVIT": "number_of_persons_not_in_motor_vehicles_in_transport_mvit",\n  "COUNTY": "county",\n  "COUNTYNAME": "county_name",\n  "CITY": "city",\n  "CITYNAME": "city_name",\n  "DAY": "day_of_crash",\n  "DAYNAME": "day_name",\n  "MONTH": "month_of_crash",\n  "MONTHNAME": "month_of_crash_name",\n  "YEAR": "year_of_crash",\n  "DAY_WEEK": "day_of_week",\n  "DAY_WEEKNAME": "day_of_week_name",\n  "HOUR": "hour_of_crash",\n  "HOURNAME": "hour_of_crash_name",\n  "MINUTE": "minute_of_crash",\n  "MINUTENAME": "minute_of_crash_name",\n  "NHS": "national_highway_system",\n  "NHSNAME": "national_highway_system_name",\n  "ROUTE": "route_signing",\n  "ROUTENAME": "route_signing_name",\n  "TWAY_ID": "trafficway_identifier",\n  "TWAY_ID2": "trafficway_identifier_2",\n  "RUR_URB": "land_use",\n  "RUR_URBNAME": "land_use_name",\n  "FUNC_SYS": "functional_system",\n  "FUNC_SYSNAME": "functional_system_name",\n  "RD_OWNER": "ownership",\n  "RD_OWNERNAME": "ownership_name",\n  "MILEPT": "milepoint",\n  "MILEPTNAME": "milepoint_name",\n  "LATITUDE": "latitude",\n  "LATITUDENAME": "latitude_name",\n  "LONGITUD": "longitude",\n  "LONGITUDNAME": "longitude_name",\n  "SP_JUR": "special_jurisdiction",\n  "SP_JURNAME": "special_jurisdiction_name",\n  "HARM_EV": "first_harmful_event",\n  "HARM_EVNAME": "first_harmful_event_name",\n  "MAN_COLL": "manner_of_collision",\n  "MAN_COLLNAME": "manner_of_collision_name",\n  "RELJCT1": "relation_to_junction_within_interchange_area",\n  "RELJCT1NAME": "relation_to_junction_within_interchange_area_name",\n  "RELJCT2": "relation_to_junction_specific_location",\n  "RELJCT2NAME": "relation_to_junction_specific_location_name",\n  "TYP_INT" : "type_of_intersection",\n  "TYP_INTNAME": "type_of_intersection_name",\n  "WRK_ZONE": "work_zone",\n  "WRK_ZONENAME": "work_zone_name",\n  "REL_ROAD": "relation_to_trafficway",\n  "REL_ROADNAME": "relation_to_trafficway_name",\n  "LGT_COND": "light_condition",\n  "LGT_CONDNAME": "light_condition_name",\n  "WEATHER1": "atmospheric_conditions_1",\n  "WEATHER1NAME": "atmospheric_conditions_1_name",\n  "WEATHER2": "atmospheric_conditions_2",\n  "WEATHER2NAME": "atmospheric_conditions_2_name",\n  "WEATHER": "atmospheric_conditions",\n  "WEATHERNAME": "atmospheric_conditions_name",\n  "SCH_BUS": "school_bus_related",\n  "SCH_BUSNAME": "school_bus_related_name",\n  "RAIL": "rail_grade_crossing_identifier",\n  "RAILNAME": "rail_grade_crossing_identifier_name",\n  "NOT_HOUR": "hour_of_notification",\n  "NOT_HOURNAME": "hour_of_notification_name",\n  "NOT_MIN": "minute_of_notification",\n  "NOT_MINNAME": "minute_of_notification_name",\n  "ARR_HOUR": "hour_of_arrival_at_scene",\n  "ARR_HOURNAME": "hour_of_arrival_at_scene_name",\n  "ARR_MIN": "minute_of_arrival_at_scene",\n  "ARR_MINNAME": "minute_of_arrival_at_scene_name",\n  "HOSP_HR": "hour_of_ems_arrival_at_hospital",\n  "HOSP_HRNAME": "hour_of_ems_arrival_at_hospital_name",\n  "HOSP_MN": "minute_of_ems_arrival_at_hospital",\n  "HOSP_MNNAME": "minute_of_ems_arrival_at_hospital_name",\n  "CF1": "related_factors_crash_level_1",\n  "CF1NAME": "related_factors_crash_level_1_name",\n  "CF2": "related_factors_crash_level_2",\n  "CF2NAME": "related_factors_crash_level_2_name",\n  "CF3": "related_factors_crash_level_3",\n  "CF3NAME": "related_factors_crash_level_3_name",\n  "FATALS": "number_of_fatalities",\n  "DRUNK_DR": "number_of_drunk_drivers"\n}',
        },
        resources={
            "request_ephemeral_storage": "4G",
            "request_cpu": "1",
            "request_memory": "4G",
        },
    )

    # Run CSV transform within kubernetes pod for accident pipeline
    accident_2020_transform_csv = kubernetes_pod.KubernetesPodOperator(
        task_id="accident_2020_transform_csv",
        startup_timeout_seconds=600,
        name="accident_2020",
        namespace="composer",
        service_account_name="datasets",
        image_pull_policy="Always",
        image="{{ var.json.nhtsa_traffic_fatalities.container_registry.run_csv_transform_kub }}",
        env_vars={
            "PIPELINE_NAME": "{{ var.json.nhtsa_traffic_fatalities.accident_2020.pipeline_name }}",
            "SOURCE_URL": "{{ var.json.nhtsa_traffic_fatalities.accident_2020.source_url }}",
            "CHUNKSIZE": "{{ var.json.nhtsa_traffic_fatalities.accident_2020.chunksize }}",
            "SOURCE_ZIPFILE_EXTRACTED": "accident.csv",
            "SOURCE_FILE": "{{ var.json.nhtsa_traffic_fatalities.accident_2020.source_file }}",
            "PROJECT_ID": "{{ var.value.gcp_project }}",
            "DATASET_ID": "{{ var.json.nhtsa_traffic_fatalities.accident_2020.dataset_id }}",
            "TABLE_ID": "{{ var.json.nhtsa_traffic_fatalities.accident_2020.destination_table }}",
            "START_YEAR": "{{ var.json.nhtsa_traffic_fatalities.accident_2020.start_year }}",
            "END_YEAR": "{{ var.json.nhtsa_traffic_fatalities.accident_2020.end_year }}",
            "DROP_DEST_TABLE": "{{ var.json.nhtsa_traffic_fatalities.accident_2020.drop_dest_table }}",
            "TARGET_GCS_BUCKET": "{{ var.value.composer_bucket }}",
            "TARGET_GCS_PATH": "{{ var.json.nhtsa_traffic_fatalities.accident_2020.target_gcs_path }}",
            "SCHEMA_PATH": "{{ var.json.nhtsa_traffic_fatalities.accident_2020.schema_path }}",
            "INPUT_CSV_HEADERS": '[\n  "state_number",\n  "state_name",\n  "consecutive_number",\n  "number_of_vehicle_forms_submitted_all",\n  "number_of_motor_vehicles_in_transport_mvit",\n  "number_of_parked_working_vehicles",\n  "number_of_forms_submitted_for_persons_not_in_motor_vehicles",\n  "number_of_forms_submitted_for_persons_in_motor_vehicles",\n  "number_of_persons_in_motor_vehicles_in_transport_mvit",\n  "number_of_persons_not_in_motor_vehicles_in_transport_mvit",\n  "county",\n  "county_name",\n  "city",\n  "city_name",\n  "day_of_crash",\n  "day_name",\n  "month_of_crash",\n  "month_of_crash_name",\n  "year_of_crash",\n  "day_of_week",\n  "day_of_week_name",\n  "hour_of_crash",\n  "hour_of_crash_name",\n  "minute_of_crash",\n  "minute_of_crash_name",\n  "national_highway_system",\n  "national_highway_system_name",\n  "route_signing",\n  "route_signing_name",\n  "trafficway_identifier",\n  "trafficway_identifier_2",\n  "land_use",\n  "land_use_name",\n  "functional_system",\n  "functional_system_name",\n  "ownership",\n  "ownership_name",\n  "milepoint",\n  "milepoint_name",\n  "latitude",\n  "latitude_name",\n  "longitude",\n  "longitude_name",\n  "special_jurisdiction",\n  "special_jurisdiction_name",\n  "first_harmful_event",\n  "first_harmful_event_name",\n  "manner_of_collision",\n  "manner_of_collision_name",\n  "relation_to_junction_within_interchange_area",\n  "relation_to_junction_within_interchange_area_name",\n  "relation_to_junction_specific_location",\n  "relation_to_junction_specific_location_name",\n  "type_of_intersection",\n  "type_of_intersection_name",\n  "work_zone",\n  "work_zone_name",\n  "relation_to_trafficway",\n  "relation_to_trafficway_name",\n  "light_condition",\n  "light_condition_name",\n  "atmospheric_conditions_1",\n  "atmospheric_conditions_1_name",\n  "atmospheric_conditions_2",\n  "atmospheric_conditions_2_name",\n  "rail_grade_crossing_identifier",\n  "rail_grade_crossing_identifier_name",\n  "hour_of_notification",\n  "hour_of_notification_name",\n  "minute_of_notification",\n  "minute_of_notification_name",\n  "hour_of_arrival_at_scene",\n  "hour_of_arrival_at_scene_name",\n  "minute_of_arrival_at_scene",\n  "minute_of_arrival_at_scene_name",\n  "hour_of_ems_arrival_at_hospital",\n  "hour_of_ems_arrival_at_hospital_name",\n  "minute_of_ems_arrival_at_hospital",\n  "minute_of_ems_arrival_at_hospital_name",\n  "number_of_fatalities",\n  "number_of_drunk_drivers"\n]',
            "INPUT_DTYPES": '{\n  "state_number": "str",\n  "state_name": "str",\n  "consecutive_number": "str",\n  "number_of_vehicle_forms_submitted_all": "str",\n  "number_of_motor_vehicles_in_transport_mvit": "str",\n  "number_of_parked_working_vehicles": "str",\n  "number_of_forms_submitted_for_persons_not_in_motor_vehicles": "str",\n  "number_of_forms_submitted_for_persons_in_motor_vehicles": "str",\n  "number_of_persons_in_motor_vehicles_in_transport_mvit": "str",\n  "number_of_persons_not_in_motor_vehicles_in_transport_mvit": "str",\n  "county": "str",\n  "county_name": "str",\n  "city": "str",\n  "city_name": "str",\n  "day_of_crash": "str",\n  "day_name": "str",\n  "month_of_crash": "str",\n  "month_of_crash_name": "str",\n  "year_of_crash": "str",\n  "day_of_week": "str",\n  "day_of_week_name": "str",\n  "hour_of_crash": "str",\n  "hour_of_crash_name": "str",\n  "minute_of_crash": "str",\n  "minute_of_crash_name": "str",\n  "national_highway_system": "str",\n  "national_highway_system_name": "str",\n  "route_signing": "str",\n  "route_signing_name": "str",\n  "trafficway_identifier": "str",\n  "trafficway_identifier_2": "str",\n  "land_use": "str",\n  "land_use_name": "str",\n  "functional_system": "str",\n  "functional_system_name": "str",\n  "ownership": "str",\n  "ownership_name": "str",\n  "milepoint": "str",\n  "milepoint_name": "str",\n  "latitude": "str",\n  "latitude_name": "str",\n  "longitude": "str",\n  "longitude_name": "str",\n  "special_jurisdiction": "str",\n  "special_jurisdiction_name": "str",\n  "first_harmful_event": "str",\n  "first_harmful_event_name": "str",\n  "manner_of_collision": "str",\n  "manner_of_collision_name": "str",\n  "relation_to_junction_within_interchange_area": "str",\n  "relation_to_junction_within_interchange_area_name": "str",\n  "relation_to_junction_specific_location": "str",\n  "relation_to_junction_specific_location_name": "str",\n  "type_of_intersection": "str",\n  "type_of_intersection_name": "str",\n  "work_zone": "str",\n  "work_zone_name": "str",\n  "relation_to_trafficway": "str",\n  "relation_to_trafficway_name": "str",\n  "light_condition": "str",\n  "light_condition_name": "str",\n  "atmospheric_conditions_1": "str",\n  "atmospheric_conditions_1_name": "str",\n  "atmospheric_conditions_2": "str",\n  "atmospheric_conditions_2_name": "str",\n  "rail_grade_crossing_identifier": "str",\n  "rail_grade_crossing_identifier_name": "str",\n  "hour_of_notification": "str",\n  "hour_of_notification_name": "str",\n  "minute_of_notification": "str",\n  "minute_of_notification_name": "str",\n  "hour_of_arrival_at_scene": "str",\n  "hour_of_arrival_at_scene_name": "str",\n  "minute_of_arrival_at_scene": "str",\n  "minute_of_arrival_at_scene_name": "str",\n  "hour_of_ems_arrival_at_hospital": "str",\n  "hour_of_ems_arrival_at_hospital_name": "str",\n  "minute_of_ems_arrival_at_hospital": "str",\n  "minute_of_ems_arrival_at_hospital_name": "str",\n  "number_of_fatalities": "str",\n  "number_of_drunk_drivers": "str"\n}',
            "RENAME_MAPPINGS_LIST": '{\n  "STATE": "state_number",\n  "STATENAME": "state_name",\n  "ST_CASE": "consecutive_number",\n  "VE_TOTAL": "number_of_vehicle_forms_submitted_all",\n  "VE_FORMS": "number_of_motor_vehicles_in_transport_mvit",\n  "PVH_INVL": "number_of_parked_working_vehicles",\n  "PEDS": "number_of_forms_submitted_for_persons_not_in_motor_vehicles",\n  "PERSONS": "number_of_forms_submitted_for_persons_in_motor_vehicles",\n  "PERMVIT": "number_of_persons_in_motor_vehicles_in_transport_mvit",\n  "PERNOTMVIT": "number_of_persons_not_in_motor_vehicles_in_transport_mvit",\n  "COUNTY": "county",\n  "COUNTYNAME": "county_name",\n  "CITY": "city",\n  "CITYNAME": "city_name",\n  "DAY": "day_of_crash",\n  "DAYNAME": "day_name",\n  "MONTH": "month_of_crash",\n  "MONTHNAME": "month_of_crash_name",\n  "YEAR": "year_of_crash",\n  "DAY_WEEK": "day_of_week",\n  "DAY_WEEKNAME": "day_of_week_name",\n  "HOUR": "hour_of_crash",\n  "HOURNAME": "hour_of_crash_name",\n  "MINUTE": "minute_of_crash",\n  "MINUTENAME": "minute_of_crash_name",\n  "NHS": "national_highway_system",\n  "NHSNAME": "national_highway_system_name",\n  "ROUTE": "route_signing",\n  "ROUTENAME": "route_signing_name",\n  "TWAY_ID": "trafficway_identifier",\n  "TWAY_ID2": "trafficway_identifier_2",\n  "RUR_URB": "land_use",\n  "RUR_URBNAME": "land_use_name",\n  "FUNC_SYS": "functional_system",\n  "FUNC_SYSNAME": "functional_system_name",\n  "RD_OWNER": "ownership",\n  "RD_OWNERNAME": "ownership_name",\n  "MILEPT": "milepoint",\n  "MILEPTNAME": "milepoint_name",\n  "LATITUDE": "latitude",\n  "LATITUDENAME": "latitude_name",\n  "LONGITUD": "longitude",\n  "LONGITUDNAME": "longitude_name",\n  "SP_JUR": "special_jurisdiction",\n  "SP_JURNAME": "special_jurisdiction_name",\n  "HARM_EV": "first_harmful_event",\n  "HARM_EVNAME": "first_harmful_event_name",\n  "MAN_COLL": "manner_of_collision",\n  "MAN_COLLNAME": "manner_of_collision_name",\n  "RELJCT1": "relation_to_junction_within_interchange_area",\n  "RELJCT1NAME": "relation_to_junction_within_interchange_area_name",\n  "RELJCT2": "relation_to_junction_specific_location",\n  "RELJCT2NAME": "relation_to_junction_specific_location_name",\n  "TYP_INT" : "type_of_intersection",\n  "TYP_INTNAME": "type_of_intersection_name",\n  "WRK_ZONE": "work_zone",\n  "WRK_ZONENAME": "work_zone_name",\n  "REL_ROAD": "relation_to_trafficway",\n  "REL_ROADNAME": "relation_to_trafficway_name",\n  "LGT_COND": "light_condition",\n  "LGT_CONDNAME": "light_condition_name",\n  "WEATHER1": "atmospheric_conditions_1",\n  "WEATHER1NAME": "atmospheric_conditions_1_name",\n  "WEATHER2": "atmospheric_conditions_2",\n  "WEATHER2NAME": "atmospheric_conditions_2_name",\n  "RAIL": "rail_grade_crossing_identifier",\n  "RAILNAME": "rail_grade_crossing_identifier_name",\n  "NOT_HOUR": "hour_of_notification",\n  "NOT_HOURNAME": "hour_of_notification_name",\n  "NOT_MIN": "minute_of_notification",\n  "NOT_MINNAME": "minute_of_notification_name",\n  "ARR_HOUR": "hour_of_arrival_at_scene",\n  "ARR_HOURNAME": "hour_of_arrival_at_scene_name",\n  "ARR_MIN": "minute_of_arrival_at_scene",\n  "ARR_MINNAME": "minute_of_arrival_at_scene_name",\n  "HOSP_HR": "hour_of_ems_arrival_at_hospital",\n  "HOSP_HRNAME": "hour_of_ems_arrival_at_hospital_name",\n  "HOSP_MN": "minute_of_ems_arrival_at_hospital",\n  "HOSP_MNNAME": "minute_of_ems_arrival_at_hospital_name",\n  "FATALS": "number_of_fatalities",\n  "DRUNK_DR": "number_of_drunk_drivers"\n}',
        },
        resources={
            "request_ephemeral_storage": "4G",
            "request_cpu": "1",
            "request_memory": "4G",
        },
    )

    # Run CSV transform within kubernetes pod for cevent pipeline
    cevent_2015_2020_transform_csv = kubernetes_pod.KubernetesPodOperator(
        task_id="cevent_2015_2020_transform_csv",
        startup_timeout_seconds=600,
        name="cevent_2015_2020",
        namespace="composer",
        service_account_name="datasets",
        image_pull_policy="Always",
        image="{{ var.json.nhtsa_traffic_fatalities.container_registry.run_csv_transform_kub }}",
        env_vars={
            "PIPELINE_NAME": "{{ var.json.nhtsa_traffic_fatalities.cevent_2015_2020.pipeline_name }}",
            "SOURCE_URL": "{{ var.json.nhtsa_traffic_fatalities.cevent_2015_2020.source_url }}",
            "CHUNKSIZE": "{{ var.json.nhtsa_traffic_fatalities.cevent_2015_2020.chunksize }}",
            "SOURCE_ZIPFILE_EXTRACTED": "cevent.csv",
            "SOURCE_FILE": "{{ var.json.nhtsa_traffic_fatalities.cevent_2015_2020.source_file }}",
            "PROJECT_ID": "{{ var.value.gcp_project }}",
            "DATASET_ID": "{{ var.json.nhtsa_traffic_fatalities.cevent_2015_2020.dataset_id }}",
            "TABLE_ID": "{{ var.json.nhtsa_traffic_fatalities.cevent_2015_2020.destination_table }}",
            "START_YEAR": "{{ var.json.nhtsa_traffic_fatalities.cevent_2015_2020.start_year }}",
            "END_YEAR": "{{ var.json.nhtsa_traffic_fatalities.cevent_2015_2020.end_year }}",
            "DROP_DEST_TABLE": "{{ var.json.nhtsa_traffic_fatalities.cevent_2015_2020.drop_dest_table }}",
            "TARGET_GCS_BUCKET": "{{ var.value.composer_bucket }}",
            "TARGET_GCS_PATH": "{{ var.json.nhtsa_traffic_fatalities.cevent_2015_2020.target_gcs_path }}",
            "SCHEMA_PATH": "{{ var.json.nhtsa_traffic_fatalities.cevent_2015_2020.schema_path }}",
            "INPUT_CSV_HEADERS": '[\n  "state_number",\n  "state_name",\n  "consecutive_number",\n  "event_number",\n  "vehicle_number_this_vehicle",\n  "area_of_impact_this_vehicle",\n  "area_of_impact_this_vehicle_name",\n  "sequence_of_events",\n  "sequence_of_events_name",\n  "vehicle_number_other_vehicle",\n  "vehicle_number_other_vehicle_name",\n  "area_of_impact_other_vehicle",\n  "area_of_impact_other_vehicle_name"\n]',
            "INPUT_DTYPES": '{\n  "state_number": "str",\n  "state_name": "str",\n  "consecutive_number": "str",\n  "event_number": "str",\n  "vehicle_number_this_vehicle": "str",\n  "area_of_impact_this_vehicle": "str",\n  "area_of_impact_this_vehicle_name": "str",\n  "sequence_of_events": "str",\n  "sequence_of_events_name": "str",\n  "vehicle_number_other_vehicle": "str",\n  "vehicle_number_other_vehicle_name": "str",\n  "area_of_impact_other_vehicle": "str",\n  "area_of_impact_other_vehicle_name": "str"\n}',
            "RENAME_MAPPINGS_LIST": '{\n  "STATE": "state_number",\n  "STATENAME": "state_name",\n  "ST_CASE": "consecutive_number",\n  "EVENTNUM": "event_number",\n  "VNUMBER1": "vehicle_number_this_vehicle",\n  "AOI1": "area_of_impact_this_vehicle",\n  "AOI1NAME": "area_of_impact_this_vehicle_name",\n  "SOE": "sequence_of_events",\n  "SOENAME": "sequence_of_events_name",\n  "VNUMBER2": "vehicle_number_other_vehicle",\n  "VNUMBER2NAME": "vehicle_number_other_vehicle_name",\n  "AOI2": "area_of_impact_other_vehicle",\n  "AOI2NAME": "area_of_impact_other_vehicle_name"\n}',
        },
        resources={
            "request_ephemeral_storage": "4G",
            "request_cpu": "1",
            "request_memory": "4G",
        },
    )

    # Run CSV transform within kubernetes pod for damage pipeline
    damage_2015_2020_transform_csv = kubernetes_pod.KubernetesPodOperator(
        task_id="damage_2015_2020_transform_csv",
        startup_timeout_seconds=600,
        name="damage_2015_2020",
        namespace="composer",
        service_account_name="datasets",
        image_pull_policy="Always",
        image="{{ var.json.nhtsa_traffic_fatalities.container_registry.run_csv_transform_kub }}",
        env_vars={
            "PIPELINE_NAME": "{{ var.json.nhtsa_traffic_fatalities.damage_2015_2020.pipeline_name }}",
            "SOURCE_URL": "{{ var.json.nhtsa_traffic_fatalities.damage_2015_2020.source_url }}",
            "CHUNKSIZE": "{{ var.json.nhtsa_traffic_fatalities.damage_2015_2020.chunksize }}",
            "SOURCE_ZIPFILE_EXTRACTED": "damage.csv",
            "SOURCE_FILE": "{{ var.json.nhtsa_traffic_fatalities.damage_2015_2020.source_file }}",
            "PROJECT_ID": "{{ var.value.gcp_project }}",
            "DATASET_ID": "{{ var.json.nhtsa_traffic_fatalities.damage_2015_2020.dataset_id }}",
            "TABLE_ID": "{{ var.json.nhtsa_traffic_fatalities.damage_2015_2020.destination_table }}",
            "START_YEAR": "{{ var.json.nhtsa_traffic_fatalities.damage_2015_2020.start_year }}",
            "END_YEAR": "{{ var.json.nhtsa_traffic_fatalities.damage_2015_2020.end_year }}",
            "DROP_DEST_TABLE": "{{ var.json.nhtsa_traffic_fatalities.damage_2015_2020.drop_dest_table }}",
            "TARGET_GCS_BUCKET": "{{ var.value.composer_bucket }}",
            "TARGET_GCS_PATH": "{{ var.json.nhtsa_traffic_fatalities.damage_2015_2020.target_gcs_path }}",
            "SCHEMA_PATH": "{{ var.json.nhtsa_traffic_fatalities.damage_2015_2020.schema_path }}",
            "INPUT_CSV_HEADERS": '[\n  "state_number",\n  "state_name",\n  "consecutive_number",\n  "vehicle_number",\n  "damaged_areas",\n  "damaged_areas_name"\n]',
            "INPUT_DTYPES": '{\n  "state_number": "str",\n  "state_name": "str",\n  "consecutive_number": "str",\n  "vehicle_number": "str",\n  "damaged_areas": "str",\n  "damaged_areas_name": "str"\n}',
            "RENAME_MAPPINGS_LIST": '{\n  "STATE": "state_number",\n  "STATENAME": "state_name",\n  "ST_CASE": "consecutive_number",\n  "VEH_NO": "vehicle_number",\n  "MDAREAS": "damaged_areas",\n  "MDAREASNAME": "area_of_impact_this_vehicle"\n}',
        },
        resources={
            "request_ephemeral_storage": "4G",
            "request_cpu": "1",
            "request_memory": "4G",
        },
    )

    # Run CSV transform within kubernetes pod for distract pipeline
    distract_2015_2020_transform_csv = kubernetes_pod.KubernetesPodOperator(
        task_id="distract_2015_2020_transform_csv",
        startup_timeout_seconds=600,
        name="distract_2015_2020",
        namespace="composer",
        service_account_name="datasets",
        image_pull_policy="Always",
        image="{{ var.json.nhtsa_traffic_fatalities.container_registry.run_csv_transform_kub }}",
        env_vars={
            "PIPELINE_NAME": "{{ var.json.nhtsa_traffic_fatalities.distract_2015_2020.pipeline_name }}",
            "SOURCE_URL": "{{ var.json.nhtsa_traffic_fatalities.distract_2015_2020.source_url }}",
            "CHUNKSIZE": "{{ var.json.nhtsa_traffic_fatalities.distract_2015_2020.chunksize }}",
            "SOURCE_ZIPFILE_EXTRACTED": "distract.csv",
            "SOURCE_FILE": "{{ var.json.nhtsa_traffic_fatalities.distract_2015_2020.source_file }}",
            "PROJECT_ID": "{{ var.value.gcp_project }}",
            "DATASET_ID": "{{ var.json.nhtsa_traffic_fatalities.distract_2015_2020.dataset_id }}",
            "TABLE_ID": "{{ var.json.nhtsa_traffic_fatalities.distract_2015_2020.destination_table }}",
            "START_YEAR": "{{ var.json.nhtsa_traffic_fatalities.distract_2015_2020.start_year }}",
            "END_YEAR": "{{ var.json.nhtsa_traffic_fatalities.distract_2015_2020.end_year }}",
            "DROP_DEST_TABLE": "{{ var.json.nhtsa_traffic_fatalities.distract_2015_2020.drop_dest_table }}",
            "TARGET_GCS_BUCKET": "{{ var.value.composer_bucket }}",
            "TARGET_GCS_PATH": "{{ var.json.nhtsa_traffic_fatalities.distract_2015_2020.target_gcs_path }}",
            "SCHEMA_PATH": "{{ var.json.nhtsa_traffic_fatalities.distract_2015_2020.schema_path }}",
            "INPUT_CSV_HEADERS": '[\n  "state_number",\n  "state_name",\n  "consecutive_number",\n  "vehicle_number",\n  "driver_distracted_by",\n  "driver_distracted_by_name"\n]',
            "INPUT_DTYPES": '{\n  "state_number": "str",\n  "state_name": "str",\n  "consecutive_number": "str",\n  "vehicle_number": "str",\n  "driver_distracted_by": "str",\n  "driver_distracted_by_name": "str"\n}',
            "RENAME_MAPPINGS_LIST": '{\n  "STATE": "state_number",\n  "STATENAME": "state_name",\n  "ST_CASE": "consecutive_number",\n  "VEH_NO": "vehicle_number",\n  "MDRDSTRD": "driver_distracted_by",\n  "MDRDSTRDNAME": "driver_distracted_by_name"\n}',
        },
        resources={
            "request_ephemeral_storage": "4G",
            "request_cpu": "1",
            "request_memory": "4G",
        },
    )

    # Run CSV transform within kubernetes pod for drimpair pipeline
    drimpair_2015_2020_transform_csv = kubernetes_pod.KubernetesPodOperator(
        task_id="drimpair_2015_2020_transform_csv",
        startup_timeout_seconds=600,
        name="drimpair_2015_2020",
        namespace="composer",
        service_account_name="datasets",
        image_pull_policy="Always",
        image="{{ var.json.nhtsa_traffic_fatalities.container_registry.run_csv_transform_kub }}",
        env_vars={
            "PIPELINE_NAME": "{{ var.json.nhtsa_traffic_fatalities.drimpair_2015_2020.pipeline_name }}",
            "SOURCE_URL": "{{ var.json.nhtsa_traffic_fatalities.drimpair_2015_2020.source_url }}",
            "CHUNKSIZE": "{{ var.json.nhtsa_traffic_fatalities.drimpair_2015_2020.chunksize }}",
            "SOURCE_ZIPFILE_EXTRACTED": "drimpair.csv",
            "SOURCE_FILE": "{{ var.json.nhtsa_traffic_fatalities.drimpair_2015_2020.source_file }}",
            "PROJECT_ID": "{{ var.value.gcp_project }}",
            "DATASET_ID": "{{ var.json.nhtsa_traffic_fatalities.drimpair_2015_2020.dataset_id }}",
            "TABLE_ID": "{{ var.json.nhtsa_traffic_fatalities.drimpair_2015_2020.destination_table }}",
            "START_YEAR": "{{ var.json.nhtsa_traffic_fatalities.drimpair_2015_2020.start_year }}",
            "END_YEAR": "{{ var.json.nhtsa_traffic_fatalities.drimpair_2015_2020.end_year }}",
            "DROP_DEST_TABLE": "{{ var.json.nhtsa_traffic_fatalities.drimpair_2015_2020.drop_dest_table }}",
            "TARGET_GCS_BUCKET": "{{ var.value.composer_bucket }}",
            "TARGET_GCS_PATH": "{{ var.json.nhtsa_traffic_fatalities.drimpair_2015_2020.target_gcs_path }}",
            "SCHEMA_PATH": "{{ var.json.nhtsa_traffic_fatalities.drimpair_2015_2020.schema_path }}",
            "INPUT_CSV_HEADERS": '[\n  "state_number",\n  "state_name",\n  "consecutive_number",\n  "vehicle_number",\n  "condition_impairment_at_time_of_crash_driver",\n  "condition_impairment_at_time_of_crash_driver_name"\n]',
            "INPUT_DTYPES": '{\n  "state_number": "str",\n  "state_name": "str",\n  "consecutive_number": "str",\n  "vehicle_number": "str",\n  "condition_impairment_at_time_of_crash_driver": "str",\n  "condition_impairment_at_time_of_crash_driver_name": "str"\n}',
            "RENAME_MAPPINGS_LIST": '{\n  "STATE": "state_number",\n  "STATENAME": "state_name",\n  "ST_CASE": "consecutive_number",\n  "VEH_NO": "vehicle_number",\n  "DRIMPAIR": "condition_impairment_at_time_of_crash_driver",\n  "DRIMPAIRNAME": "condition_impairment_at_time_of_crash_driver_name"\n}',
        },
        resources={
            "request_ephemeral_storage": "4G",
            "request_cpu": "1",
            "request_memory": "4G",
        },
    )

    # Run CSV transform within kubernetes pod for factor pipeline
    factor_2015_2020_transform_csv = kubernetes_pod.KubernetesPodOperator(
        task_id="factor_2015_2020_transform_csv",
        startup_timeout_seconds=600,
        name="factor_2015_2020",
        namespace="composer",
        service_account_name="datasets",
        image_pull_policy="Always",
        image="{{ var.json.nhtsa_traffic_fatalities.container_registry.run_csv_transform_kub }}",
        env_vars={
            "PIPELINE_NAME": "{{ var.json.nhtsa_traffic_fatalities.factor_2015_2020.pipeline_name }}",
            "SOURCE_URL": "{{ var.json.nhtsa_traffic_fatalities.factor_2015_2020.source_url }}",
            "CHUNKSIZE": "{{ var.json.nhtsa_traffic_fatalities.factor_2015_2020.chunksize }}",
            "SOURCE_ZIPFILE_EXTRACTED": "factor.csv",
            "SOURCE_FILE": "{{ var.json.nhtsa_traffic_fatalities.factor_2015_2020.source_file }}",
            "PROJECT_ID": "{{ var.value.gcp_project }}",
            "DATASET_ID": "{{ var.json.nhtsa_traffic_fatalities.factor_2015_2020.dataset_id }}",
            "TABLE_ID": "{{ var.json.nhtsa_traffic_fatalities.factor_2015_2020.destination_table }}",
            "START_YEAR": "{{ var.json.nhtsa_traffic_fatalities.factor_2015_2020.start_year }}",
            "END_YEAR": "{{ var.json.nhtsa_traffic_fatalities.factor_2015_2020.end_year }}",
            "DROP_DEST_TABLE": "{{ var.json.nhtsa_traffic_fatalities.factor_2015_2020.drop_dest_table }}",
            "TARGET_GCS_BUCKET": "{{ var.value.composer_bucket }}",
            "TARGET_GCS_PATH": "{{ var.json.nhtsa_traffic_fatalities.factor_2015_2020.target_gcs_path }}",
            "SCHEMA_PATH": "{{ var.json.nhtsa_traffic_fatalities.factor_2015_2020.schema_path }}",
            "INPUT_CSV_HEADERS": '[\n  "state_number",\n  "state_name",\n  "consecutive_number",\n  "vehicle_number",\n  "contributing_circumstances_motor_vehicle",\n  "contributing_circumstances_motor_vehicle_name"\n]',
            "INPUT_DTYPES": '{\n  "state_number": "str",\n  "state_name": "str",\n  "consecutive_number": "str",\n  "vehicle_number": "str",\n  "contributing_circumstances_motor_vehicle": "str",\n  "contributing_circumstances_motor_vehicle_name": "str"\n}',
            "RENAME_MAPPINGS_LIST": '{\n  "STATE": "state_number",\n  "STATENAME": "state_name",\n  "ST_CASE": "consecutive_number",\n  "VEH_NO": "vehicle_number",\n  "MFACTOR": "contributing_circumstances_motor_vehicle",\n  "MFACTORNAME": "contributing_circumstances_motor_vehicle_name"\n}',
        },
        resources={
            "request_ephemeral_storage": "4G",
            "request_cpu": "1",
            "request_memory": "4G",
        },
    )

    # Run CSV transform within kubernetes pod for maneuver pipeline
    maneuver_2015_2020_transform_csv = kubernetes_pod.KubernetesPodOperator(
        task_id="maneuver_2015_2020_transform_csv",
        startup_timeout_seconds=600,
        name="maneuver_2015_2020",
        namespace="composer",
        service_account_name="datasets",
        image_pull_policy="Always",
        image="{{ var.json.nhtsa_traffic_fatalities.container_registry.run_csv_transform_kub }}",
        env_vars={
            "PIPELINE_NAME": "{{ var.json.nhtsa_traffic_fatalities.maneuver_2015_2020.pipeline_name }}",
            "SOURCE_URL": "{{ var.json.nhtsa_traffic_fatalities.maneuver_2015_2020.source_url }}",
            "CHUNKSIZE": "{{ var.json.nhtsa_traffic_fatalities.maneuver_2015_2020.chunksize }}",
            "SOURCE_ZIPFILE_EXTRACTED": "maneuver.csv",
            "SOURCE_FILE": "{{ var.json.nhtsa_traffic_fatalities.maneuver_2015_2020.source_file }}",
            "PROJECT_ID": "{{ var.value.gcp_project }}",
            "DATASET_ID": "{{ var.json.nhtsa_traffic_fatalities.maneuver_2015_2020.dataset_id }}",
            "TABLE_ID": "{{ var.json.nhtsa_traffic_fatalities.maneuver_2015_2020.destination_table }}",
            "START_YEAR": "{{ var.json.nhtsa_traffic_fatalities.maneuver_2015_2020.start_year }}",
            "END_YEAR": "{{ var.json.nhtsa_traffic_fatalities.maneuver_2015_2020.end_year }}",
            "DROP_DEST_TABLE": "{{ var.json.nhtsa_traffic_fatalities.maneuver_2015_2020.drop_dest_table }}",
            "TARGET_GCS_BUCKET": "{{ var.value.composer_bucket }}",
            "TARGET_GCS_PATH": "{{ var.json.nhtsa_traffic_fatalities.maneuver_2015_2020.target_gcs_path }}",
            "SCHEMA_PATH": "{{ var.json.nhtsa_traffic_fatalities.maneuver_2015_2020.schema_path }}",
            "INPUT_CSV_HEADERS": '[\n  "state_number",\n  "state_name",\n  "consecutive_number",\n  "vehicle_number",\n  "driver_maneuvered_to_avoid",\n  "driver_maneuvered_to_avoid_name"\n]',
            "INPUT_DTYPES": '{\n  "state_number": "str",\n  "state_name": "str",\n  "consecutive_number": "str",\n  "vehicle_number": "str",\n  "driver_maneuvered_to_avoid": "str",\n  "driver_maneuvered_to_avoid_name": "str"\n}',
            "RENAME_MAPPINGS_LIST": '{\n  "STATE": "state_number",\n  "STATENAME": "state_name",\n  "ST_CASE": "consecutive_number",\n  "VEH_NO": "vehicle_number",\n  "MDRMANAV": "driver_maneuvered_to_avoid",\n  "MDRMANAVNAME": "driver_maneuvered_to_avoid_name"\n}',
        },
        resources={
            "request_ephemeral_storage": "4G",
            "request_cpu": "1",
            "request_memory": "4G",
        },
    )

    # Run CSV transform within kubernetes pod for nmcrash pipeline
    nmcrash_2015_2020_transform_csv = kubernetes_pod.KubernetesPodOperator(
        task_id="nmcrash_2015_2020_transform_csv",
        startup_timeout_seconds=600,
        name="nmcrash_2015_2020",
        namespace="composer",
        service_account_name="datasets",
        image_pull_policy="Always",
        image="{{ var.json.nhtsa_traffic_fatalities.container_registry.run_csv_transform_kub }}",
        env_vars={
            "PIPELINE_NAME": "{{ var.json.nhtsa_traffic_fatalities.nmcrash_2015_2020.pipeline_name }}",
            "SOURCE_URL": "{{ var.json.nhtsa_traffic_fatalities.nmcrash_2015_2020.source_url }}",
            "CHUNKSIZE": "{{ var.json.nhtsa_traffic_fatalities.nmcrash_2015_2020.chunksize }}",
            "SOURCE_ZIPFILE_EXTRACTED": "nmcrash.csv",
            "SOURCE_FILE": "{{ var.json.nhtsa_traffic_fatalities.nmcrash_2015_2020.source_file }}",
            "PROJECT_ID": "{{ var.value.gcp_project }}",
            "DATASET_ID": "{{ var.json.nhtsa_traffic_fatalities.nmcrash_2015_2020.dataset_id }}",
            "TABLE_ID": "{{ var.json.nhtsa_traffic_fatalities.nmcrash_2015_2020.destination_table }}",
            "START_YEAR": "{{ var.json.nhtsa_traffic_fatalities.nmcrash_2015_2020.start_year }}",
            "END_YEAR": "{{ var.json.nhtsa_traffic_fatalities.nmcrash_2015_2020.end_year }}",
            "DROP_DEST_TABLE": "{{ var.json.nhtsa_traffic_fatalities.nmcrash_2015_2020.drop_dest_table }}",
            "TARGET_GCS_BUCKET": "{{ var.value.composer_bucket }}",
            "TARGET_GCS_PATH": "{{ var.json.nhtsa_traffic_fatalities.nmcrash_2015_2020.target_gcs_path }}",
            "SCHEMA_PATH": "{{ var.json.nhtsa_traffic_fatalities.nmcrash_2015_2020.schema_path }}",
            "INPUT_CSV_HEADERS": '[\n  "state_number",\n  "state_name",\n  "consecutive_number",\n  "vehicle_number",\n  "person_number",\n  "non_motorist_contributing_circumstances",\n  "non_motorist_contributing_circumstances_name"\n]',
            "INPUT_DTYPES": '{\n  "state_number": "str",\n  "state_name": "str",\n  "consecutive_number": "str",\n  "vehicle_number": "str",\n  "person_number": "str",\n  "non_motorist_contributing_circumstances": "str",\n  "non_motorist_contributing_circumstances_name": "str"\n}',
            "RENAME_MAPPINGS_LIST": '{\n  "STATE": "state_number",\n  "STATENAME": "state_name",\n  "ST_CASE": "consecutive_number",\n  "VEH_NO": "vehicle_number",\n  "PER_NO": "person_number",\n  "MTM_CRSH": "non_motorist_contributing_circumstances",\n  "MTM_CRSHNAME": "non_motorist_contributing_circumstances_name"\n}',
        },
        resources={
            "request_ephemeral_storage": "4G",
            "request_cpu": "1",
            "request_memory": "4G",
        },
    )

    # Run CSV transform within kubernetes pod for nmimpair pipeline
    nmimpair_2015_2020_transform_csv = kubernetes_pod.KubernetesPodOperator(
        task_id="nmimpair_2015_2020_transform_csv",
        startup_timeout_seconds=600,
        name="nmimpair_2015_2020",
        namespace="composer",
        service_account_name="datasets",
        image_pull_policy="Always",
        image="{{ var.json.nhtsa_traffic_fatalities.container_registry.run_csv_transform_kub }}",
        env_vars={
            "PIPELINE_NAME": "{{ var.json.nhtsa_traffic_fatalities.nmimpair_2015_2020.pipeline_name }}",
            "SOURCE_URL": "{{ var.json.nhtsa_traffic_fatalities.nmimpair_2015_2020.source_url }}",
            "CHUNKSIZE": "{{ var.json.nhtsa_traffic_fatalities.nmimpair_2015_2020.chunksize }}",
            "SOURCE_ZIPFILE_EXTRACTED": "nmimpair.csv",
            "SOURCE_FILE": "{{ var.json.nhtsa_traffic_fatalities.nmimpair_2015_2020.source_file }}",
            "PROJECT_ID": "{{ var.value.gcp_project }}",
            "DATASET_ID": "{{ var.json.nhtsa_traffic_fatalities.nmimpair_2015_2020.dataset_id }}",
            "TABLE_ID": "{{ var.json.nhtsa_traffic_fatalities.nmimpair_2015_2020.destination_table }}",
            "START_YEAR": "{{ var.json.nhtsa_traffic_fatalities.nmimpair_2015_2020.start_year }}",
            "END_YEAR": "{{ var.json.nhtsa_traffic_fatalities.nmimpair_2015_2020.end_year }}",
            "DROP_DEST_TABLE": "{{ var.json.nhtsa_traffic_fatalities.nmimpair_2015_2020.drop_dest_table }}",
            "TARGET_GCS_BUCKET": "{{ var.value.composer_bucket }}",
            "TARGET_GCS_PATH": "{{ var.json.nhtsa_traffic_fatalities.nmimpair_2015_2020.target_gcs_path }}",
            "SCHEMA_PATH": "{{ var.json.nhtsa_traffic_fatalities.nmimpair_2015_2020.schema_path }}",
            "INPUT_CSV_HEADERS": '[\n  "state_number",\n  "state_name",\n  "consecutive_number",\n  "vehicle_number",\n  "person_number",\n  "condition_impairment_at_time_of_crash_non_motorist",\n  "condition_impairment_at_time_of_crash_non_motorist_name"\n]',
            "INPUT_DTYPES": '{\n  "state_number": "str",\n  "state_name": "str",\n  "consecutive_number": "str",\n  "vehicle_number": "str",\n  "person_number": "str",\n  "condition_impairment_at_time_of_crash_non_motorist": "str",\n  "condition_impairment_at_time_of_crash_non_motorist_name": "str"\n}',
            "RENAME_MAPPINGS_LIST": '{\n  "STATE": "state_number",\n  "STATENAME": "state_name",\n  "ST_CASE": "consecutive_number",\n  "VEH_NO": "vehicle_number",\n  "PER_NO": "person_number",\n  "NMIMPAIR": "condition_impairment_at_time_of_crash_non_motorist",\n  "NMIMPAIRNAME": "condition_impairment_at_time_of_crash_non_motorist_name"\n}',
        },
        resources={
            "request_ephemeral_storage": "4G",
            "request_cpu": "1",
            "request_memory": "4G",
        },
    )

    # Run CSV transform within kubernetes pod for nmprior pipeline
    nmprior_2015_2020_transform_csv = kubernetes_pod.KubernetesPodOperator(
        task_id="nmprior_2015_2020_transform_csv",
        startup_timeout_seconds=600,
        name="nmprior_2015_2020",
        namespace="composer",
        service_account_name="datasets",
        image_pull_policy="Always",
        image="{{ var.json.nhtsa_traffic_fatalities.container_registry.run_csv_transform_kub }}",
        env_vars={
            "PIPELINE_NAME": "{{ var.json.nhtsa_traffic_fatalities.nmprior_2015_2020.pipeline_name }}",
            "SOURCE_URL": "{{ var.json.nhtsa_traffic_fatalities.nmprior_2015_2020.source_url }}",
            "CHUNKSIZE": "{{ var.json.nhtsa_traffic_fatalities.nmprior_2015_2020.chunksize }}",
            "SOURCE_ZIPFILE_EXTRACTED": "nmprior.csv",
            "SOURCE_FILE": "{{ var.json.nhtsa_traffic_fatalities.nmprior_2015_2020.source_file }}",
            "PROJECT_ID": "{{ var.value.gcp_project }}",
            "DATASET_ID": "{{ var.json.nhtsa_traffic_fatalities.nmprior_2015_2020.dataset_id }}",
            "TABLE_ID": "{{ var.json.nhtsa_traffic_fatalities.nmprior_2015_2020.destination_table }}",
            "START_YEAR": "{{ var.json.nhtsa_traffic_fatalities.nmprior_2015_2020.start_year }}",
            "END_YEAR": "{{ var.json.nhtsa_traffic_fatalities.nmprior_2015_2020.end_year }}",
            "DROP_DEST_TABLE": "{{ var.json.nhtsa_traffic_fatalities.nmprior_2015_2020.drop_dest_table }}",
            "TARGET_GCS_BUCKET": "{{ var.value.composer_bucket }}",
            "TARGET_GCS_PATH": "{{ var.json.nhtsa_traffic_fatalities.nmprior_2015_2020.target_gcs_path }}",
            "SCHEMA_PATH": "{{ var.json.nhtsa_traffic_fatalities.nmprior_2015_2020.schema_path }}",
            "INPUT_CSV_HEADERS": '[\n  "state_number",\n  "state_name",\n  "consecutive_number",\n  "vehicle_number",\n  "person_number",\n  "non_motorist_action_circumstances",\n  "non_motorist_action_circumstances_name"\n]',
            "INPUT_DTYPES": '{\n  "state_number": "str",\n  "state_name": "str",\n  "consecutive_number": "str",\n  "vehicle_number": "str",\n  "person_number": "str",\n  "non_motorist_action_circumstances": "str",\n  "non_motorist_action_circumstances_name": "str"\n}',
            "RENAME_MAPPINGS_LIST": '{\n  "STATE": "state_number",\n  "STATENAME": "state_name",\n  "ST_CASE": "consecutive_number",\n  "VEH_NO": "vehicle_number",\n  "PER_NO": "person_number",\n  "NMIMPAIR": "non_motorist_action_circumstances",\n  "NMIMPAIRNAME": "non_motorist_action_circumstances_name"\n}',
        },
        resources={
            "request_ephemeral_storage": "4G",
            "request_cpu": "1",
            "request_memory": "4G",
        },
    )

    # Run CSV transform within kubernetes pod for parkwork_2015 pipeline
    parkwork_2015_transform_csv = kubernetes_pod.KubernetesPodOperator(
        task_id="parkwork_2015_transform_csv",
        startup_timeout_seconds=600,
        name="parkwork_2015",
        namespace="composer",
        service_account_name="datasets",
        image_pull_policy="Always",
        image="{{ var.json.nhtsa_traffic_fatalities.container_registry.run_csv_transform_kub }}",
        env_vars={
            "PIPELINE_NAME": "{{ var.json.nhtsa_traffic_fatalities.parkwork_2015.pipeline_name }}",
            "SOURCE_URL": "{{ var.json.nhtsa_traffic_fatalities.parkwork_2015.source_url }}",
            "CHUNKSIZE": "{{ var.json.nhtsa_traffic_fatalities.parkwork_2015.chunksize }}",
            "SOURCE_ZIPFILE_EXTRACTED": "parkwork.csv",
            "SOURCE_FILE": "{{ var.json.nhtsa_traffic_fatalities.parkwork_2015.source_file }}",
            "PROJECT_ID": "{{ var.value.gcp_project }}",
            "DATASET_ID": "{{ var.json.nhtsa_traffic_fatalities.parkwork_2015.dataset_id }}",
            "TABLE_ID": "{{ var.json.nhtsa_traffic_fatalities.parkwork_2015.destination_table }}",
            "START_YEAR": "{{ var.json.nhtsa_traffic_fatalities.parkwork_2015.start_year }}",
            "END_YEAR": "{{ var.json.nhtsa_traffic_fatalities.parkwork_2015.end_year }}",
            "DROP_DEST_TABLE": "{{ var.json.nhtsa_traffic_fatalities.parkwork_2015.drop_dest_table }}",
            "TARGET_GCS_BUCKET": "{{ var.value.composer_bucket }}",
            "TARGET_GCS_PATH": "{{ var.json.nhtsa_traffic_fatalities.parkwork_2015.target_gcs_path }}",
            "SCHEMA_PATH": "{{ var.json.nhtsa_traffic_fatalities.parkwork_2015.schema_path }}",
            "INPUT_CSV_HEADERS": '[\n  "state_number",\n  "state_name",\n  "consecutive_number",\n  "vehicle_number",\n  "number_of_motor_vehicles_in_transport_mvit",\n  "number_of_occupants",\n  "number_of_occupants_name",\n  "day_of_crash",\n  "day_of_crash_name",\n  "month_of_crash",\n  "month_of_crash_name",\n  "hour_of_crash",\n  "hour_of_crash_name",\n  "minute_of_crash",\n  "minute_of_crash_name",\n  "first_harmful_event",\n  "first_harmful_event_name",\n  "manner_of_collision",\n  "manner_of_collision_name",\n  "unit_type",\n  "unit_type_name",\n  "hit_and_run",\n  "hit_and_run_name",\n  "registration_state",\n  "registration_state_name",\n  "registered_vehicle_owner",\n  "registered_vehicle_owner_name",\n  "vehicle_make",\n  "vehicle_make_name",\n  "vehicle_model",\n  "make_model_combined",\n  "body_type",\n  "body_type_name",\n  "vehicle_model_year",\n  "vehicle_model_year_name",\n  "vehicle_identification_number_vin",\n  "vehicle_identification_number_vin_name",\n  "vin_character_1",\n  "vin_character_2",\n  "vin_character_3",\n  "vin_character_4",\n  "vin_character_5",\n  "vin_character_6",\n  "vin_character_7",\n  "vin_character_8",\n  "vin_character_9",\n  "vin_character_10",\n  "vin_character_11",\n  "vin_character_12",\n  "vehicle_trailing",\n  "vehicle_trailing_name",\n  "mcid_issuing_authority",\n  "mcid_issuing_authority_name",\n  "mcid_identification_number",\n  "mcid_identification_number_name",\n  "motor_carrier_identification_number",\n  "motor_carrier_identification_number_name",\n  "gross_vehicle_weight_rating",\n  "gross_vehicle_weight_rating_name",\n  "vehicle_configuration",\n  "vehicle_configuration_name",\n  "cargo_body_type",\n  "cargo_body_type_name",\n  "hazardous_material_involvement",\n  "hazardous_material_involvement_name",\n  "hazardous_material_placard",\n  "hazardous_material_placard_name",\n  "hazardous_material_identification_number",\n  "hazardous_material_identification_number_name",\n  "hazardous_material_class_number",\n  "hazardous_material_class_number_name",\n  "release_of_hazardous_material_from_the_cargo_compartment",\n  "release_of_hazardous_material_from_the_cargo_compartment_name",\n  "bus_use",\n  "bus_use_name",\n  "special_use",\n  "special_use_name",\n  "emergency_motor_vehicle_use",\n  "emergency_motor_vehicle_use_name",\n  "underride_override",\n  "underride_override_name",\n  "initial_contact_point",\n  "initial_contact_point_name",\n  "extent_of_damage",\n  "extent_of_damage_name",\n  "vehicle_removal",\n  "vehicle_removal_name",\n  "most_harmful_event",\n  "most_harmful_event_name",\n  "related_factors_vehicle_level1",\n  "related_factors_vehicle_level1_name",\n  "related_factors_vehicle_level2",\n  "related_factors_vehicle_level2_name",\n  "fire_occurrence",\n  "fire_occurrence_name",\n  "fatalities_in_vehicle"\n]',
            "INPUT_DTYPES": '{\n  "state_number": "str",\n  "state_name": "str",\n  "consecutive_number": "str",\n  "vehicle_number": "str",\n  "number_of_motor_vehicles_in_transport_mvit": "str",\n  "number_of_occupants": "str",\n  "number_of_occupants_name": "str",\n  "day_of_crash": "str",\n  "day_of_crash_name": "str",\n  "month_of_crash": "str",\n  "month_of_crash_name": "str",\n  "hour_of_crash": "str",\n  "hour_of_crash_name": "str",\n  "minute_of_crash": "str",\n  "minute_of_crash_name": "str",\n  "first_harmful_event": "str",\n  "first_harmful_event_name": "str",\n  "manner_of_collision": "str",\n  "manner_of_collision_name": "str",\n  "unit_type": "str",\n  "unit_type_name": "str",\n  "hit_and_run": "str",\n  "hit_and_run_name": "str",\n  "registration_state": "str",\n  "registration_state_name": "str",\n  "registered_vehicle_owner": "str",\n  "registered_vehicle_owner_name": "str",\n  "vehicle_make": "str",\n  "vehicle_make_name": "str",\n  "vehicle_model": "str",\n  "make_model_combined": "str",\n  "body_type": "str",\n  "body_type_name": "str",\n  "vehicle_model_year": "str",\n  "vehicle_model_year_name": "str",\n  "vehicle_identification_number_vin": "str",\n  "vehicle_identification_number_vin_name": "str",\n  "vin_character_1": "str",\n  "vin_character_2": "str",\n  "vin_character_3": "str",\n  "vin_character_4": "str",\n  "vin_character_5": "str",\n  "vin_character_6": "str",\n  "vin_character_7": "str",\n  "vin_character_8": "str",\n  "vin_character_9": "str",\n  "vin_character_10": "str",\n  "vin_character_11": "str",\n  "vin_character_12": "str",\n  "vehicle_trailing": "str",\n  "vehicle_trailing_name": "str",\n  "mcid_issuing_authority": "str",\n  "mcid_issuing_authority_name": "str",\n  "mcid_identification_number": "str",\n  "mcid_identification_number_name": "str",\n  "motor_carrier_identification_number": "str",\n  "motor_carrier_identification_number_name": "str",\n  "gross_vehicle_weight_rating": "str",\n  "gross_vehicle_weight_rating_name": "str",\n  "vehicle_configuration": "str",\n  "vehicle_configuration_name": "str",\n  "cargo_body_type": "str",\n  "cargo_body_type_name": "str",\n  "hazardous_material_involvement": "str",\n  "hazardous_material_involvement_name": "str",\n  "hazardous_material_placard": "str",\n  "hazardous_material_placard_name": "str",\n  "hazardous_material_identification_number": "str",\n  "hazardous_material_identification_number_name": "str",\n  "hazardous_material_class_number": "str",\n  "hazardous_material_class_number_name": "str",\n  "release_of_hazardous_material_from_the_cargo_compartment": "str",\n  "release_of_hazardous_material_from_the_cargo_compartment_name": "str",\n  "bus_use": "str",\n  "bus_use_name": "str",\n  "special_use": "str",\n  "special_use_name": "str",\n  "emergency_motor_vehicle_use": "str",\n  "emergency_motor_vehicle_use_name": "str",\n  "underride_override": "str",\n  "underride_override_name": "str",\n  "initial_contact_point": "str",\n  "initial_contact_point_name": "str",\n  "extent_of_damage": "str",\n  "extent_of_damage_name": "str",\n  "vehicle_removal": "str",\n  "vehicle_removal_name": "str",\n  "most_harmful_event": "str",\n  "most_harmful_event_name": "str",\n  "related_factors_vehicle_level1": "str",\n  "related_factors_vehicle_level1_name": "str",\n  "related_factors_vehicle_level2": "str",\n  "related_factors_vehicle_level2_name": "str",\n  "fire_occurrence": "str",\n  "fire_occurrence_name": "str",\n  "fatalities_in_vehicle": "str"\n}',
            "RENAME_MAPPINGS_LIST": '{\n  "STATE": "state_number",\n  "STATENAME": "state_name",\n  "ST_CASE": "consecutive_number",\n  "VEH_NO": "vehicle_number",\n  "PVE_FORMS": "number_of_motor_vehicles_in_transport_mvit",\n  "PNUMOCCS": "number_of_occupants",\n  "PNUMOCCSNAME": "number_of_occupants_name",\n  "PDAY": "day_of_crash",\n  "PDAYNAME": "day_of_crash_name",\n  "PMONTH": "month_of_crash",\n  "PMONTHNAME": "month_of_crash_name",\n  "PHOUR": "hour_of_crash",\n  "PHOURNAME": "hour_of_crash_name",\n  "PMINUTE": "minute_of_crash",\n  "PMINUTENAME": "minute_of_crash_name",\n  "PHARM_EV": "first_harmful_event",\n  "PHARM_EVNAME": "first_harmful_event_name",\n  "PMAN_COLL": "manner_of_collision",\n  "PMAN_COLLNAME": "manner_of_collision_name",\n  "PTYPE": "unit_type",\n  "PTYPENAME": "unit_type_name",\n  "PHIT_RUN": "hit_and_run",\n  "PHIT_RUNNAME": "hit_and_run_name",\n  "PREG_STAT": "registration_state",\n  "PREG_STATNAME": "registration_state_name",\n  "POWNER": "registered_vehicle_owner",\n  "POWNERNAME": "registered_vehicle_owner_name",\n  "PMAKE": "vehicle_make",\n  "PMAKENAME": "vehicle_make_name",\n  "PMODEL": "vehicle_model",\n  "PMAK_MOD": "make_model_combined",\n  "PBODYTYP": "body_type",\n  "PBODYTYPNAME": "body_type_name",\n  "PMODYEAR": "vehicle_model_year",\n  "PMODYEARNAME": "vehicle_model_year_name",\n  "PVIN": "vehicle_identification_number_vin",\n  "PVINNAME": "vehicle_identification_number_vin_name",\n  "PVIN_1": "vin_character_1",\n  "PVIN_2": "vin_character_2",\n  "PVIN_3": "vin_character_3",\n  "PVIN_4": "vin_character_4",\n  "PVIN_5": "vin_character_5",\n  "PVIN_6": "vin_character_6",\n  "PVIN_7": "vin_character_7",\n  "PVIN_8": "vin_character_8",\n  "PVIN_9": "vin_character_9",\n  "PVIN_10": "vin_character_10",\n  "PVIN_11": "vin_character_11",\n  "PVIN_12": "vin_character_12",\n  "PTRAILER": "vehicle_trailing",\n  "PTRAILERNAME": "vehicle_trailing_name",\n  "PMCARR_I1": "mcid_issuing_authority",\n  "PMCARR_I1NAME": "mcid_issuing_authority_name",\n  "PMCARR_I2": "mcid_identification_number",\n  "PMCARR_I2NAME": "mcid_identification_number_name",\n  "PMCARR_ID": "motor_carrier_identification_number",\n  "PMCARR_IDNAME": "motor_carrier_identification_number_name",\n  "PGVWR": "gross_vehicle_weight_rating",\n  "PGVWRNAME": "gross_vehicle_weight_rating_name",\n  "PV_CONFIG": "vehicle_configuration",\n  "PV_CONFIGNAME": "vehicle_configuration_name",\n  "PCARGTYP": "cargo_body_type",\n  "PCARGTYPNAME": "cargo_body_type_name",\n  "PHAZ_INV": "hazardous_material_involvement",\n  "PHAZ_INVNAME": "hazardous_material_involvement_name",\n  "PHAZPLAC": "hazardous_material_placard",\n  "PHAZPLACNAME": "hazardous_material_placard_name",\n  "PHAZ_ID": "hazardous_material_identification_number",\n  "PHAZ_IDNAME": "hazardous_material_identification_number_name",\n  "PHAZ_CNO": "hazardous_material_class_number",\n  "PHAZ_CNONAME": "hazardous_material_class_number_name",\n  "PHAZ_REL": "release_of_hazardous_material_from_the_cargo_compartment",\n  "PHAZ_RELNAME": "release_of_hazardous_material_from_the_cargo_compartment_name",\n  "PBUS_USE": "bus_use",\n  "PBUS_USENAME": "bus_use_name",\n  "PSP_USE": "special_use",\n  "PSP_USENAME": "special_use_name",\n  "PEM_USE": "emergency_motor_vehicle_use",\n  "PEM_USENAME": "emergency_motor_vehicle_use_name",\n  "PUNDERIDE": "underride_override",\n  "PUNDERIDENAME": "underride_override_name",\n  "PIMPACT1": "initial_contact_point",\n  "PIMPACT1NAME": "initial_contact_point_name",\n  "PVEH_SEV": "extent_of_damage",\n  "PVEH_SEVNAME": "extent_of_damage_name",\n  "PTOWED": "vehicle_removal",\n  "PTOWEDNAME": "vehicle_removal_name",\n  "PM_HARM": "most_harmful_event",\n  "PM_HARMNAME": "most_harmful_event_name",\n  "PVEH_SC1": "related_factors_vehicle_level1",\n  "PVEH_SC1NAME": "related_factors_vehicle_level1_name",\n  "PVEH_SC2": "related_factors_vehicle_level2",\n  "PVEH_SC2NAME": "related_factors_vehicle_level2_name",\n  "PFIRE": "fire_occurrence",\n  "PFIRENAME": "fire_occurrence_name",\n  "PDEATHS": "fatalities_in_vehicle"\n}',
        },
        resources={
            "request_ephemeral_storage": "4G",
            "request_cpu": "1",
            "request_memory": "4G",
        },
    )

    # Run CSV transform within kubernetes pod for parkwork_2016_2017 pipelines
    parkwork_2016_2017_transform_csv = kubernetes_pod.KubernetesPodOperator(
        task_id="parkwork_2016_2017_transform_csv",
        startup_timeout_seconds=600,
        name="parkwork_2016_2017",
        namespace="composer",
        service_account_name="datasets",
        image_pull_policy="Always",
        image="{{ var.json.nhtsa_traffic_fatalities.container_registry.run_csv_transform_kub }}",
        env_vars={
            "PIPELINE_NAME": "{{ var.json.nhtsa_traffic_fatalities.parkwork_2016_2017.pipeline_name }}",
            "SOURCE_URL": "{{ var.json.nhtsa_traffic_fatalities.parkwork_2016_2017.source_url }}",
            "CHUNKSIZE": "{{ var.json.nhtsa_traffic_fatalities.parkwork_2016_2017.chunksize }}",
            "SOURCE_ZIPFILE_EXTRACTED": "parkwork.csv",
            "SOURCE_FILE": "{{ var.json.nhtsa_traffic_fatalities.parkwork_2016_2017.source_file }}",
            "PROJECT_ID": "{{ var.value.gcp_project }}",
            "DATASET_ID": "{{ var.json.nhtsa_traffic_fatalities.parkwork_2016_2017.dataset_id }}",
            "TABLE_ID": "{{ var.json.nhtsa_traffic_fatalities.parkwork_2016_2017.destination_table }}",
            "START_YEAR": "{{ var.json.nhtsa_traffic_fatalities.parkwork_2016_2017.start_year }}",
            "END_YEAR": "{{ var.json.nhtsa_traffic_fatalities.parkwork_2016_2017.end_year }}",
            "DROP_DEST_TABLE": "{{ var.json.nhtsa_traffic_fatalities.parkwork_2016_2017.drop_dest_table }}",
            "TARGET_GCS_BUCKET": "{{ var.value.composer_bucket }}",
            "TARGET_GCS_PATH": "{{ var.json.nhtsa_traffic_fatalities.parkwork_2016_2017.target_gcs_path }}",
            "SCHEMA_PATH": "{{ var.json.nhtsa_traffic_fatalities.parkwork_2016_2017.schema_path }}",
            "INPUT_CSV_HEADERS": '[\n  "state_number",\n  "state_name",\n  "consecutive_number",\n  "vehicle_number",\n  "number_of_motor_vehicles_in_transport_mvit",\n  "number_of_occupants",\n  "number_of_occupants_name",\n  "day_of_crash",\n  "day_of_crash_name",\n  "month_of_crash",\n  "month_of_crash_name",\n  "hour_of_crash",\n  "hour_of_crash_name",\n  "minute_of_crash",\n  "minute_of_crash_name",\n  "first_harmful_event",\n  "first_harmful_event_name",\n  "manner_of_collision",\n  "manner_of_collision_name",\n  "unit_type",\n  "unit_type_name",\n  "hit_and_run",\n  "hit_and_run_name",\n  "registration_state",\n  "registration_state_name",\n  "registered_vehicle_owner",\n  "registered_vehicle_owner_name",\n  "vehicle_make",\n  "vehicle_make_name",\n  "vehicle_model",\n  "make_model_combined",\n  "body_type",\n  "body_type_name",\n  "vehicle_model_year",\n  "vehicle_model_year_name",\n  "vehicle_identification_number_vin",\n  "vehicle_identification_number_vin_name",\n  "vin_character_1",\n  "vin_character_2",\n  "vin_character_3",\n  "vin_character_4",\n  "vin_character_5",\n  "vin_character_6",\n  "vin_character_7",\n  "vin_character_8",\n  "vin_character_9",\n  "vin_character_10",\n  "vin_character_11",\n  "vin_character_12",\n  "vehicle_trailing",\n  "vehicle_trailing_name",\n  "mcid_issuing_authority",\n  "mcid_issuing_authority_name",\n  "mcid_identification_number",\n  "mcid_identification_number_name",\n  "motor_carrier_identification_number",\n  "motor_carrier_identification_number_name",\n  "gross_vehicle_weight_rating",\n  "gross_vehicle_weight_rating_name",\n  "vehicle_configuration",\n  "vehicle_configuration_name",\n  "cargo_body_type",\n  "cargo_body_type_name",\n  "hazardous_material_involvement",\n  "hazardous_material_involvement_name",\n  "hazardous_material_placard",\n  "hazardous_material_placard_name",\n  "hazardous_material_identification_number",\n  "hazardous_material_identification_number_name",\n  "hazardous_material_class_number",\n  "hazardous_material_class_number_name",\n  "release_of_hazardous_material_from_the_cargo_compartment",\n  "release_of_hazardous_material_from_the_cargo_compartment_name",\n  "bus_use",\n  "bus_use_name",\n  "special_use",\n  "special_use_name",\n  "emergency_motor_vehicle_use",\n  "emergency_motor_vehicle_use_name",\n  "underride_override",\n  "underride_override_name",\n  "initial_contact_point",\n  "initial_contact_point_name",\n  "extent_of_damage",\n  "extent_of_damage_name",\n  "vehicle_removal",\n  "vehicle_removal_name",\n  "most_harmful_event",\n  "most_harmful_event_name",\n  "related_factors_vehicle_level1",\n  "related_factors_vehicle_level1_name",\n  "related_factors_vehicle_level2",\n  "related_factors_vehicle_level2_name",\n  "fire_occurrence",\n  "fire_occurrence_name",\n  "fatalities_in_vehicle",\n  "ptrlr1vin",\n  "ptrlr1vinname",\n  "ptrlr2vin",\n  "ptrlr2vinname",\n  "ptrlr3vin",\n  "ptrlr3vinname"\n]',
            "INPUT_DTYPES": '{\n  "state_number": "str",\n  "state_name": "str",\n  "consecutive_number": "str",\n  "vehicle_number": "str",\n  "number_of_motor_vehicles_in_transport_mvit": "str",\n  "number_of_occupants": "str",\n  "number_of_occupants_name": "str",\n  "day_of_crash": "str",\n  "day_of_crash_name": "str",\n  "month_of_crash": "str",\n  "month_of_crash_name": "str",\n  "hour_of_crash": "str",\n  "hour_of_crash_name": "str",\n  "minute_of_crash": "str",\n  "minute_of_crash_name": "str",\n  "first_harmful_event": "str",\n  "first_harmful_event_name": "str",\n  "manner_of_collision": "str",\n  "manner_of_collision_name": "str",\n  "unit_type": "str",\n  "unit_type_name": "str",\n  "hit_and_run": "str",\n  "hit_and_run_name": "str",\n  "registration_state": "str",\n  "registration_state_name": "str",\n  "registered_vehicle_owner": "str",\n  "registered_vehicle_owner_name": "str",\n  "vehicle_make": "str",\n  "vehicle_make_name": "str",\n  "vehicle_model": "str",\n  "make_model_combined": "str",\n  "body_type": "str",\n  "body_type_name": "str",\n  "vehicle_model_year": "str",\n  "vehicle_model_year_name": "str",\n  "vehicle_identification_number_vin": "str",\n  "vehicle_identification_number_vin_name": "str",\n  "vin_character_1": "str",\n  "vin_character_2": "str",\n  "vin_character_3": "str",\n  "vin_character_4": "str",\n  "vin_character_5": "str",\n  "vin_character_6": "str",\n  "vin_character_7": "str",\n  "vin_character_8": "str",\n  "vin_character_9": "str",\n  "vin_character_10": "str",\n  "vin_character_11": "str",\n  "vin_character_12": "str",\n  "vehicle_trailing": "str",\n  "vehicle_trailing_name": "str",\n  "mcid_issuing_authority": "str",\n  "mcid_issuing_authority_name": "str",\n  "mcid_identification_number": "str",\n  "mcid_identification_number_name": "str",\n  "motor_carrier_identification_number": "str",\n  "motor_carrier_identification_number_name": "str",\n  "gross_vehicle_weight_rating": "str",\n  "gross_vehicle_weight_rating_name": "str",\n  "vehicle_configuration": "str",\n  "vehicle_configuration_name": "str",\n  "cargo_body_type": "str",\n  "cargo_body_type_name": "str",\n  "hazardous_material_involvement": "str",\n  "hazardous_material_involvement_name": "str",\n  "hazardous_material_placard": "str",\n  "hazardous_material_placard_name": "str",\n  "hazardous_material_identification_number": "str",\n  "hazardous_material_identification_number_name": "str",\n  "hazardous_material_class_number": "str",\n  "hazardous_material_class_number_name": "str",\n  "release_of_hazardous_material_from_the_cargo_compartment": "str",\n  "release_of_hazardous_material_from_the_cargo_compartment_name": "str",\n  "bus_use": "str",\n  "bus_use_name": "str",\n  "special_use": "str",\n  "special_use_name": "str",\n  "emergency_motor_vehicle_use": "str",\n  "emergency_motor_vehicle_use_name": "str",\n  "underride_override": "str",\n  "underride_override_name": "str",\n  "initial_contact_point": "str",\n  "initial_contact_point_name": "str",\n  "extent_of_damage": "str",\n  "extent_of_damage_name": "str",\n  "vehicle_removal": "str",\n  "vehicle_removal_name": "str",\n  "most_harmful_event": "str",\n  "most_harmful_event_name": "str",\n  "related_factors_vehicle_level1": "str",\n  "related_factors_vehicle_level1_name": "str",\n  "related_factors_vehicle_level2": "str",\n  "related_factors_vehicle_level2_name": "str",\n  "fire_occurrence": "str",\n  "fire_occurrence_name": "str",\n  "fatalities_in_vehicle": "str",\n  "ptrlr1vin": "str",\n  "ptrlr1vinname": "str",\n  "ptrlr2vin": "str",\n  "ptrlr2vinname": "str",\n  "ptrlr3vin": "str",\n  "ptrlr3vinname": "str"\n}',
            "RENAME_MAPPINGS_LIST": '{\n  "STATE": "state_number",\n  "STATENAME": "state_name",\n  "ST_CASE": "consecutive_number",\n  "VEH_NO": "vehicle_number",\n  "PVE_FORMS": "number_of_motor_vehicles_in_transport_mvit",\n  "PNUMOCCS": "number_of_occupants",\n  "PNUMOCCSNAME": "number_of_occupants_name",\n  "PDAY": "day_of_crash",\n  "PDAYNAME": "day_of_crash_name",\n  "PMONTH": "month_of_crash",\n  "PMONTHNAME": "month_of_crash_name",\n  "PHOUR": "hour_of_crash",\n  "PHOURNAME": "hour_of_crash_name",\n  "PMINUTE": "minute_of_crash",\n  "PMINUTENAME": "minute_of_crash_name",\n  "PHARM_EV": "first_harmful_event",\n  "PHARM_EVNAME": "first_harmful_event_name",\n  "PMAN_COLL": "manner_of_collision",\n  "PMAN_COLLNAME": "manner_of_collision_name",\n  "PTYPE": "unit_type",\n  "PTYPENAME": "unit_type_name",\n  "PHIT_RUN": "hit_and_run",\n  "PHIT_RUNNAME": "hit_and_run_name",\n  "PREG_STAT": "registration_state",\n  "PREG_STATNAME": "registration_state_name",\n  "POWNER": "registered_vehicle_owner",\n  "POWNERNAME": "registered_vehicle_owner_name",\n  "PMAKE": "vehicle_make",\n  "PMAKENAME": "vehicle_make_name",\n  "PMODEL": "vehicle_model",\n  "PMAK_MOD": "make_model_combined",\n  "PBODYTYP": "body_type",\n  "PBODYTYPNAME": "body_type_name",\n  "PMODYEAR": "vehicle_model_year",\n  "PMODYEARNAME": "vehicle_model_year_name",\n  "PVIN": "vehicle_identification_number_vin",\n  "PVINNAME": "vehicle_identification_number_vin_name",\n  "PVIN_1": "vin_character_1",\n  "PVIN_2": "vin_character_2",\n  "PVIN_3": "vin_character_3",\n  "PVIN_4": "vin_character_4",\n  "PVIN_5": "vin_character_5",\n  "PVIN_6": "vin_character_6",\n  "PVIN_7": "vin_character_7",\n  "PVIN_8": "vin_character_8",\n  "PVIN_9": "vin_character_9",\n  "PVIN_10": "vin_character_10",\n  "PVIN_11": "vin_character_11",\n  "PVIN_12": "vin_character_12",\n  "PTRAILER": "vehicle_trailing",\n  "PTRAILERNAME": "vehicle_trailing_name",\n  "PMCARR_I1": "mcid_issuing_authority",\n  "PMCARR_I1NAME": "mcid_issuing_authority_name",\n  "PMCARR_I2": "mcid_identification_number",\n  "PMCARR_I2NAME": "mcid_identification_number_name",\n  "PMCARR_ID": "motor_carrier_identification_number",\n  "PMCARR_IDNAME": "motor_carrier_identification_number_name",\n  "PGVWR": "gross_vehicle_weight_rating",\n  "PGVWRNAME": "gross_vehicle_weight_rating_name",\n  "PV_CONFIG": "vehicle_configuration",\n  "PV_CONFIGNAME": "vehicle_configuration_name",\n  "PCARGTYP": "cargo_body_type",\n  "PCARGTYPNAME": "cargo_body_type_name",\n  "PHAZ_INV": "hazardous_material_involvement",\n  "PHAZ_INVNAME": "hazardous_material_involvement_name",\n  "PHAZPLAC": "hazardous_material_placard",\n  "PHAZPLACNAME": "hazardous_material_placard_name",\n  "PHAZ_ID": "hazardous_material_identification_number",\n  "PHAZ_IDNAME": "hazardous_material_identification_number_name",\n  "PHAZ_CNO": "hazardous_material_class_number",\n  "PHAZ_CNONAME": "hazardous_material_class_number_name",\n  "PHAZ_REL": "release_of_hazardous_material_from_the_cargo_compartment",\n  "PHAZ_RELNAME": "release_of_hazardous_material_from_the_cargo_compartment_name",\n  "PBUS_USE": "bus_use",\n  "PBUS_USENAME": "bus_use_name",\n  "PSP_USE": "special_use",\n  "PSP_USENAME": "special_use_name",\n  "PEM_USE": "emergency_motor_vehicle_use",\n  "PEM_USENAME": "emergency_motor_vehicle_use_name",\n  "PUNDERIDE": "underride_override",\n  "PUNDERIDENAME": "underride_override_name",\n  "PIMPACT1": "initial_contact_point",\n  "PIMPACT1NAME": "initial_contact_point_name",\n  "PVEH_SEV": "extent_of_damage",\n  "PVEH_SEVNAME": "extent_of_damage_name",\n  "PTOWED": "vehicle_removal",\n  "PTOWEDNAME": "vehicle_removal_name",\n  "PM_HARM": "most_harmful_event",\n  "PM_HARMNAME": "most_harmful_event_name",\n  "PVEH_SC1": "related_factors_vehicle_level1",\n  "PVEH_SC1NAME": "related_factors_vehicle_level1_name",\n  "PVEH_SC2": "related_factors_vehicle_level2",\n  "PVEH_SC2NAME": "related_factors_vehicle_level2_name",\n  "PFIRE": "fire_occurrence",\n  "PFIRENAME": "fire_occurrence_name",\n  "PDEATHS": "fatalities_in_vehicle",\n  "PTRLR1VIN": "ptrlr1vin",\n  "PTRLR1VINNAME": "ptrlr1vinname",\n  "PTRLR2VIN": "ptrlr2vin",\n  "PTRLR2VINNAME": "ptrlr2vinname",\n  "PTRLR3VIN": "ptrlr3vin",\n  "PTRLR3VINNAME": "ptrlr3vinname"\n}',
        },
        resources={
            "request_ephemeral_storage": "4G",
            "request_cpu": "1",
            "request_memory": "4G",
        },
    )

    # Run CSV transform within kubernetes pod for parkwork_2018 pipelines
    parkwork_2018_transform_csv = kubernetes_pod.KubernetesPodOperator(
        task_id="parkwork_2018_transform_csv",
        startup_timeout_seconds=600,
        name="parkwork_2018",
        namespace="composer",
        service_account_name="datasets",
        image_pull_policy="Always",
        image="{{ var.json.nhtsa_traffic_fatalities.container_registry.run_csv_transform_kub }}",
        env_vars={
            "PIPELINE_NAME": "{{ var.json.nhtsa_traffic_fatalities.parkwork_2018.pipeline_name }}",
            "SOURCE_URL": "{{ var.json.nhtsa_traffic_fatalities.parkwork_2018.source_url }}",
            "CHUNKSIZE": "{{ var.json.nhtsa_traffic_fatalities.parkwork_2018.chunksize }}",
            "SOURCE_ZIPFILE_EXTRACTED": "parkwork.csv",
            "SOURCE_FILE": "{{ var.json.nhtsa_traffic_fatalities.parkwork_2018.source_file }}",
            "PROJECT_ID": "{{ var.value.gcp_project }}",
            "DATASET_ID": "{{ var.json.nhtsa_traffic_fatalities.parkwork_2018.dataset_id }}",
            "TABLE_ID": "{{ var.json.nhtsa_traffic_fatalities.parkwork_2018.destination_table }}",
            "START_YEAR": "{{ var.json.nhtsa_traffic_fatalities.parkwork_2018.start_year }}",
            "END_YEAR": "{{ var.json.nhtsa_traffic_fatalities.parkwork_2018.end_year }}",
            "DROP_DEST_TABLE": "{{ var.json.nhtsa_traffic_fatalities.parkwork_2018.drop_dest_table }}",
            "TARGET_GCS_BUCKET": "{{ var.value.composer_bucket }}",
            "TARGET_GCS_PATH": "{{ var.json.nhtsa_traffic_fatalities.parkwork_2018.target_gcs_path }}",
            "SCHEMA_PATH": "{{ var.json.nhtsa_traffic_fatalities.parkwork_2018.schema_path }}",
            "INPUT_CSV_HEADERS": '[\n  "state_number",\n  "state_name",\n  "consecutive_number",\n  "vehicle_number",\n  "number_of_motor_vehicles_in_transport_mvit",\n  "number_of_occupants",\n  "number_of_occupants_name",\n  "day_of_crash",\n  "day_of_crash_name",\n  "month_of_crash",\n  "month_of_crash_name",\n  "hour_of_crash",\n  "hour_of_crash_name",\n  "minute_of_crash",\n  "minute_of_crash_name",\n  "first_harmful_event",\n  "first_harmful_event_name",\n  "manner_of_collision",\n  "manner_of_collision_name",\n  "unit_type",\n  "unit_type_name",\n  "hit_and_run",\n  "hit_and_run_name",\n  "registration_state",\n  "registration_state_name",\n  "registered_vehicle_owner",\n  "registered_vehicle_owner_name",\n  "vehicle_make",\n  "vehicle_make_name",\n  "vehicle_model",\n  "make_model_combined",\n  "make_model_combined_name",\n  "body_type",\n  "body_type_name",\n  "vehicle_model_year",\n  "vehicle_model_year_name",\n  "vehicle_identification_number_vin",\n  "vehicle_identification_number_vin_name",\n  "vin_character_1",\n  "vin_character_2",\n  "vin_character_3",\n  "vin_character_4",\n  "vin_character_5",\n  "vin_character_6",\n  "vin_character_7",\n  "vin_character_8",\n  "vin_character_9",\n  "vin_character_10",\n  "vin_character_11",\n  "vin_character_12",\n  "vehicle_trailing",\n  "vehicle_trailing_name",\n  "mcid_issuing_authority",\n  "mcid_issuing_authority_name",\n  "mcid_identification_number",\n  "mcid_identification_number_name",\n  "motor_carrier_identification_number",\n  "motor_carrier_identification_number_name",\n  "gross_vehicle_weight_rating",\n  "gross_vehicle_weight_rating_name",\n  "vehicle_configuration",\n  "vehicle_configuration_name",\n  "cargo_body_type",\n  "cargo_body_type_name",\n  "hazardous_material_involvement",\n  "hazardous_material_involvement_name",\n  "hazardous_material_placard",\n  "hazardous_material_placard_name",\n  "hazardous_material_identification_number",\n  "hazardous_material_identification_number_name",\n  "hazardous_material_class_number",\n  "hazardous_material_class_number_name",\n  "release_of_hazardous_material_from_the_cargo_compartment",\n  "release_of_hazardous_material_from_the_cargo_compartment_name",\n  "bus_use",\n  "bus_use_name",\n  "special_use",\n  "special_use_name",\n  "emergency_motor_vehicle_use",\n  "emergency_motor_vehicle_use_name",\n  "underride_override",\n  "underride_override_name",\n  "initial_contact_point",\n  "initial_contact_point_name",\n  "extent_of_damage",\n  "extent_of_damage_name",\n  "vehicle_removal",\n  "vehicle_removal_name",\n  "most_harmful_event",\n  "most_harmful_event_name",\n  "related_factors_vehicle_level1",\n  "related_factors_vehicle_level1_name",\n  "related_factors_vehicle_level2",\n  "related_factors_vehicle_level2_name",\n  "fire_occurrence",\n  "fire_occurrence_name",\n  "fatalities_in_vehicle",\n  "ptrlr1vin",\n  "ptrlr1vinname",\n  "ptrlr2vin",\n  "ptrlr2vinname",\n  "ptrlr3vin",\n  "ptrlr3vinname"\n]',
            "INPUT_DTYPES": '{\n  "state_number": "str",\n  "state_name": "str",\n  "consecutive_number": "str",\n  "vehicle_number": "str",\n  "number_of_motor_vehicles_in_transport_mvit": "str",\n  "number_of_occupants": "str",\n  "number_of_occupants_name": "str",\n  "day_of_crash": "str",\n  "day_of_crash_name": "str",\n  "month_of_crash": "str",\n  "month_of_crash_name": "str",\n  "hour_of_crash": "str",\n  "hour_of_crash_name": "str",\n  "minute_of_crash": "str",\n  "minute_of_crash_name": "str",\n  "first_harmful_event": "str",\n  "first_harmful_event_name": "str",\n  "manner_of_collision": "str",\n  "manner_of_collision_name": "str",\n  "unit_type": "str",\n  "unit_type_name": "str",\n  "hit_and_run": "str",\n  "hit_and_run_name": "str",\n  "registration_state": "str",\n  "registration_state_name": "str",\n  "registered_vehicle_owner": "str",\n  "registered_vehicle_owner_name": "str",\n  "vehicle_make": "str",\n  "vehicle_make_name": "str",\n  "vehicle_model": "str",\n  "make_model_combined": "str",\n  "make_model_combined_name": "str",\n  "body_type": "str",\n  "body_type_name": "str",\n  "vehicle_model_year": "str",\n  "vehicle_model_year_name": "str",\n  "vehicle_identification_number_vin": "str",\n  "vehicle_identification_number_vin_name": "str",\n  "vin_character_1": "str",\n  "vin_character_2": "str",\n  "vin_character_3": "str",\n  "vin_character_4": "str",\n  "vin_character_5": "str",\n  "vin_character_6": "str",\n  "vin_character_7": "str",\n  "vin_character_8": "str",\n  "vin_character_9": "str",\n  "vin_character_10": "str",\n  "vin_character_11": "str",\n  "vin_character_12": "str",\n  "vehicle_trailing": "str",\n  "vehicle_trailing_name": "str",\n  "mcid_issuing_authority": "str",\n  "mcid_issuing_authority_name": "str",\n  "mcid_identification_number": "str",\n  "mcid_identification_number_name": "str",\n  "motor_carrier_identification_number": "str",\n  "motor_carrier_identification_number_name": "str",\n  "gross_vehicle_weight_rating": "str",\n  "gross_vehicle_weight_rating_name": "str",\n  "vehicle_configuration": "str",\n  "vehicle_configuration_name": "str",\n  "cargo_body_type": "str",\n  "cargo_body_type_name": "str",\n  "hazardous_material_involvement": "str",\n  "hazardous_material_involvement_name": "str",\n  "hazardous_material_placard": "str",\n  "hazardous_material_placard_name": "str",\n  "hazardous_material_identification_number": "str",\n  "hazardous_material_identification_number_name": "str",\n  "hazardous_material_class_number": "str",\n  "hazardous_material_class_number_name": "str",\n  "release_of_hazardous_material_from_the_cargo_compartment": "str",\n  "release_of_hazardous_material_from_the_cargo_compartment_name": "str",\n  "bus_use": "str",\n  "bus_use_name": "str",\n  "special_use": "str",\n  "special_use_name": "str",\n  "emergency_motor_vehicle_use": "str",\n  "emergency_motor_vehicle_use_name": "str",\n  "underride_override": "str",\n  "underride_override_name": "str",\n  "initial_contact_point": "str",\n  "initial_contact_point_name": "str",\n  "extent_of_damage": "str",\n  "extent_of_damage_name": "str",\n  "vehicle_removal": "str",\n  "vehicle_removal_name": "str",\n  "most_harmful_event": "str",\n  "most_harmful_event_name": "str",\n  "related_factors_vehicle_level1": "str",\n  "related_factors_vehicle_level1_name": "str",\n  "related_factors_vehicle_level2": "str",\n  "related_factors_vehicle_level2_name": "str",\n  "fire_occurrence": "str",\n  "fire_occurrence_name": "str",\n  "fatalities_in_vehicle": "str",\n  "ptrlr1vin": "str",\n  "ptrlr1vinname": "str",\n  "ptrlr2vin": "str",\n  "ptrlr2vinname": "str",\n  "ptrlr3vin": "str",\n  "ptrlr3vinname": "str"\n}',
            "RENAME_MAPPINGS_LIST": '{\n  "STATE": "state_number",\n  "STATENAME": "state_name",\n  "ST_CASE": "consecutive_number",\n  "VEH_NO": "vehicle_number",\n  "PVE_FORMS": "number_of_motor_vehicles_in_transport_mvit",\n  "PNUMOCCS": "number_of_occupants",\n  "PNUMOCCSNAME": "number_of_occupants_name",\n  "PDAY": "day_of_crash",\n  "PDAYNAME": "day_of_crash_name",\n  "PMONTH": "month_of_crash",\n  "PMONTHNAME": "month_of_crash_name",\n  "PHOUR": "hour_of_crash",\n  "PHOURNAME": "hour_of_crash_name",\n  "PMINUTE": "minute_of_crash",\n  "PMINUTENAME": "minute_of_crash_name",\n  "PHARM_EV": "first_harmful_event",\n  "PHARM_EVNAME": "first_harmful_event_name",\n  "PMAN_COLL": "manner_of_collision",\n  "PMAN_COLLNAME": "manner_of_collision_name",\n  "PTYPE": "unit_type",\n  "PTYPENAME": "unit_type_name",\n  "PHIT_RUN": "hit_and_run",\n  "PHIT_RUNNAME": "hit_and_run_name",\n  "PREG_STAT": "registration_state",\n  "PREG_STATNAME": "registration_state_name",\n  "POWNER": "registered_vehicle_owner",\n  "POWNERNAME": "registered_vehicle_owner_name",\n  "PMAKE": "vehicle_make",\n  "PMAKENAME": "vehicle_make_name",\n  "PMODEL": "vehicle_model",\n  "PMAK_MOD": "make_model_combined",\n  "PMAK_MODNAME": "make_model_combined_name",\n  "PBODYTYP": "body_type",\n  "PBODYTYPNAME": "body_type_name",\n  "PMODYEAR": "vehicle_model_year",\n  "PMODYEARNAME": "vehicle_model_year_name",\n  "PVIN": "vehicle_identification_number_vin",\n  "PVINNAME": "vehicle_identification_number_vin_name",\n  "PVIN_1": "vin_character_1",\n  "PVIN_2": "vin_character_2",\n  "PVIN_3": "vin_character_3",\n  "PVIN_4": "vin_character_4",\n  "PVIN_5": "vin_character_5",\n  "PVIN_6": "vin_character_6",\n  "PVIN_7": "vin_character_7",\n  "PVIN_8": "vin_character_8",\n  "PVIN_9": "vin_character_9",\n  "PVIN_10": "vin_character_10",\n  "PVIN_11": "vin_character_11",\n  "PVIN_12": "vin_character_12",\n  "PTRAILER": "vehicle_trailing",\n  "PTRAILERNAME": "vehicle_trailing_name",\n  "PMCARR_I1": "mcid_issuing_authority",\n  "PMCARR_I1NAME": "mcid_issuing_authority_name",\n  "PMCARR_I2": "mcid_identification_number",\n  "PMCARR_I2NAME": "mcid_identification_number_name",\n  "PMCARR_ID": "motor_carrier_identification_number",\n  "PMCARR_IDNAME": "motor_carrier_identification_number_name",\n  "PGVWR": "gross_vehicle_weight_rating",\n  "PGVWRNAME": "gross_vehicle_weight_rating_name",\n  "PV_CONFIG": "vehicle_configuration",\n  "PV_CONFIGNAME": "vehicle_configuration_name",\n  "PCARGTYP": "cargo_body_type",\n  "PCARGTYPNAME": "cargo_body_type_name",\n  "PHAZ_INV": "hazardous_material_involvement",\n  "PHAZ_INVNAME": "hazardous_material_involvement_name",\n  "PHAZPLAC": "hazardous_material_placard",\n  "PHAZPLACNAME": "hazardous_material_placard_name",\n  "PHAZ_ID": "hazardous_material_identification_number",\n  "PHAZ_IDNAME": "hazardous_material_identification_number_name",\n  "PHAZ_CNO": "hazardous_material_class_number",\n  "PHAZ_CNONAME": "hazardous_material_class_number_name",\n  "PHAZ_REL": "release_of_hazardous_material_from_the_cargo_compartment",\n  "PHAZ_RELNAME": "release_of_hazardous_material_from_the_cargo_compartment_name",\n  "PBUS_USE": "bus_use",\n  "PBUS_USENAME": "bus_use_name",\n  "PSP_USE": "special_use",\n  "PSP_USENAME": "special_use_name",\n  "PEM_USE": "emergency_motor_vehicle_use",\n  "PEM_USENAME": "emergency_motor_vehicle_use_name",\n  "PUNDERIDE": "underride_override",\n  "PUNDERIDENAME": "underride_override_name",\n  "PIMPACT1": "initial_contact_point",\n  "PIMPACT1NAME": "initial_contact_point_name",\n  "PVEH_SEV": "extent_of_damage",\n  "PVEH_SEVNAME": "extent_of_damage_name",\n  "PTOWED": "vehicle_removal",\n  "PTOWEDNAME": "vehicle_removal_name",\n  "PM_HARM": "most_harmful_event",\n  "PM_HARMNAME": "most_harmful_event_name",\n  "PVEH_SC1": "related_factors_vehicle_level1",\n  "PVEH_SC1NAME": "related_factors_vehicle_level1_name",\n  "PVEH_SC2": "related_factors_vehicle_level2",\n  "PVEH_SC2NAME": "related_factors_vehicle_level2_name",\n  "PFIRE": "fire_occurrence",\n  "PFIRENAME": "fire_occurrence_name",\n  "PDEATHS": "fatalities_in_vehicle",\n  "PTRLR1VIN": "ptrlr1vin",\n  "PTRLR1VINNAME": "ptrlr1vinname",\n  "PTRLR2VIN": "ptrlr2vin",\n  "PTRLR2VINNAME": "ptrlr2vinname",\n  "PTRLR3VIN": "ptrlr3vin",\n  "PTRLR3VINNAME": "ptrlr3vinname"\n}',
        },
        resources={
            "request_ephemeral_storage": "4G",
            "request_cpu": "1",
            "request_memory": "4G",
        },
    )

    # Run CSV transform within kubernetes pod for parkwork_2019 pipelines
    parkwork_2019_transform_csv = kubernetes_pod.KubernetesPodOperator(
        task_id="parkwork_2019_transform_csv",
        startup_timeout_seconds=600,
        name="parkwork_2019",
        namespace="composer",
        service_account_name="datasets",
        image_pull_policy="Always",
        image="{{ var.json.nhtsa_traffic_fatalities.container_registry.run_csv_transform_kub }}",
        env_vars={
            "PIPELINE_NAME": "{{ var.json.nhtsa_traffic_fatalities.parkwork_2019.pipeline_name }}",
            "SOURCE_URL": "{{ var.json.nhtsa_traffic_fatalities.parkwork_2019.source_url }}",
            "CHUNKSIZE": "{{ var.json.nhtsa_traffic_fatalities.parkwork_2019.chunksize }}",
            "SOURCE_ZIPFILE_EXTRACTED": "parkwork.csv",
            "SOURCE_FILE": "{{ var.json.nhtsa_traffic_fatalities.parkwork_2019.source_file }}",
            "PROJECT_ID": "{{ var.value.gcp_project }}",
            "DATASET_ID": "{{ var.json.nhtsa_traffic_fatalities.parkwork_2019.dataset_id }}",
            "TABLE_ID": "{{ var.json.nhtsa_traffic_fatalities.parkwork_2019.destination_table }}",
            "START_YEAR": "{{ var.json.nhtsa_traffic_fatalities.parkwork_2019.start_year }}",
            "END_YEAR": "{{ var.json.nhtsa_traffic_fatalities.parkwork_2019.end_year }}",
            "DROP_DEST_TABLE": "{{ var.json.nhtsa_traffic_fatalities.parkwork_2019.drop_dest_table }}",
            "TARGET_GCS_BUCKET": "{{ var.value.composer_bucket }}",
            "TARGET_GCS_PATH": "{{ var.json.nhtsa_traffic_fatalities.parkwork_2019.target_gcs_path }}",
            "SCHEMA_PATH": "{{ var.json.nhtsa_traffic_fatalities.parkwork_2019.schema_path }}",
            "INPUT_CSV_HEADERS": '[\n  "state_number",\n  "state_name",\n  "consecutive_number",\n  "vehicle_number",\n  "number_of_motor_vehicles_in_transport_mvit",\n  "number_of_occupants",\n  "number_of_occupants_name",\n  "day_of_crash",\n  "day_of_crash_name",\n  "month_of_crash",\n  "month_of_crash_name",\n  "hour_of_crash",\n  "hour_of_crash_name",\n  "minute_of_crash",\n  "minute_of_crash_name",\n  "first_harmful_event",\n  "first_harmful_event_name",\n  "manner_of_collision",\n  "manner_of_collision_name",\n  "unit_type",\n  "unit_type_name",\n  "hit_and_run",\n  "hit_and_run_name",\n  "registration_state",\n  "registration_state_name",\n  "registered_vehicle_owner",\n  "registered_vehicle_owner_name",\n  "vehicle_make",\n  "vehicle_make_name",\n  "vehicle_model",\n  "make_model_combined",\n  "body_type",\n  "body_type_name",\n  "vehicle_model_year",\n  "vehicle_model_year_name",\n  "vehicle_identification_number_vin",\n  "vehicle_identification_number_vin_name",\n  "vin_character_1",\n  "vin_character_2",\n  "vin_character_3",\n  "vin_character_4",\n  "vin_character_5",\n  "vin_character_6",\n  "vin_character_7",\n  "vin_character_8",\n  "vin_character_9",\n  "vin_character_10",\n  "vin_character_11",\n  "vin_character_12",\n  "vehicle_trailing",\n  "vehicle_trailing_name",\n  "mcid_issuing_authority",\n  "mcid_issuing_authority_name",\n  "mcid_identification_number",\n  "mcid_identification_number_name",\n  "motor_carrier_identification_number",\n  "motor_carrier_identification_number_name",\n  "gross_vehicle_weight_rating",\n  "gross_vehicle_weight_rating_name",\n  "vehicle_configuration",\n  "vehicle_configuration_name",\n  "cargo_body_type",\n  "cargo_body_type_name",\n  "hazardous_material_involvement",\n  "hazardous_material_involvement_name",\n  "hazardous_material_placard",\n  "hazardous_material_placard_name",\n  "hazardous_material_identification_number",\n  "hazardous_material_identification_number_name",\n  "hazardous_material_class_number",\n  "hazardous_material_class_number_name",\n  "release_of_hazardous_material_from_the_cargo_compartment",\n  "release_of_hazardous_material_from_the_cargo_compartment_name",\n  "bus_use",\n  "bus_use_name",\n  "special_use",\n  "special_use_name",\n  "emergency_motor_vehicle_use",\n  "emergency_motor_vehicle_use_name",\n  "underride_override",\n  "underride_override_name",\n  "initial_contact_point",\n  "initial_contact_point_name",\n  "extent_of_damage",\n  "extent_of_damage_name",\n  "vehicle_removal",\n  "vehicle_removal_name",\n  "most_harmful_event",\n  "most_harmful_event_name",\n  "related_factors_vehicle_level1",\n  "related_factors_vehicle_level1_name",\n  "related_factors_vehicle_level2",\n  "related_factors_vehicle_level2_name",\n  "fire_occurrence",\n  "fire_occurrence_name",\n  "fatalities_in_vehicle",\n  "ptrlr1vin",\n  "ptrlr1vinname",\n  "ptrlr2vin",\n  "ptrlr2vinname",\n  "ptrlr3vin",\n  "ptrlr3vinname"\n]',
            "INPUT_DTYPES": '{\n  "state_number": "str",\n  "state_name": "str",\n  "consecutive_number": "str",\n  "vehicle_number": "str",\n  "number_of_motor_vehicles_in_transport_mvit": "str",\n  "number_of_occupants": "str",\n  "number_of_occupants_name": "str",\n  "day_of_crash": "str",\n  "day_of_crash_name": "str",\n  "month_of_crash": "str",\n  "month_of_crash_name": "str",\n  "hour_of_crash": "str",\n  "hour_of_crash_name": "str",\n  "minute_of_crash": "str",\n  "minute_of_crash_name": "str",\n  "first_harmful_event": "str",\n  "first_harmful_event_name": "str",\n  "manner_of_collision": "str",\n  "manner_of_collision_name": "str",\n  "unit_type": "str",\n  "unit_type_name": "str",\n  "hit_and_run": "str",\n  "hit_and_run_name": "str",\n  "registration_state": "str",\n  "registration_state_name": "str",\n  "registered_vehicle_owner": "str",\n  "registered_vehicle_owner_name": "str",\n  "vehicle_make": "str",\n  "vehicle_make_name": "str",\n  "vehicle_model": "str",\n  "make_model_combined": "str",\n  "body_type": "str",\n  "body_type_name": "str",\n  "vehicle_model_year": "str",\n  "vehicle_model_year_name": "str",\n  "vehicle_identification_number_vin": "str",\n  "vehicle_identification_number_vin_name": "str",\n  "vin_character_1": "str",\n  "vin_character_2": "str",\n  "vin_character_3": "str",\n  "vin_character_4": "str",\n  "vin_character_5": "str",\n  "vin_character_6": "str",\n  "vin_character_7": "str",\n  "vin_character_8": "str",\n  "vin_character_9": "str",\n  "vin_character_10": "str",\n  "vin_character_11": "str",\n  "vin_character_12": "str",\n  "vehicle_trailing": "str",\n  "vehicle_trailing_name": "str",\n  "mcid_issuing_authority": "str",\n  "mcid_issuing_authority_name": "str",\n  "mcid_identification_number": "str",\n  "mcid_identification_number_name": "str",\n  "motor_carrier_identification_number": "str",\n  "motor_carrier_identification_number_name": "str",\n  "gross_vehicle_weight_rating": "str",\n  "gross_vehicle_weight_rating_name": "str",\n  "vehicle_configuration": "str",\n  "vehicle_configuration_name": "str",\n  "cargo_body_type": "str",\n  "cargo_body_type_name": "str",\n  "hazardous_material_involvement": "str",\n  "hazardous_material_involvement_name": "str",\n  "hazardous_material_placard": "str",\n  "hazardous_material_placard_name": "str",\n  "hazardous_material_identification_number": "str",\n  "hazardous_material_identification_number_name": "str",\n  "hazardous_material_class_number": "str",\n  "hazardous_material_class_number_name": "str",\n  "release_of_hazardous_material_from_the_cargo_compartment": "str",\n  "release_of_hazardous_material_from_the_cargo_compartment_name": "str",\n  "bus_use": "str",\n  "bus_use_name": "str",\n  "special_use": "str",\n  "special_use_name": "str",\n  "emergency_motor_vehicle_use": "str",\n  "emergency_motor_vehicle_use_name": "str",\n  "underride_override": "str",\n  "underride_override_name": "str",\n  "initial_contact_point": "str",\n  "initial_contact_point_name": "str",\n  "extent_of_damage": "str",\n  "extent_of_damage_name": "str",\n  "vehicle_removal": "str",\n  "vehicle_removal_name": "str",\n  "most_harmful_event": "str",\n  "most_harmful_event_name": "str",\n  "related_factors_vehicle_level1": "str",\n  "related_factors_vehicle_level1_name": "str",\n  "related_factors_vehicle_level2": "str",\n  "related_factors_vehicle_level2_name": "str",\n  "fire_occurrence": "str",\n  "fire_occurrence_name": "str",\n  "fatalities_in_vehicle": "str",\n  "ptrlr1vin": "str",\n  "ptrlr1vinname": "str",\n  "ptrlr2vin": "str",\n  "ptrlr2vinname": "str",\n  "ptrlr3vin": "str",\n  "ptrlr3vinname": "str"\n}',
            "RENAME_MAPPINGS_LIST": '{\n  "STATE": "state_number",\n  "STATENAME": "state_name",\n  "ST_CASE": "consecutive_number",\n  "VEH_NO": "vehicle_number",\n  "PVE_FORMS": "number_of_motor_vehicles_in_transport_mvit",\n  "PNUMOCCS": "number_of_occupants",\n  "PNUMOCCSNAME": "number_of_occupants_name",\n  "PDAY": "day_of_crash",\n  "PDAYNAME": "day_of_crash_name",\n  "PMONTH": "month_of_crash",\n  "PMONTHNAME": "month_of_crash_name",\n  "PHOUR": "hour_of_crash",\n  "PHOURNAME": "hour_of_crash_name",\n  "PMINUTE": "minute_of_crash",\n  "PMINUTENAME": "minute_of_crash_name",\n  "PHARM_EV": "first_harmful_event",\n  "PHARM_EVNAME": "first_harmful_event_name",\n  "PMAN_COLL": "manner_of_collision",\n  "PMAN_COLLNAME": "manner_of_collision_name",\n  "PTYPE": "unit_type",\n  "PTYPENAME": "unit_type_name",\n  "PHIT_RUN": "hit_and_run",\n  "PHIT_RUNNAME": "hit_and_run_name",\n  "PREG_STAT": "registration_state",\n  "PREG_STATNAME": "registration_state_name",\n  "POWNER": "registered_vehicle_owner",\n  "POWNERNAME": "registered_vehicle_owner_name",\n  "PMAKE": "vehicle_make",\n  "PMAKENAME": "vehicle_make_name",\n  "PMODEL": "vehicle_model",\n  "PMAK_MOD": "make_model_combined",\n  "PBODYTYP": "body_type",\n  "PBODYTYPNAME": "body_type_name",\n  "PMODYEAR": "vehicle_model_year",\n  "PMODYEARNAME": "vehicle_model_year_name",\n  "PVIN": "vehicle_identification_number_vin",\n  "PVINNAME": "vehicle_identification_number_vin_name",\n  "PVIN_1": "vin_character_1",\n  "PVIN_2": "vin_character_2",\n  "PVIN_3": "vin_character_3",\n  "PVIN_4": "vin_character_4",\n  "PVIN_5": "vin_character_5",\n  "PVIN_6": "vin_character_6",\n  "PVIN_7": "vin_character_7",\n  "PVIN_8": "vin_character_8",\n  "PVIN_9": "vin_character_9",\n  "PVIN_10": "vin_character_10",\n  "PVIN_11": "vin_character_11",\n  "PVIN_12": "vin_character_12",\n  "PTRAILER": "vehicle_trailing",\n  "PTRAILERNAME": "vehicle_trailing_name",\n  "PMCARR_I1": "mcid_issuing_authority",\n  "PMCARR_I1NAME": "mcid_issuing_authority_name",\n  "PMCARR_I2": "mcid_identification_number",\n  "PMCARR_I2NAME": "mcid_identification_number_name",\n  "PMCARR_ID": "motor_carrier_identification_number",\n  "PMCARR_IDNAME": "motor_carrier_identification_number_name",\n  "PGVWR": "gross_vehicle_weight_rating",\n  "PGVWRNAME": "gross_vehicle_weight_rating_name",\n  "PV_CONFIG": "vehicle_configuration",\n  "PV_CONFIGNAME": "vehicle_configuration_name",\n  "PCARGTYP": "cargo_body_type",\n  "PCARGTYPNAME": "cargo_body_type_name",\n  "PHAZ_INV": "hazardous_material_involvement",\n  "PHAZ_INVNAME": "hazardous_material_involvement_name",\n  "PHAZPLAC": "hazardous_material_placard",\n  "PHAZPLACNAME": "hazardous_material_placard_name",\n  "PHAZ_ID": "hazardous_material_identification_number",\n  "PHAZ_IDNAME": "hazardous_material_identification_number_name",\n  "PHAZ_CNO": "hazardous_material_class_number",\n  "PHAZ_CNONAME": "hazardous_material_class_number_name",\n  "PHAZ_REL": "release_of_hazardous_material_from_the_cargo_compartment",\n  "PHAZ_RELNAME": "release_of_hazardous_material_from_the_cargo_compartment_name",\n  "PBUS_USE": "bus_use",\n  "PBUS_USENAME": "bus_use_name",\n  "PSP_USE": "special_use",\n  "PSP_USENAME": "special_use_name",\n  "PEM_USE": "emergency_motor_vehicle_use",\n  "PEM_USENAME": "emergency_motor_vehicle_use_name",\n  "PUNDERIDE": "underride_override",\n  "PUNDERIDENAME": "underride_override_name",\n  "PIMPACT1": "initial_contact_point",\n  "PIMPACT1NAME": "initial_contact_point_name",\n  "PVEH_SEV": "extent_of_damage",\n  "PVEH_SEVNAME": "extent_of_damage_name",\n  "PTOWED": "vehicle_removal",\n  "PTOWEDNAME": "vehicle_removal_name",\n  "PM_HARM": "most_harmful_event",\n  "PM_HARMNAME": "most_harmful_event_name",\n  "PVEH_SC1": "related_factors_vehicle_level1",\n  "PVEH_SC1NAME": "related_factors_vehicle_level1_name",\n  "PVEH_SC2": "related_factors_vehicle_level2",\n  "PVEH_SC2NAME": "related_factors_vehicle_level2_name",\n  "PFIRE": "fire_occurrence",\n  "PFIRENAME": "fire_occurrence_name",\n  "PDEATHS": "fatalities_in_vehicle",\n  "PTRLR1VIN": "ptrlr1vin",\n  "PTRLR1VINNAME": "ptrlr1vinname",\n  "PTRLR2VIN": "ptrlr2vin",\n  "PTRLR2VINNAME": "ptrlr2vinname",\n  "PTRLR3VIN": "ptrlr3vin",\n  "PTRLR3VINNAME": "ptrlr3vinname"\n}',
        },
        resources={
            "request_ephemeral_storage": "4G",
            "request_cpu": "1",
            "request_memory": "4G",
        },
    )

    # Run CSV transform within kubernetes pod for parkwork_2020 pipelines
    parkwork_2020_transform_csv = kubernetes_pod.KubernetesPodOperator(
        task_id="parkwork_2020_transform_csv",
        startup_timeout_seconds=600,
        name="parkwork_2020",
        namespace="composer",
        service_account_name="datasets",
        image_pull_policy="Always",
        image="{{ var.json.nhtsa_traffic_fatalities.container_registry.run_csv_transform_kub }}",
        env_vars={
            "PIPELINE_NAME": "{{ var.json.nhtsa_traffic_fatalities.parkwork_2020.pipeline_name }}",
            "SOURCE_URL": "{{ var.json.nhtsa_traffic_fatalities.parkwork_2020.source_url }}",
            "CHUNKSIZE": "{{ var.json.nhtsa_traffic_fatalities.parkwork_2020.chunksize }}",
            "SOURCE_ZIPFILE_EXTRACTED": "parkwork.csv",
            "SOURCE_FILE": "{{ var.json.nhtsa_traffic_fatalities.parkwork_2020.source_file }}",
            "PROJECT_ID": "{{ var.value.gcp_project }}",
            "DATASET_ID": "{{ var.json.nhtsa_traffic_fatalities.parkwork_2020.dataset_id }}",
            "TABLE_ID": "{{ var.json.nhtsa_traffic_fatalities.parkwork_2020.destination_table }}",
            "START_YEAR": "{{ var.json.nhtsa_traffic_fatalities.parkwork_2020.start_year }}",
            "END_YEAR": "{{ var.json.nhtsa_traffic_fatalities.parkwork_2020.end_year }}",
            "DROP_DEST_TABLE": "{{ var.json.nhtsa_traffic_fatalities.parkwork_2020.drop_dest_table }}",
            "TARGET_GCS_BUCKET": "{{ var.value.composer_bucket }}",
            "TARGET_GCS_PATH": "{{ var.json.nhtsa_traffic_fatalities.parkwork_2020.target_gcs_path }}",
            "SCHEMA_PATH": "{{ var.json.nhtsa_traffic_fatalities.parkwork_2020.schema_path }}",
            "INPUT_CSV_HEADERS": '[\n  "state_number",\n  "state_name",\n  "consecutive_number",\n  "vehicle_number",\n  "number_of_motor_vehicles_in_transport_mvit",\n  "number_of_occupants",\n  "number_of_occupants_name",\n  "day_of_crash",\n  "day_of_crash_name",\n  "month_of_crash",\n  "month_of_crash_name",\n  "hour_of_crash",\n  "hour_of_crash_name",\n  "minute_of_crash",\n  "minute_of_crash_name",\n  "first_harmful_event",\n  "first_harmful_event_name",\n  "manner_of_collision",\n  "manner_of_collision_name",\n  "unit_type",\n  "unit_type_name",\n  "hit_and_run",\n  "hit_and_run_name",\n  "registration_state",\n  "registration_state_name",\n  "registered_vehicle_owner",\n  "registered_vehicle_owner_name",\n  "vehicle_make",\n  "vehicle_make_name",\n  "vehicle_model",\n  "make_model_combined",\n  "body_type",\n  "body_type_name",\n  "vehicle_model_year",\n  "vehicle_model_year_name",\n  "vehicle_identification_number_vin",\n  "vehicle_identification_number_vin_name",\n  "vin_character_1",\n  "vin_character_2",\n  "vin_character_3",\n  "vin_character_4",\n  "vin_character_5",\n  "vin_character_6",\n  "vin_character_7",\n  "vin_character_8",\n  "vin_character_9",\n  "vin_character_10",\n  "vin_character_11",\n  "vin_character_12",\n  "vehicle_trailing",\n  "vehicle_trailing_name",\n  "mcid_issuing_authority",\n  "mcid_issuing_authority_name",\n  "mcid_identification_number",\n  "mcid_identification_number_name",\n  "motor_carrier_identification_number",\n  "motor_carrier_identification_number_name",\n  "vehicle_configuration",\n  "vehicle_configuration_name",\n  "cargo_body_type",\n  "cargo_body_type_name",\n  "hazardous_material_involvement",\n  "hazardous_material_involvement_name",\n  "hazardous_material_placard",\n  "hazardous_material_placard_name",\n  "hazardous_material_identification_number",\n  "hazardous_material_identification_number_name",\n  "hazardous_material_class_number",\n  "hazardous_material_class_number_name",\n  "release_of_hazardous_material_from_the_cargo_compartment",\n  "release_of_hazardous_material_from_the_cargo_compartment_name",\n  "bus_use",\n  "bus_use_name",\n  "special_use",\n  "special_use_name",\n  "emergency_motor_vehicle_use",\n  "emergency_motor_vehicle_use_name",\n  "underride_override",\n  "underride_override_name",\n  "initial_contact_point",\n  "initial_contact_point_name",\n  "extent_of_damage",\n  "extent_of_damage_name",\n  "vehicle_removal",\n  "vehicle_removal_name",\n  "most_harmful_event",\n  "most_harmful_event_name",\n  "fire_occurrence",\n  "fire_occurrence_name",\n  "fatalities_in_vehicle",\n  "ptrlr1vin",\n  "ptrlr1vinname",\n  "ptrlr2vin",\n  "ptrlr2vinname",\n  "ptrlr3vin",\n  "ptrlr3vinname",\n  "pvpicmake",\n  "pvpicmakename",\n  "pvpicmodel",\n  "pvpicmodelname",\n  "pvpicbodyclass",\n  "pvpicbodyclassname",\n  "picfinalbody",\n  "picfinalbodyname",\n  "pgvwr_from",\n  "pgvwr_fromname",\n  "pgvwr_to",\n  "pgvwr_toname",\n  "ptrlr1gvwr",\n  "ptrlr1gvwrname",\n  "ptrlr2gvwr",\n  "ptrlr2gvwrname",\n  "ptrlr3gvwr",\n  "ptrlr3gvwrname"\n]',
            "INPUT_DTYPES": '{\n  "state_number": "str",\n  "state_name": "str",\n  "consecutive_number": "str",\n  "vehicle_number": "str",\n  "number_of_motor_vehicles_in_transport_mvit": "str",\n  "number_of_occupants": "str",\n  "number_of_occupants_name": "str",\n  "day_of_crash": "str",\n  "day_of_crash_name": "str",\n  "month_of_crash": "str",\n  "month_of_crash_name": "str",\n  "hour_of_crash": "str",\n  "hour_of_crash_name": "str",\n  "minute_of_crash": "str",\n  "minute_of_crash_name": "str",\n  "first_harmful_event": "str",\n  "first_harmful_event_name": "str",\n  "manner_of_collision": "str",\n  "manner_of_collision_name": "str",\n  "unit_type": "str",\n  "unit_type_name": "str",\n  "hit_and_run": "str",\n  "hit_and_run_name": "str",\n  "registration_state": "str",\n  "registration_state_name": "str",\n  "registered_vehicle_owner": "str",\n  "registered_vehicle_owner_name": "str",\n  "vehicle_make": "str",\n  "vehicle_make_name": "str",\n  "vehicle_model": "str",\n  "make_model_combined": "str",\n  "body_type": "str",\n  "body_type_name": "str",\n  "vehicle_model_year": "str",\n  "vehicle_model_year_name": "str",\n  "vehicle_identification_number_vin": "str",\n  "vehicle_identification_number_vin_name": "str",\n  "vin_character_1": "str",\n  "vin_character_2": "str",\n  "vin_character_3": "str",\n  "vin_character_4": "str",\n  "vin_character_5": "str",\n  "vin_character_6": "str",\n  "vin_character_7": "str",\n  "vin_character_8": "str",\n  "vin_character_9": "str",\n  "vin_character_10": "str",\n  "vin_character_11": "str",\n  "vin_character_12": "str",\n  "vehicle_trailing": "str",\n  "vehicle_trailing_name": "str",\n  "mcid_issuing_authority": "str",\n  "mcid_issuing_authority_name": "str",\n  "mcid_identification_number": "str",\n  "mcid_identification_number_name": "str",\n  "motor_carrier_identification_number": "str",\n  "motor_carrier_identification_number_name": "str",\n  "vehicle_configuration": "str",\n  "vehicle_configuration_name": "str",\n  "cargo_body_type": "str",\n  "cargo_body_type_name": "str",\n  "hazardous_material_involvement": "str",\n  "hazardous_material_involvement_name": "str",\n  "hazardous_material_placard": "str",\n  "hazardous_material_placard_name": "str",\n  "hazardous_material_identification_number": "str",\n  "hazardous_material_identification_number_name": "str",\n  "hazardous_material_class_number": "str",\n  "hazardous_material_class_number_name": "str",\n  "release_of_hazardous_material_from_the_cargo_compartment": "str",\n  "release_of_hazardous_material_from_the_cargo_compartment_name": "str",\n  "bus_use": "str",\n  "bus_use_name": "str",\n  "special_use": "str",\n  "special_use_name": "str",\n  "emergency_motor_vehicle_use": "str",\n  "emergency_motor_vehicle_use_name": "str",\n  "underride_override": "str",\n  "underride_override_name": "str",\n  "initial_contact_point": "str",\n  "initial_contact_point_name": "str",\n  "extent_of_damage": "str",\n  "extent_of_damage_name": "str",\n  "vehicle_removal": "str",\n  "vehicle_removal_name": "str",\n  "most_harmful_event": "str",\n  "most_harmful_event_name": "str",\n  "fire_occurrence": "str",\n  "fire_occurrence_name": "str",\n  "fatalities_in_vehicle": "str",\n  "ptrlr1vin": "str",\n  "ptrlr1vinname": "str",\n  "ptrlr2vin": "str",\n  "ptrlr2vinname": "str",\n  "ptrlr3vin": "str",\n  "ptrlr3vinname": "str",\n  "pvpicmake": "str",\n  "pvpicmakename": "str",\n  "pvpicmodel": "str",\n  "pvpicmodelname": "str",\n  "pvpicbodyclass": "str",\n  "pvpicbodyclassname": "str",\n  "picfinalbody": "str",\n  "picfinalbodyname": "str",\n  "pgvwr_from": "str",\n  "pgvwr_fromname": "str",\n  "pgvwr_to": "str",\n  "pgvwr_toname": "str",\n  "ptrlr1gvwr": "str",\n  "ptrlr1gvwrname": "str",\n  "ptrlr2gvwr": "str",\n  "ptrlr2gvwrname": "str",\n  "ptrlr3gvwr": "str",\n  "ptrlr3gvwrname": "str"\n}',
            "RENAME_MAPPINGS_LIST": '{\n  "STATE": "state_number",\n  "STATENAME": "state_name",\n  "ST_CASE": "consecutive_number",\n  "VEH_NO": "vehicle_number",\n  "PVE_FORMS": "number_of_motor_vehicles_in_transport_mvit",\n  "PNUMOCCS": "number_of_occupants",\n  "PNUMOCCSNAME": "number_of_occupants_name",\n  "PDAY": "day_of_crash",\n  "PDAYNAME": "day_of_crash_name",\n  "PMONTH": "month_of_crash",\n  "PMONTHNAME": "month_of_crash_name",\n  "PHOUR": "hour_of_crash",\n  "PHOURNAME": "hour_of_crash_name",\n  "PMINUTE": "minute_of_crash",\n  "PMINUTENAME": "minute_of_crash_name",\n  "PHARM_EV": "first_harmful_event",\n  "PHARM_EVNAME": "first_harmful_event_name",\n  "PMAN_COLL": "manner_of_collision",\n  "PMAN_COLLNAME": "manner_of_collision_name",\n  "PTYPE": "unit_type",\n  "PTYPENAME": "unit_type_name",\n  "PHIT_RUN": "hit_and_run",\n  "PHIT_RUNNAME": "hit_and_run_name",\n  "PREG_STAT": "registration_state",\n  "PREG_STATNAME": "registration_state_name",\n  "POWNER": "registered_vehicle_owner",\n  "POWNERNAME": "registered_vehicle_owner_name",\n  "PMAKE": "vehicle_make",\n  "PMAKENAME": "vehicle_make_name",\n  "PMODEL": "vehicle_model",\n  "PMAK_MOD": "make_model_combined",\n  "PBODYTYP": "body_type",\n  "PBODYTYPNAME": "body_type_name",\n  "PMODYEAR": "vehicle_model_year",\n  "PMODYEARNAME": "vehicle_model_year_name",\n  "PVIN": "vehicle_identification_number_vin",\n  "PVINNAME": "vehicle_identification_number_vin_name",\n  "PVIN_1": "vin_character_1",\n  "PVIN_2": "vin_character_2",\n  "PVIN_3": "vin_character_3",\n  "PVIN_4": "vin_character_4",\n  "PVIN_5": "vin_character_5",\n  "PVIN_6": "vin_character_6",\n  "PVIN_7": "vin_character_7",\n  "PVIN_8": "vin_character_8",\n  "PVIN_9": "vin_character_9",\n  "PVIN_10": "vin_character_10",\n  "PVIN_11": "vin_character_11",\n  "PVIN_12": "vin_character_12",\n  "PTRAILER": "vehicle_trailing",\n  "PTRAILERNAME": "vehicle_trailing_name",\n  "PMCARR_I1": "mcid_issuing_authority",\n  "PMCARR_I1NAME": "mcid_issuing_authority_name",\n  "PMCARR_I2": "mcid_identification_number",\n  "PMCARR_I2NAME": "mcid_identification_number_name",\n  "PMCARR_ID": "motor_carrier_identification_number",\n  "PMCARR_IDNAME": "motor_carrier_identification_number_name",\n  "PV_CONFIG": "vehicle_configuration",\n  "PV_CONFIGNAME": "vehicle_configuration_name",\n  "PCARGTYP": "cargo_body_type",\n  "PCARGTYPNAME": "cargo_body_type_name",\n  "PHAZ_INV": "hazardous_material_involvement",\n  "PHAZ_INVNAME": "hazardous_material_involvement_name",\n  "PHAZPLAC": "hazardous_material_placard",\n  "PHAZPLACNAME": "hazardous_material_placard_name",\n  "PHAZ_ID": "hazardous_material_identification_number",\n  "PHAZ_IDNAME": "hazardous_material_identification_number_name",\n  "PHAZ_CNO": "hazardous_material_class_number",\n  "PHAZ_CNONAME": "hazardous_material_class_number_name",\n  "PHAZ_REL": "release_of_hazardous_material_from_the_cargo_compartment",\n  "PHAZ_RELNAME": "release_of_hazardous_material_from_the_cargo_compartment_name",\n  "PBUS_USE": "bus_use",\n  "PBUS_USENAME": "bus_use_name",\n  "PSP_USE": "special_use",\n  "PSP_USENAME": "special_use_name",\n  "PEM_USE": "emergency_motor_vehicle_use",\n  "PEM_USENAME": "emergency_motor_vehicle_use_name",\n  "PUNDERIDE": "underride_override",\n  "PUNDERIDENAME": "underride_override_name",\n  "PIMPACT1": "initial_contact_point",\n  "PIMPACT1NAME": "initial_contact_point_name",\n  "PVEH_SEV": "extent_of_damage",\n  "PVEH_SEVNAME": "extent_of_damage_name",\n  "PTOWED": "vehicle_removal",\n  "PTOWEDNAME": "vehicle_removal_name",\n  "PM_HARM": "most_harmful_event",\n  "PM_HARMNAME": "most_harmful_event_name",\n  "PFIRE": "fire_occurrence",\n  "PFIRENAME": "fire_occurrence_name",\n  "PDEATHS": "fatalities_in_vehicle",\n  "PTRLR1VIN": "ptrlr1vin",\n  "PTRLR1VINNAME": "ptrlr1vinname",\n  "PTRLR2VIN": "ptrlr2vin",\n  "PTRLR2VINNAME": "ptrlr2vinname",\n  "PTRLR3VIN": "ptrlr3vin",\n  "PTRLR3VINNAME": "ptrlr3vinname",\n  "PVPICMAKE": "pvpicmake",\n  "PVPICMAKENAME": "pvpicmakename",\n  "PVPICMODEL": "pvpicmodel",\n  "PVPICMODELNAME": "pvpicmodelname",\n  "PVPICBODYCLASS": "pvpicbodyclass",\n  "PVPICBODYCLASSNAME": "pvpicbodyclassname",\n  "PICFINALBODY": "picfinalbody",\n  "PICFINALBODYNAME": "picfinalbodyname",\n  "PGVWR_FROM": "pgvwr_from",\n  "PGVWR_FROMNAME": "pgvwr_fromname",\n  "PGVWR_TO": "pgvwr_to",\n  "PGVWR_TONAME": "pgvwr_toname",\n  "PTRLR1GVWR": "ptrlr1gvwr",\n  "PTRLR1GVWRNAME": "ptrlr1gvwrname",\n  "PTRLR2GVWR": "ptrlr2gvwr",\n  "PTRLR2GVWRNAME": "ptrlr2gvwrname",\n  "PTRLR3GVWR": "ptrlr3gvwr",\n  "PTRLR3GVWRNAME": "ptrlr3gvwrname"\n}',
        },
        resources={
            "request_ephemeral_storage": "4G",
            "request_cpu": "1",
            "request_memory": "4G",
        },
    )

    # Run CSV transform within kubernetes pod for pbtype pipelines
    pbtype_transform_csv = kubernetes_pod.KubernetesPodOperator(
        task_id="pbtype_transform_csv",
        startup_timeout_seconds=600,
        name="pbtype",
        namespace="composer",
        service_account_name="datasets",
        image_pull_policy="Always",
        image="{{ var.json.nhtsa_traffic_fatalities.container_registry.run_csv_transform_kub }}",
        env_vars={
            "PIPELINE_NAME": "{{ var.json.nhtsa_traffic_fatalities.pbtype_2015_2020.pipeline_name }}",
            "SOURCE_URL": "{{ var.json.nhtsa_traffic_fatalities.pbtype_2015_2020.source_url }}",
            "CHUNKSIZE": "{{ var.json.nhtsa_traffic_fatalities.pbtype_2015_2020.chunksize }}",
            "SOURCE_ZIPFILE_EXTRACTED": "pbtype.csv",
            "SOURCE_FILE": "{{ var.json.nhtsa_traffic_fatalities.pbtype_2015_2020.source_file }}",
            "PROJECT_ID": "{{ var.value.gcp_project }}",
            "DATASET_ID": "{{ var.json.nhtsa_traffic_fatalities.pbtype_2015_2020.dataset_id }}",
            "TABLE_ID": "{{ var.json.nhtsa_traffic_fatalities.pbtype_2015_2020.destination_table }}",
            "START_YEAR": "{{ var.json.nhtsa_traffic_fatalities.pbtype_2015_2020.start_year }}",
            "END_YEAR": "{{ var.json.nhtsa_traffic_fatalities.pbtype_2015_2020.end_year }}",
            "DROP_DEST_TABLE": "{{ var.json.nhtsa_traffic_fatalities.pbtype_2015_2020.drop_dest_table }}",
            "TARGET_GCS_BUCKET": "{{ var.value.composer_bucket }}",
            "TARGET_GCS_PATH": "{{ var.json.nhtsa_traffic_fatalities.pbtype_2015_2020.target_gcs_path }}",
            "SCHEMA_PATH": "{{ var.json.nhtsa_traffic_fatalities.pbtype_2015_2020.schema_path }}",
            "INPUT_CSV_HEADERS": '[\n  "state_number",\n  "state_name",\n  "consecutive_number",\n  "vehicle_number",\n  "person_number",\n  "person_type",\n  "person_type_name",\n  "age",\n  "age_name",\n  "sex",\n  "sex_name",\n  "marked_crosswalk_present",\n  "marked_crosswalk_present_name",\n  "sidewalk_present",\n  "sidewalk_present_name",\n  "school_zone",\n  "school_zone_name",\n  "crash_type_pedestrian",\n  "crash_type_pedestrian_name",\n  "crash_type_bicycle",\n  "crash_type_bicycle_name",\n  "crash_location_pedestrian",\n  "crash_location_pedestrian_name",\n  "crash_location_bicycle",\n  "crash_location_bicycle_name",\n  "pedestrian_position",\n  "pedestrian_position_name",\n  "bicyclist_position",\n  "bicyclist_position_name",\n  "pedestrian_initial_direction_of_travel",\n  "pedestrian_initial_direction_of_travel_name",\n  "bicyclist_initial_direction_of_travel",\n  "bicyclist_initial_direction_of_travel_name",\n  "motorist_initial_direction_of_travel",\n  "motorist_initial_direction_of_travel_name",\n  "motorist_maneuver",\n  "motorist_maneuver_name",\n  "intersection_leg",\n  "intersection_leg_name",\n  "pedestrian_scenario",\n  "pedestrian_scenario_name",\n  "crash_group_pedestrian",\n  "crash_group_pedestrian_name",\n  "crash_group_bicycle",\n  "crash_group_bicycle_name"\n]',
            "INPUT_DTYPES": '{\n  "state_number": "str",\n  "state_name": "str",\n  "consecutive_number": "str",\n  "vehicle_number": "str",\n  "person_number": "str",\n  "person_type": "str",\n  "person_type_name": "str",\n  "age": "str",\n  "age_name": "str",\n  "sex": "str",\n  "sex_name": "str",\n  "marked_crosswalk_present": "str",\n  "marked_crosswalk_present_name": "str",\n  "sidewalk_present": "str",\n  "sidewalk_present_name": "str",\n  "school_zone": "str",\n  "school_zone_name": "str",\n  "crash_type_pedestrian": "str",\n  "crash_type_pedestrian_name": "str",\n  "crash_type_bicycle": "str",\n  "crash_type_bicycle_name": "str",\n  "crash_location_pedestrian": "str",\n  "crash_location_pedestrian_name": "str",\n  "crash_location_bicycle": "str",\n  "crash_location_bicycle_name": "str",\n  "pedestrian_position": "str",\n  "pedestrian_position_name": "str",\n  "bicyclist_position": "str",\n  "bicyclist_position_name": "str",\n  "pedestrian_initial_direction_of_travel": "str",\n  "pedestrian_initial_direction_of_travel_name": "str",\n  "bicyclist_initial_direction_of_travel": "str",\n  "bicyclist_initial_direction_of_travel_name": "str",\n  "motorist_initial_direction_of_travel": "str",\n  "motorist_initial_direction_of_travel_name": "str",\n  "motorist_maneuver": "str",\n  "motorist_maneuver_name": "str",\n  "intersection_leg": "str",\n  "intersection_leg_name": "str",\n  "pedestrian_scenario": "str",\n  "pedestrian_scenario_name": "str",\n  "crash_group_pedestrian": "str",\n  "crash_group_pedestrian_name": "str",\n  "crash_group_bicycle": "str",\n  "crash_group_bicycle_name": "str"\n}',
            "RENAME_MAPPINGS_LIST": '{\n  "STATE": "state_number",\n  "STATENAME": "state_name",\n  "ST_CASE": "consecutive_number",\n  "VEH_NO": "vehicle_number",\n  "PER_NO": "person_number",\n  "PBPTYPE": "person_type",\n  "PBPTYPENAME": "person_type_name",\n  "PBAGE": "age",\n  "PBAGENAME": "age_name",\n  "PBSEX": "sex",\n  "PBSEXNAME": "sex_name",\n  "PBCWALK": "marked_crosswalk_present",\n  "PBCWALKNAME": "marked_crosswalk_present_name",\n  "PBSWALK": "sidewalk_present",\n  "PBSWALKNAME": "sidewalk_present_name",\n  "PBSZONE": "school_zone",\n  "PBSZONENAME": "school_zone_name",\n  "PEDCTYPE": "crash_type_pedestrian",\n  "PEDCTYPENAME": "crash_type_pedestrian_name",\n  "BIKECTYPE": "crash_type_bicycle",\n  "BIKECTYPENAME": "crash_type_bicycle_name",\n  "PEDLOC": "crash_location_pedestrian",\n  "PEDLOCNAME": "crash_location_pedestrian_name",\n  "BIKELOC": "crash_location_bicycle",\n  "BIKELOCNAME": "crash_location_bicycle_name",\n  "PEDPOS": "pedestrian_position",\n  "PEDPOSNAME": "pedestrian_position_name",\n  "BIKEPOS": "bicyclist_position",\n  "BIKEPOSNAME": "bicyclist_position_name",\n  "PEDDIR": "pedestrian_initial_direction_of_travel",\n  "PEDDIRNAME": "pedestrian_initial_direction_of_travel_name",\n  "BIKEDIR": "bicyclist_initial_direction_of_travel",\n  "BIKEDIRNAME": "bicyclist_initial_direction_of_travel_name",\n  "MOTDIR": "motorist_initial_direction_of_travel",\n  "MOTDIRNAME": "motorist_initial_direction_of_travel_name",\n  "MOTMAN": "motorist_maneuver",\n  "MOTMANNAME": "motorist_maneuver_name",\n  "PEDLEG": "intersection_leg",\n  "PEDLEGNAME": "intersection_leg_name",\n  "PEDSNR": "pedestrian_scenario",\n  "PEDSNRNAME": "pedestrian_scenario_name",\n  "PEDCGP": "crash_group_pedestrian",\n  "PEDCGPNAME": "crash_group_pedestrian_name",\n  "BIKECGP": "crash_group_bicycle",\n  "BIKECGPNAME": "crash_group_bicycle_name"\n}',
        },
        resources={
            "request_ephemeral_storage": "4G",
            "request_cpu": "1",
            "request_memory": "4G",
        },
    )

    # Run CSV transform within kubernetes pod for person pipelines
    person_2015_2017_transform_csv = kubernetes_pod.KubernetesPodOperator(
        task_id="person_2015_2017_transform_csv",
        startup_timeout_seconds=600,
        name="person",
        namespace="composer",
        service_account_name="datasets",
        image_pull_policy="Always",
        image="{{ var.json.nhtsa_traffic_fatalities.container_registry.run_csv_transform_kub }}",
        env_vars={
            "PIPELINE_NAME": "{{ var.json.nhtsa_traffic_fatalities.person_2015_2017.pipeline_name }}",
            "SOURCE_URL": "{{ var.json.nhtsa_traffic_fatalities.person_2015_2017.source_url }}",
            "CHUNKSIZE": "{{ var.json.nhtsa_traffic_fatalities.person_2015_2017.chunksize }}",
            "SOURCE_ZIPFILE_EXTRACTED": "person.csv",
            "SOURCE_FILE": "{{ var.json.nhtsa_traffic_fatalities.person_2015_2017.source_file }}",
            "PROJECT_ID": "{{ var.value.gcp_project }}",
            "DATASET_ID": "{{ var.json.nhtsa_traffic_fatalities.person_2015_2017.dataset_id }}",
            "TABLE_ID": "{{ var.json.nhtsa_traffic_fatalities.person_2015_2017.destination_table }}",
            "START_YEAR": "{{ var.json.nhtsa_traffic_fatalities.person_2015_2017.start_year }}",
            "END_YEAR": "{{ var.json.nhtsa_traffic_fatalities.person_2015_2017.end_year }}",
            "DROP_DEST_TABLE": "{{ var.json.nhtsa_traffic_fatalities.person_2015_2017.drop_dest_table }}",
            "TARGET_GCS_BUCKET": "{{ var.value.composer_bucket }}",
            "TARGET_GCS_PATH": "{{ var.json.nhtsa_traffic_fatalities.person_2015_2017.target_gcs_path }}",
            "SCHEMA_PATH": "{{ var.json.nhtsa_traffic_fatalities.person_2015_2017.schema_path }}",
            "INPUT_CSV_HEADERS": '[\n  "state_number",\n  "state_name",\n  "consecutive_number",\n  "number_of_motor_vehicles_in_transport_mvit",\n  "vehicle_number",\n  "person_number",\n  "number_of_motor_vehicle_striking_non_motorist",\n  "county",\n  "day_of_crash",\n  "day_of_crash_name",\n  "month_of_crash",\n  "month_of_crash_name",\n  "hour_of_crash",\n  "hour_of_crash_name",\n  "minute_of_crash",\n  "minute_of_crash_name",\n  "land_use",\n  "land_use_name",\n  "functional_system",\n  "functional_system_name",\n  "first_harmful_event",\n  "first_harmful_event_name",\n  "manner_of_collision",\n  "manner_of_collision_name",\n  "school_bus_related",\n  "school_bus_related_name",\n  "vehicle_make",\n  "vehicle_make_name",\n  "make_model_combined",\n  "body_type",\n  "body_type_name",\n  "model_year",\n  "model_year_name",\n  "vehicle_trailing",\n  "vehicle_trailing_name",\n  "special_use",\n  "special_use_name",\n  "emergency_motor_vehicle_use",\n  "emergency_motor_vehicle_use_name",\n  "rollover",\n  "rollover_name",\n  "initial_contact_point",\n  "initial_contact_point_name",\n  "fire_occurrence",\n  "fire_occurrence_name",\n  "age",\n  "age_name",\n  "sex",\n  "sex_name",\n  "person_type",\n  "person_type_name",\n  "injury_severity",\n  "injury_severity_name",\n  "seating_position",\n  "seating_position_name",\n  "restraint_system_helmet_use",\n  "restraint_system_helmet_use_name",\n  "indication_of_misuse_of_restraint_system_helmet",\n  "indication_of_misuse_of_restraint_system_helmet_name",\n  "air_bag_deployed",\n  "air_bag_deployed_name",\n  "ejection",\n  "ejection_name",\n  "ejection_path",\n  "ejection_path_name",\n  "extrication",\n  "extrication_name",\n  "police_reported_alcohol_involvement",\n  "police_reported_alcohol_involvement_name",\n  "method_of_alcohol_determination_by_police",\n  "method_of_alcohol_determination_by_police_name",\n  "alcohol_test_status",\n  "alcohol_test_status_name",\n  "alcohol_test_type",\n  "alcohol_test_type_name",\n  "alcohol_result",\n  "alcohol_result_name",\n  "police_reported_drug_involvement",\n  "police_reported_drug_involvement_name",\n  "method_of_drug_determination_by_police",\n  "method_of_drug_determination_by_police_name",\n  "drug_test_status",\n  "drug_test_status_name",\n  "drug_test_type1",\n  "drug_test_type1_name",\n  "drug_test_type2",\n  "drug_test_type2_name",\n  "drug_test_type3",\n  "drug_test_type3_name",\n  "drug_result1",\n  "drug_result1_name",\n  "drug_result2",\n  "drug_result2_name",\n  "drug_result3",\n  "drug_result3_name",\n  "hospital",\n  "hospital_name",\n  "died_at_scene_en_route",\n  "died_at_scene_en_route_name",\n  "day_of_death",\n  "day_of_death_name",\n  "month_of_death",\n  "month_of_death_name",\n  "year_of_death",\n  "year_of_death_name",\n  "hour_of_death",\n  "hour_of_death_name",\n  "minute_of_death",\n  "minute_of_death_name",\n  "death_time",\n  "death_time_name",\n  "lag_hours",\n  "lag_hours_name",\n  "lag_minutes",\n  "lag_minutes_name",\n  "related_factors_person_level1",\n  "related_factors_person_level1_name",\n  "related_factors_person_level2",\n  "related_factors_person_level2_name",\n  "related_factors_person_level3",\n  "related_factors_person_level3_name",\n  "fatal_injury_at_work",\n  "fatal_injury_at_work_name",\n  "hispanic_origin",\n  "hispanic_origin_name",\n  "race",\n  "race_name",\n  "non_motorist_location_at_time_of_crash",\n  "non_motorist_location_at_time_of_crash_name"\n]',
            "INPUT_DTYPES": '{\n  "state_number": "str",\n  "state_name": "str",\n  "consecutive_number": "str",\n  "number_of_motor_vehicles_in_transport_mvit": "str",\n  "vehicle_number": "str",\n  "person_number": "str",\n  "number_of_motor_vehicle_striking_non_motorist": "str",\n  "county": "str",\n  "day_of_crash": "str",\n  "day_of_crash_name": "str",\n  "month_of_crash": "str",\n  "month_of_crash_name": "str",\n  "hour_of_crash": "str",\n  "hour_of_crash_name": "str",\n  "minute_of_crash": "str",\n  "minute_of_crash_name": "str",\n  "land_use": "str",\n  "land_use_name": "str",\n  "functional_system": "str",\n  "functional_system_name": "str",\n  "first_harmful_event": "str",\n  "first_harmful_event_name": "str",\n  "manner_of_collision": "str",\n  "manner_of_collision_name": "str",\n  "school_bus_related": "str",\n  "school_bus_related_name": "str",\n  "vehicle_make": "str",\n  "vehicle_make_name": "str",\n  "make_model_combined": "str",\n  "body_type": "str",\n  "body_type_name": "str",\n  "model_year": "str",\n  "model_year_name": "str",\n  "vehicle_trailing": "str",\n  "vehicle_trailing_name": "str",\n  "special_use": "str",\n  "special_use_name": "str",\n  "emergency_motor_vehicle_use": "str",\n  "emergency_motor_vehicle_use_name": "str",\n  "rollover": "str",\n  "rollover_name": "str",\n  "initial_contact_point": "str",\n  "initial_contact_point_name": "str",\n  "fire_occurrence": "str",\n  "fire_occurrence_name": "str",\n  "age": "str",\n  "age_name": "str",\n  "sex": "str",\n  "sex_name": "str",\n  "person_type": "str",\n  "person_type_name": "str",\n  "injury_severity": "str",\n  "injury_severity_name": "str",\n  "seating_position": "str",\n  "seating_position_name": "str",\n  "restraint_system_helmet_use": "str",\n  "restraint_system_helmet_use_name": "str",\n  "indication_of_misuse_of_restraint_system_helmet": "str",\n  "indication_of_misuse_of_restraint_system_helmet_name": "str",\n  "air_bag_deployed": "str",\n  "air_bag_deployed_name": "str",\n  "ejection": "str",\n  "ejection_name": "str",\n  "ejection_path": "str",\n  "ejection_path_name": "str",\n  "extrication": "str",\n  "extrication_name": "str",\n  "police_reported_alcohol_involvement": "str",\n  "police_reported_alcohol_involvement_name": "str",\n  "method_of_alcohol_determination_by_police": "str",\n  "method_of_alcohol_determination_by_police_name": "str",\n  "alcohol_test_status": "str",\n  "alcohol_test_status_name": "str",\n  "alcohol_test_type": "str",\n  "alcohol_test_type_name": "str",\n  "alcohol_result": "str",\n  "alcohol_result_name": "str",\n  "police_reported_drug_involvement": "str",\n  "police_reported_drug_involvement_name": "str",\n  "method_of_drug_determination_by_police": "str",\n  "method_of_drug_determination_by_police_name": "str",\n  "drug_test_status": "str",\n  "drug_test_status_name": "str",\n  "drug_test_type1": "str",\n  "drug_test_type1_name": "str",\n  "drug_test_type2": "str",\n  "drug_test_type2_name": "str",\n  "drug_test_type3": "str",\n  "drug_test_type3_name": "str",\n  "drug_result1": "str",\n  "drug_result1_name": "str",\n  "drug_result2": "str",\n  "drug_result2_name": "str",\n  "drug_result3": "str",\n  "drug_result3_name": "str",\n  "hospital": "str",\n  "hospital_name": "str",\n  "died_at_scene_en_route": "str",\n  "died_at_scene_en_route_name": "str",\n  "day_of_death": "str",\n  "day_of_death_name": "str",\n  "month_of_death": "str",\n  "month_of_death_name": "str",\n  "year_of_death": "str",\n  "year_of_death_name": "str",\n  "hour_of_death": "str",\n  "hour_of_death_name": "str",\n  "minute_of_death": "str",\n  "minute_of_death_name": "str",\n  "death_time": "str",\n  "death_time_name": "str",\n  "lag_hours": "str",\n  "lag_hours_name": "str",\n  "lag_minutes": "str",\n  "lag_minutes_name": "str",\n  "related_factors_person_level1": "str",\n  "related_factors_person_level1_name": "str",\n  "related_factors_person_level2": "str",\n  "related_factors_person_level2_name": "str",\n  "related_factors_person_level3": "str",\n  "related_factors_person_level3_name": "str",\n  "fatal_injury_at_work": "str",\n  "fatal_injury_at_work_name": "str",\n  "hispanic_origin": "str",\n  "hispanic_origin_name": "str",\n  "race": "str",\n  "race_name": "str",\n  "non_motorist_location_at_time_of_crash": "str",\n  "non_motorist_location_at_time_of_crash_name": "str"\n}',
            "RENAME_MAPPINGS_LIST": '{\n  "STATE": "state_number",\n  "STATENAME": "state_name",\n  "ST_CASE": "consecutive_number",\n  "VE_FORMS": "number_of_motor_vehicles_in_transport_mvit",\n  "VEH_NO": "vehicle_number",\n  "PER_NO": "person_number",\n  "STR_VEH": "number_of_motor_vehicle_striking_non_motorist",\n  "COUNTY": "county",\n  "DAY": "day_of_crash",\n  "DAYNAME": "day_of_crash_name",\n  "MONTH": "month_of_crash",\n  "MONTHNAME": "month_of_crash_name",\n  "HOUR": "hour_of_crash",\n  "HOURNAME": "hour_of_crash_name",\n  "MINUTE": "minute_of_crash",\n  "MINUTENAME": "minute_of_crash_name",\n  "RUR_URB": "land_use",\n  "RUR_URBNAME": "land_use_name",\n  "FUNC_SYS": "functional_system",\n  "FUNC_SYSNAME": "functional_system_name",\n  "HARM_EV": "first_harmful_event",\n  "HARM_EVNAME": "first_harmful_event_name",\n  "MAN_COLL": "manner_of_collision",\n  "MAN_COLLNAME": "manner_of_collision_name",\n  "SCH_BUS": "school_bus_related",\n  "SCH_BUSNAME": "school_bus_related_name",\n  "MAKE": "vehicle_make",\n  "MAKENAME": "vehicle_make_name",\n  "MAK_MOD": "make_model_combined",\n  "BODY_TYP": "body_type",\n  "BODY_TYPNAME": "body_type_name",\n  "MOD_YEAR": "model_year",\n  "MOD_YEARNAME": "model_year_name",\n  "TOW_VEH": "vehicle_trailing",\n  "TOW_VEHNAME": "vehicle_trailing_name",\n  "SPEC_USE": "special_use",\n  "SPEC_USENAME": "special_use_name",\n  "EMER_USE": "emergency_motor_vehicle_use",\n  "EMER_USENAME": "emergency_motor_vehicle_use_name",\n  "ROLLOVER": "rollover",\n  "ROLLOVERNAME": "rollover_name",\n  "IMPACT1": "initial_contact_point",\n  "IMPACT1NAME": "initial_contact_point_name",\n  "FIRE_EXP": "fire_occurrence",\n  "FIRE_EXPNAME": "fire_occurrence_name",\n  "AGE": "age",\n  "AGENAME": "age_name",\n  "SEX": "sex",\n  "SEXNAME": "sex_name",\n  "PER_TYP": "person_type",\n  "PER_TYPNAME": "person_type_name",\n  "INJ_SEV": "injury_severity",\n  "INJ_SEVNAME": "injury_severity_name",\n  "SEAT_POS": "seating_position",\n  "SEAT_POSNAME": "seating_position_name",\n  "REST_USE": "restraint_system_helmet_use",\n  "REST_USENAME": "restraint_system_helmet_use_name",\n  "REST_MIS": "indication_of_misuse_of_restraint_system_helmet",\n  "REST_MISNAME": "indication_of_misuse_of_restraint_system_helmet_name",\n  "AIR_BAG": "air_bag_deployed",\n  "AIR_BAGNAME": "air_bag_deployed_name",\n  "EJECTION": "ejection",\n  "EJECTIONNAME": "ejection_name",\n  "EJ_PATH": "ejection_path",\n  "EJ_PATHNAME": "ejection_path_name",\n  "EXTRICAT": "extrication",\n  "EXTRICATNAME": "extrication_name",\n  "DRINKING": "police_reported_alcohol_involvement",\n  "DRINKINGNAME": "police_reported_alcohol_involvement_name",\n  "ALC_DET": "method_of_alcohol_determination_by_police",\n  "ALC_DETNAME": "method_of_alcohol_determination_by_police_name",\n  "ALC_STATUS": "alcohol_test_status",\n  "ALC_STATUSNAME": "alcohol_test_status_name",\n  "ATST_TYP": "alcohol_test_type",\n  "ATST_TYPNAME": "alcohol_test_type_name",\n  "ALC_RES": "alcohol_result",\n  "ALC_RESNAME": "alcohol_result_name",\n  "DRUGS": "police_reported_drug_involvement",\n  "DRUGSNAME": "police_reported_drug_involvement_name",\n  "DRUG_DET": "method_of_drug_determination_by_police",\n  "DRUG_DETNAME": "method_of_drug_determination_by_police_name",\n  "DSTATUS": "drug_test_status",\n  "DSTATUSNAME": "drug_test_status_name",\n  "DRUGTST1": "drug_test_type1",\n  "DRUGTST1NAME": "drug_test_type1_name",\n  "DRUGTST2": "drug_test_type2",\n  "DRUGTST2NAME": "drug_test_type2_name",\n  "DRUGTST3": "drug_test_type3",\n  "DRUGTST3NAME": "drug_test_type3_name",\n  "DRUGRES1": "drug_result1",\n  "DRUGRES1NAME": "drug_result1_name",\n  "DRUGRES2": "drug_result2",\n  "DRUGRES2NAME": "drug_result2_name",\n  "DRUGRES3": "drug_result3",\n  "DRUGRES3NAME": "drug_result3_name",\n  "HOSPITAL": "hospital",\n  "HOSPITALNAME": "hospital_name",\n  "DOA": "died_at_scene_en_route",\n  "DOANAME": "died_at_scene_en_route_name",\n  "DEATH_DA": "day_of_death",\n  "DEATH_DANAME": "day_of_death_name",\n  "DEATH_MO": "month_of_death",\n  "DEATH_MONAME": "month_of_death_name",\n  "DEATH_YR": "year_of_death",\n  "DEATH_YRNAME": "year_of_death_name",\n  "DEATH_HR": "hour_of_death",\n  "DEATH_HRNAME": "hour_of_death_name",\n  "DEATH_MN": "minute_of_death",\n  "DEATH_MNNAME": "minute_of_death_name",\n  "DEATH_TM": "death_time",\n  "DEATH_TMNAME": "death_time_name",\n  "LAG_HRS": "lag_hours",\n  "LAG_HRSNAME": "lag_hours_name",\n  "LAG_MINS": "lag_minutes",\n  "LAG_MINSNAME": "lag_minutes_name",\n  "P_SF1": "related_factors_person_level1",\n  "P_SF1NAME": "related_factors_person_level1_name",\n  "P_SF2": "related_factors_person_level2",\n  "P_SF2NAME": "related_factors_person_level2_name",\n  "P_SF3": "related_factors_person_level3",\n  "P_SF3NAME": "related_factors_person_level3_name",\n  "WORK_INJ": "fatal_injury_at_work",\n  "WORK_INJNAME": "fatal_injury_at_work_name",\n  "HISPANIC": "hispanic_origin",\n  "HISPANICNAME": "hispanic_origin_name",\n  "RACE": "race",\n  "RACENAME": "race_name",\n  "LOCATION": "non_motorist_location_at_time_of_crash",\n  "LOCATIONNAME": "non_motorist_location_at_time_of_crash_name"\n}',
        },
        resources={
            "request_ephemeral_storage": "4G",
            "request_cpu": "1",
            "request_memory": "4G",
        },
    )

    # Run CSV transform within kubernetes pod for person pipelines
    person_2018_transform_csv = kubernetes_pod.KubernetesPodOperator(
        task_id="person_2018_transform_csv",
        startup_timeout_seconds=600,
        name="person",
        namespace="composer",
        service_account_name="datasets",
        image_pull_policy="Always",
        image="{{ var.json.nhtsa_traffic_fatalities.container_registry.run_csv_transform_kub }}",
        env_vars={
            "PIPELINE_NAME": "{{ var.json.nhtsa_traffic_fatalities.person_2018.pipeline_name }}",
            "SOURCE_URL": "{{ var.json.nhtsa_traffic_fatalities.person_2018.source_url }}",
            "CHUNKSIZE": "{{ var.json.nhtsa_traffic_fatalities.person_2018.chunksize }}",
            "SOURCE_ZIPFILE_EXTRACTED": "person.csv",
            "SOURCE_FILE": "{{ var.json.nhtsa_traffic_fatalities.person_2018.source_file }}",
            "PROJECT_ID": "{{ var.value.gcp_project }}",
            "DATASET_ID": "{{ var.json.nhtsa_traffic_fatalities.person_2018.dataset_id }}",
            "TABLE_ID": "{{ var.json.nhtsa_traffic_fatalities.person_2018.destination_table }}",
            "START_YEAR": "{{ var.json.nhtsa_traffic_fatalities.person_2018.start_year }}",
            "END_YEAR": "{{ var.json.nhtsa_traffic_fatalities.person_2018.end_year }}",
            "DROP_DEST_TABLE": "{{ var.json.nhtsa_traffic_fatalities.person_2018.drop_dest_table }}",
            "TARGET_GCS_BUCKET": "{{ var.value.composer_bucket }}",
            "TARGET_GCS_PATH": "{{ var.json.nhtsa_traffic_fatalities.person_2018.target_gcs_path }}",
            "SCHEMA_PATH": "{{ var.json.nhtsa_traffic_fatalities.person_2018.schema_path }}",
            "INPUT_CSV_HEADERS": '[\n  "state_number",\n  "state_name",\n  "consecutive_number",\n  "number_of_motor_vehicles_in_transport_mvit",\n  "vehicle_number",\n  "person_number",\n  "number_of_motor_vehicle_striking_non_motorist",\n  "county",\n  "day_of_crash",\n  "day_of_crash_name",\n  "month_of_crash",\n  "month_of_crash_name",\n  "hour_of_crash",\n  "hour_of_crash_name",\n  "minute_of_crash",\n  "minute_of_crash_name",\n  "land_use",\n  "land_use_name",\n  "functional_system",\n  "functional_system_name",\n  "first_harmful_event",\n  "first_harmful_event_name",\n  "manner_of_collision",\n  "manner_of_collision_name",\n  "school_bus_related",\n  "school_bus_related_name",\n  "vehicle_make",\n  "vehicle_make_name",\n  "make_model_combined",\n  "make_model_combined_name",\n  "body_type",\n  "body_type_name",\n  "model_year",\n  "model_year_name",\n  "vehicle_trailing",\n  "vehicle_trailing_name",\n  "special_use",\n  "special_use_name",\n  "emergency_motor_vehicle_use",\n  "emergency_motor_vehicle_use_name",\n  "rollover",\n  "rollover_name",\n  "initial_contact_point",\n  "initial_contact_point_name",\n  "fire_occurrence",\n  "fire_occurrence_name",\n  "age",\n  "age_name",\n  "sex",\n  "sex_name",\n  "person_type",\n  "person_type_name",\n  "injury_severity",\n  "injury_severity_name",\n  "seating_position",\n  "seating_position_name",\n  "restraint_system_helmet_use",\n  "restraint_system_helmet_use_name",\n  "indication_of_misuse_of_restraint_system_helmet",\n  "indication_of_misuse_of_restraint_system_helmet_name",\n  "air_bag_deployed",\n  "air_bag_deployed_name",\n  "ejection",\n  "ejection_name",\n  "ejection_path",\n  "ejection_path_name",\n  "extrication",\n  "extrication_name",\n  "police_reported_alcohol_involvement",\n  "police_reported_alcohol_involvement_name",\n  "method_of_alcohol_determination_by_police",\n  "method_of_alcohol_determination_by_police_name",\n  "alcohol_test_status",\n  "alcohol_test_status_name",\n  "alcohol_test_type",\n  "alcohol_test_type_name",\n  "alcohol_result",\n  "alcohol_result_name",\n  "police_reported_drug_involvement",\n  "police_reported_drug_involvement_name",\n  "method_of_drug_determination_by_police",\n  "method_of_drug_determination_by_police_name",\n  "drug_test_status",\n  "drug_test_status_name",\n  "hospital",\n  "hospital_name",\n  "died_at_scene_en_route",\n  "died_at_scene_en_route_name",\n  "day_of_death",\n  "day_of_death_name",\n  "month_of_death",\n  "month_of_death_name",\n  "year_of_death",\n  "year_of_death_name",\n  "hour_of_death",\n  "hour_of_death_name",\n  "minute_of_death",\n  "minute_of_death_name",\n  "death_time",\n  "death_time_name",\n  "lag_hours",\n  "lag_hours_name",\n  "lag_minutes",\n  "lag_minutes_name",\n  "related_factors_person_level1",\n  "related_factors_person_level1_name",\n  "related_factors_person_level2",\n  "related_factors_person_level2_name",\n  "related_factors_person_level3",\n  "related_factors_person_level3_name",\n  "fatal_injury_at_work",\n  "fatal_injury_at_work_name",\n  "hispanic_origin",\n  "hispanic_origin_name",\n  "race",\n  "race_name",\n  "non_motorist_location_at_time_of_crash",\n  "non_motorist_location_at_time_of_crash_name"\n]',
            "INPUT_DTYPES": '{\n  "state_number": "str",\n  "state_name": "str",\n  "consecutive_number": "str",\n  "number_of_motor_vehicles_in_transport_mvit": "str",\n  "vehicle_number": "str",\n  "person_number": "str",\n  "number_of_motor_vehicle_striking_non_motorist": "str",\n  "county": "str",\n  "day_of_crash": "str",\n  "day_of_crash_name": "str",\n  "month_of_crash": "str",\n  "month_of_crash_name": "str",\n  "hour_of_crash": "str",\n  "hour_of_crash_name": "str",\n  "minute_of_crash": "str",\n  "minute_of_crash_name": "str",\n  "land_use": "str",\n  "land_use_name": "str",\n  "functional_system": "str",\n  "functional_system_name": "str",\n  "first_harmful_event": "str",\n  "first_harmful_event_name": "str",\n  "manner_of_collision": "str",\n  "manner_of_collision_name": "str",\n  "school_bus_related": "str",\n  "school_bus_related_name": "str",\n  "vehicle_make": "str",\n  "vehicle_make_name": "str",\n  "make_model_combined": "str",\n  "make_model_combined_name": "str",\n  "body_type": "str",\n  "body_type_name": "str",\n  "model_year": "str",\n  "model_year_name": "str",\n  "vehicle_trailing": "str",\n  "vehicle_trailing_name": "str",\n  "special_use": "str",\n  "special_use_name": "str",\n  "emergency_motor_vehicle_use": "str",\n  "emergency_motor_vehicle_use_name": "str",\n  "rollover": "str",\n  "rollover_name": "str",\n  "initial_contact_point": "str",\n  "initial_contact_point_name": "str",\n  "fire_occurrence": "str",\n  "fire_occurrence_name": "str",\n  "age": "str",\n  "age_name": "str",\n  "sex": "str",\n  "sex_name": "str",\n  "person_type": "str",\n  "person_type_name": "str",\n  "injury_severity": "str",\n  "injury_severity_name": "str",\n  "seating_position": "str",\n  "seating_position_name": "str",\n  "restraint_system_helmet_use": "str",\n  "restraint_system_helmet_use_name": "str",\n  "indication_of_misuse_of_restraint_system_helmet": "str",\n  "indication_of_misuse_of_restraint_system_helmet_name": "str",\n  "air_bag_deployed": "str",\n  "air_bag_deployed_name": "str",\n  "ejection": "str",\n  "ejection_name": "str",\n  "ejection_path": "str",\n  "ejection_path_name": "str",\n  "extrication": "str",\n  "extrication_name": "str",\n  "police_reported_alcohol_involvement": "str",\n  "police_reported_alcohol_involvement_name": "str",\n  "method_of_alcohol_determination_by_police": "str",\n  "method_of_alcohol_determination_by_police_name": "str",\n  "alcohol_test_status": "str",\n  "alcohol_test_status_name": "str",\n  "alcohol_test_type": "str",\n  "alcohol_test_type_name": "str",\n  "alcohol_result": "str",\n  "alcohol_result_name": "str",\n  "police_reported_drug_involvement": "str",\n  "police_reported_drug_involvement_name": "str",\n  "method_of_drug_determination_by_police": "str",\n  "method_of_drug_determination_by_police_name": "str",\n  "drug_test_status": "str",\n  "drug_test_status_name": "str",\n  "hospital": "str",\n  "hospital_name": "str",\n  "died_at_scene_en_route": "str",\n  "died_at_scene_en_route_name": "str",\n  "day_of_death": "str",\n  "day_of_death_name": "str",\n  "month_of_death": "str",\n  "month_of_death_name": "str",\n  "year_of_death": "str",\n  "year_of_death_name": "str",\n  "hour_of_death": "str",\n  "hour_of_death_name": "str",\n  "minute_of_death": "str",\n  "minute_of_death_name": "str",\n  "death_time": "str",\n  "death_time_name": "str",\n  "lag_hours": "str",\n  "lag_hours_name": "str",\n  "lag_minutes": "str",\n  "lag_minutes_name": "str",\n  "related_factors_person_level1": "str",\n  "related_factors_person_level1_name": "str",\n  "related_factors_person_level2": "str",\n  "related_factors_person_level2_name": "str",\n  "related_factors_person_level3": "str",\n  "related_factors_person_level3_name": "str",\n  "fatal_injury_at_work": "str",\n  "fatal_injury_at_work_name": "str",\n  "hispanic_origin": "str",\n  "hispanic_origin_name": "str",\n  "race": "str",\n  "race_name": "str",\n  "non_motorist_location_at_time_of_crash": "str",\n  "non_motorist_location_at_time_of_crash_name": "str"\n}',
            "RENAME_MAPPINGS_LIST": '{\n  "STATE": "state_number",\n  "STATENAME": "state_name",\n  "ST_CASE": "consecutive_number",\n  "VE_FORMS": "number_of_motor_vehicles_in_transport_mvit",\n  "VEH_NO": "vehicle_number",\n  "PER_NO": "person_number",\n  "STR_VEH": "number_of_motor_vehicle_striking_non_motorist",\n  "COUNTY": "county",\n  "DAY": "day_of_crash",\n  "DAYNAME": "day_of_crash_name",\n  "MONTH": "month_of_crash",\n  "MONTHNAME": "month_of_crash_name",\n  "HOUR": "hour_of_crash",\n  "HOURNAME": "hour_of_crash_name",\n  "MINUTE": "minute_of_crash",\n  "MINUTENAME": "minute_of_crash_name",\n  "RUR_URB": "land_use",\n  "RUR_URBNAME": "land_use_name",\n  "FUNC_SYS": "functional_system",\n  "FUNC_SYSNAME": "functional_system_name",\n  "HARM_EV": "first_harmful_event",\n  "HARM_EVNAME": "first_harmful_event_name",\n  "MAN_COLL": "manner_of_collision",\n  "MAN_COLLNAME": "manner_of_collision_name",\n  "SCH_BUS": "school_bus_related",\n  "SCH_BUSNAME": "school_bus_related_name",\n  "MAKE": "vehicle_make",\n  "MAKENAME": "vehicle_make_name",\n  "MAK_MOD": "make_model_combined",\n  "MAK_MODNAME": "make_model_combined_name",\n  "BODY_TYP": "body_type",\n  "BODY_TYPNAME": "body_type_name",\n  "MOD_YEAR": "model_year",\n  "MOD_YEARNAME": "model_year_name",\n  "TOW_VEH": "vehicle_trailing",\n  "TOW_VEHNAME": "vehicle_trailing_name",\n  "SPEC_USE": "special_use",\n  "SPEC_USENAME": "special_use_name",\n  "EMER_USE": "emergency_motor_vehicle_use",\n  "EMER_USENAME": "emergency_motor_vehicle_use_name",\n  "ROLLOVER": "rollover",\n  "ROLLOVERNAME": "rollover_name",\n  "IMPACT1": "initial_contact_point",\n  "IMPACT1NAME": "initial_contact_point_name",\n  "FIRE_EXP": "fire_occurrence",\n  "FIRE_EXPNAME": "fire_occurrence_name",\n  "AGE": "age",\n  "AGENAME": "age_name",\n  "SEX": "sex",\n  "SEXNAME": "sex_name",\n  "PER_TYP": "person_type",\n  "PER_TYPNAME": "person_type_name",\n  "INJ_SEV": "injury_severity",\n  "INJ_SEVNAME": "injury_severity_name",\n  "SEAT_POS": "seating_position",\n  "SEAT_POSNAME": "seating_position_name",\n  "REST_USE": "restraint_system_helmet_use",\n  "REST_USENAME": "restraint_system_helmet_use_name",\n  "REST_MIS": "indication_of_misuse_of_restraint_system_helmet",\n  "REST_MISNAME": "indication_of_misuse_of_restraint_system_helmet_name",\n  "AIR_BAG": "air_bag_deployed",\n  "AIR_BAGNAME": "air_bag_deployed_name",\n  "EJECTION": "ejection",\n  "EJECTIONNAME": "ejection_name",\n  "EJ_PATH": "ejection_path",\n  "EJ_PATHNAME": "ejection_path_name",\n  "EXTRICAT": "extrication",\n  "EXTRICATNAME": "extrication_name",\n  "DRINKING": "police_reported_alcohol_involvement",\n  "DRINKINGNAME": "police_reported_alcohol_involvement_name",\n  "ALC_DET": "method_of_alcohol_determination_by_police",\n  "ALC_DETNAME": "method_of_alcohol_determination_by_police_name",\n  "ALC_STATUS": "alcohol_test_status",\n  "ALC_STATUSNAME": "alcohol_test_status_name",\n  "ATST_TYP": "alcohol_test_type",\n  "ATST_TYPNAME": "alcohol_test_type_name",\n  "ALC_RES": "alcohol_result",\n  "ALC_RESNAME": "alcohol_result_name",\n  "DRUGS": "police_reported_drug_involvement",\n  "DRUGSNAME": "police_reported_drug_involvement_name",\n  "DRUG_DET": "method_of_drug_determination_by_police",\n  "DRUG_DETNAME": "method_of_drug_determination_by_police_name",\n  "DSTATUS": "drug_test_status",\n  "DSTATUSNAME": "drug_test_status_name",\n  "HOSPITAL": "hospital",\n  "HOSPITALNAME": "hospital_name",\n  "DOA": "died_at_scene_en_route",\n  "DOANAME": "died_at_scene_en_route_name",\n  "DEATH_DA": "day_of_death",\n  "DEATH_DANAME": "day_of_death_name",\n  "DEATH_MO": "month_of_death",\n  "DEATH_MONAME": "month_of_death_name",\n  "DEATH_YR": "year_of_death",\n  "DEATH_YRNAME": "year_of_death_name",\n  "DEATH_HR": "hour_of_death",\n  "DEATH_HRNAME": "hour_of_death_name",\n  "DEATH_MN": "minute_of_death",\n  "DEATH_MNNAME": "minute_of_death_name",\n  "DEATH_TM": "death_time",\n  "DEATH_TMNAME": "death_time_name",\n  "LAG_HRS": "lag_hours",\n  "LAG_HRSNAME": "lag_hours_name",\n  "LAG_MINS": "lag_minutes",\n  "LAG_MINSNAME": "lag_minutes_name",\n  "P_SF1": "related_factors_person_level1",\n  "P_SF1NAME": "related_factors_person_level1_name",\n  "P_SF2": "related_factors_person_level2",\n  "P_SF2NAME": "related_factors_person_level2_name",\n  "P_SF3": "related_factors_person_level3",\n  "P_SF3NAME": "related_factors_person_level3_name",\n  "WORK_INJ": "fatal_injury_at_work",\n  "WORK_INJNAME": "fatal_injury_at_work_name",\n  "HISPANIC": "hispanic_origin",\n  "HISPANICNAME": "hispanic_origin_name",\n  "RACE": "race",\n  "RACENAME": "race_name",\n  "LOCATION": "non_motorist_location_at_time_of_crash",\n  "LOCATIONNAME": "non_motorist_location_at_time_of_crash_name"\n}',
        },
        resources={
            "request_ephemeral_storage": "4G",
            "request_cpu": "1",
            "request_memory": "4G",
        },
    )

    # Run CSV transform within kubernetes pod for person pipelines
    person_2019_transform_csv = kubernetes_pod.KubernetesPodOperator(
        task_id="person_2019_transform_csv",
        startup_timeout_seconds=600,
        name="person",
        namespace="composer",
        service_account_name="datasets",
        image_pull_policy="Always",
        image="{{ var.json.nhtsa_traffic_fatalities.container_registry.run_csv_transform_kub }}",
        env_vars={
            "PIPELINE_NAME": "{{ var.json.nhtsa_traffic_fatalities.person_2019.pipeline_name }}",
            "SOURCE_URL": "{{ var.json.nhtsa_traffic_fatalities.person_2019.source_url }}",
            "CHUNKSIZE": "{{ var.json.nhtsa_traffic_fatalities.person_2019.chunksize }}",
            "SOURCE_ZIPFILE_EXTRACTED": "person.csv",
            "SOURCE_FILE": "{{ var.json.nhtsa_traffic_fatalities.person_2019.source_file }}",
            "PROJECT_ID": "{{ var.value.gcp_project }}",
            "DATASET_ID": "{{ var.json.nhtsa_traffic_fatalities.person_2019.dataset_id }}",
            "TABLE_ID": "{{ var.json.nhtsa_traffic_fatalities.person_2019.destination_table }}",
            "START_YEAR": "{{ var.json.nhtsa_traffic_fatalities.person_2019.start_year }}",
            "END_YEAR": "{{ var.json.nhtsa_traffic_fatalities.person_2019.end_year }}",
            "DROP_DEST_TABLE": "{{ var.json.nhtsa_traffic_fatalities.person_2019.drop_dest_table }}",
            "TARGET_GCS_BUCKET": "{{ var.value.composer_bucket }}",
            "TARGET_GCS_PATH": "{{ var.json.nhtsa_traffic_fatalities.person_2019.target_gcs_path }}",
            "SCHEMA_PATH": "{{ var.json.nhtsa_traffic_fatalities.person_2019.schema_path }}",
            "INPUT_CSV_HEADERS": '[\n  "state_number",\n  "state_name",\n  "consecutive_number",\n  "number_of_motor_vehicles_in_transport_mvit",\n  "vehicle_number",\n  "person_number",\n  "number_of_motor_vehicle_striking_non_motorist",\n  "county",\n  "day_of_crash",\n  "day_of_crash_name",\n  "month_of_crash",\n  "month_of_crash_name",\n  "hour_of_crash",\n  "hour_of_crash_name",\n  "minute_of_crash",\n  "minute_of_crash_name",\n  "land_use",\n  "land_use_name",\n  "functional_system",\n  "functional_system_name",\n  "first_harmful_event",\n  "first_harmful_event_name",\n  "manner_of_collision",\n  "manner_of_collision_name",\n  "school_bus_related",\n  "school_bus_related_name",\n  "vehicle_make",\n  "vehicle_make_name",\n  "make_model_combined",\n  "body_type",\n  "body_type_name",\n  "model_year",\n  "model_year_name",\n  "vehicle_trailing",\n  "vehicle_trailing_name",\n  "special_use",\n  "special_use_name",\n  "emergency_motor_vehicle_use",\n  "emergency_motor_vehicle_use_name",\n  "rollover",\n  "rollover_name",\n  "initial_contact_point",\n  "initial_contact_point_name",\n  "fire_occurrence",\n  "fire_occurrence_name",\n  "age",\n  "age_name",\n  "sex",\n  "sex_name",\n  "person_type",\n  "person_type_name",\n  "injury_severity",\n  "injury_severity_name",\n  "seating_position",\n  "seating_position_name",\n  "restraint_system_helmet_use",\n  "restraint_system_helmet_use_name",\n  "indication_of_misuse_of_restraint_system_helmet",\n  "indication_of_misuse_of_restraint_system_helmet_name",\n  "air_bag_deployed",\n  "air_bag_deployed_name",\n  "ejection",\n  "ejection_name",\n  "ejection_path",\n  "ejection_path_name",\n  "extrication",\n  "extrication_name",\n  "police_reported_alcohol_involvement",\n  "police_reported_alcohol_involvement_name",\n  "method_of_alcohol_determination_by_police",\n  "method_of_alcohol_determination_by_police_name",\n  "alcohol_test_status",\n  "alcohol_test_status_name",\n  "alcohol_test_type",\n  "alcohol_test_type_name",\n  "alcohol_result",\n  "alcohol_result_name",\n  "police_reported_drug_involvement",\n  "police_reported_drug_involvement_name",\n  "method_of_drug_determination_by_police",\n  "method_of_drug_determination_by_police_name",\n  "drug_test_status",\n  "drug_test_status_name",\n  "hospital",\n  "hospital_name",\n  "died_at_scene_en_route",\n  "died_at_scene_en_route_name",\n  "day_of_death",\n  "day_of_death_name",\n  "month_of_death",\n  "month_of_death_name",\n  "year_of_death",\n  "year_of_death_name",\n  "hour_of_death",\n  "hour_of_death_name",\n  "minute_of_death",\n  "minute_of_death_name",\n  "death_time",\n  "death_time_name",\n  "lag_hours",\n  "lag_hours_name",\n  "lag_minutes",\n  "lag_minutes_name",\n  "related_factors_person_level1",\n  "related_factors_person_level1_name",\n  "related_factors_person_level2",\n  "related_factors_person_level2_name",\n  "related_factors_person_level3",\n  "related_factors_person_level3_name",\n  "fatal_injury_at_work",\n  "fatal_injury_at_work_name",\n  "hispanic_origin",\n  "hispanic_origin_name",\n  "non_motorist_location_at_time_of_crash",\n  "non_motorist_location_at_time_of_crash_name",\n  "helm_use",\n  "helm_usename",\n  "helm_mis",\n  "helm_misname"\n]',
            "INPUT_DTYPES": '{\n  "state_number": "str",\n  "state_name": "str",\n  "consecutive_number": "str",\n  "number_of_motor_vehicles_in_transport_mvit": "str",\n  "vehicle_number": "str",\n  "person_number": "str",\n  "number_of_motor_vehicle_striking_non_motorist": "str",\n  "county": "str",\n  "day_of_crash": "str",\n  "day_of_crash_name": "str",\n  "month_of_crash": "str",\n  "month_of_crash_name": "str",\n  "hour_of_crash": "str",\n  "hour_of_crash_name": "str",\n  "minute_of_crash": "str",\n  "minute_of_crash_name": "str",\n  "land_use": "str",\n  "land_use_name": "str",\n  "functional_system": "str",\n  "functional_system_name": "str",\n  "first_harmful_event": "str",\n  "first_harmful_event_name": "str",\n  "manner_of_collision": "str",\n  "manner_of_collision_name": "str",\n  "school_bus_related": "str",\n  "school_bus_related_name": "str",\n  "vehicle_make": "str",\n  "vehicle_make_name": "str",\n  "make_model_combined": "str",\n  "body_type": "str",\n  "body_type_name": "str",\n  "model_year": "str",\n  "model_year_name": "str",\n  "vehicle_trailing": "str",\n  "vehicle_trailing_name": "str",\n  "special_use": "str",\n  "special_use_name": "str",\n  "emergency_motor_vehicle_use": "str",\n  "emergency_motor_vehicle_use_name": "str",\n  "rollover": "str",\n  "rollover_name": "str",\n  "initial_contact_point": "str",\n  "initial_contact_point_name": "str",\n  "fire_occurrence": "str",\n  "fire_occurrence_name": "str",\n  "age": "str",\n  "age_name": "str",\n  "sex": "str",\n  "sex_name": "str",\n  "person_type": "str",\n  "person_type_name": "str",\n  "injury_severity": "str",\n  "injury_severity_name": "str",\n  "seating_position": "str",\n  "seating_position_name": "str",\n  "restraint_system_helmet_use": "str",\n  "restraint_system_helmet_use_name": "str",\n  "indication_of_misuse_of_restraint_system_helmet": "str",\n  "indication_of_misuse_of_restraint_system_helmet_name": "str",\n  "air_bag_deployed": "str",\n  "air_bag_deployed_name": "str",\n  "ejection": "str",\n  "ejection_name": "str",\n  "ejection_path": "str",\n  "ejection_path_name": "str",\n  "extrication": "str",\n  "extrication_name": "str",\n  "police_reported_alcohol_involvement": "str",\n  "police_reported_alcohol_involvement_name": "str",\n  "method_of_alcohol_determination_by_police": "str",\n  "method_of_alcohol_determination_by_police_name": "str",\n  "alcohol_test_status": "str",\n  "alcohol_test_status_name": "str",\n  "alcohol_test_type": "str",\n  "alcohol_test_type_name": "str",\n  "alcohol_result": "str",\n  "alcohol_result_name": "str",\n  "police_reported_drug_involvement": "str",\n  "police_reported_drug_involvement_name": "str",\n  "method_of_drug_determination_by_police": "str",\n  "method_of_drug_determination_by_police_name": "str",\n  "drug_test_status": "str",\n  "drug_test_status_name": "str",\n  "hospital": "str",\n  "hospital_name": "str",\n  "died_at_scene_en_route": "str",\n  "died_at_scene_en_route_name": "str",\n  "day_of_death": "str",\n  "day_of_death_name": "str",\n  "month_of_death": "str",\n  "month_of_death_name": "str",\n  "year_of_death": "str",\n  "year_of_death_name": "str",\n  "hour_of_death": "str",\n  "hour_of_death_name": "str",\n  "minute_of_death": "str",\n  "minute_of_death_name": "str",\n  "death_time": "str",\n  "death_time_name": "str",\n  "lag_hours": "str",\n  "lag_hours_name": "str",\n  "lag_minutes": "str",\n  "lag_minutes_name": "str",\n  "related_factors_person_level1": "str",\n  "related_factors_person_level1_name": "str",\n  "related_factors_person_level2": "str",\n  "related_factors_person_level2_name": "str",\n  "related_factors_person_level3": "str",\n  "related_factors_person_level3_name": "str",\n  "fatal_injury_at_work": "str",\n  "fatal_injury_at_work_name": "str",\n  "hispanic_origin": "str",\n  "hispanic_origin_name": "str",\n  "non_motorist_location_at_time_of_crash": "str",\n  "non_motorist_location_at_time_of_crash_name": "str",\n  "helm_use": "str",\n  "helm_usename": "str",\n  "helm_mis": "str",\n  "helm_misname": "str"\n}',
            "RENAME_MAPPINGS_LIST": '{\n  "STATE": "state_number",\n  "STATENAME": "state_name",\n  "ST_CASE": "consecutive_number",\n  "VE_FORMS": "number_of_motor_vehicles_in_transport_mvit",\n  "VEH_NO": "vehicle_number",\n  "PER_NO": "person_number",\n  "STR_VEH": "number_of_motor_vehicle_striking_non_motorist",\n  "COUNTY": "county",\n  "DAY": "day_of_crash",\n  "DAYNAME": "day_of_crash_name",\n  "MONTH": "month_of_crash",\n  "MONTHNAME": "month_of_crash_name",\n  "HOUR": "hour_of_crash",\n  "HOURNAME": "hour_of_crash_name",\n  "MINUTE": "minute_of_crash",\n  "MINUTENAME": "minute_of_crash_name",\n  "RUR_URB": "land_use",\n  "RUR_URBNAME": "land_use_name",\n  "FUNC_SYS": "functional_system",\n  "FUNC_SYSNAME": "functional_system_name",\n  "HARM_EV": "first_harmful_event",\n  "HARM_EVNAME": "first_harmful_event_name",\n  "MAN_COLL": "manner_of_collision",\n  "MAN_COLLNAME": "manner_of_collision_name",\n  "SCH_BUS": "school_bus_related",\n  "SCH_BUSNAME": "school_bus_related_name",\n  "MAKE": "vehicle_make",\n  "MAKENAME": "vehicle_make_name",\n  "MAK_MOD": "make_model_combined",\n  "BODY_TYP": "body_type",\n  "BODY_TYPNAME": "body_type_name",\n  "MOD_YEAR": "model_year",\n  "MOD_YEARNAME": "model_year_name",\n  "TOW_VEH": "vehicle_trailing",\n  "TOW_VEHNAME": "vehicle_trailing_name",\n  "SPEC_USE": "special_use",\n  "SPEC_USENAME": "special_use_name",\n  "EMER_USE": "emergency_motor_vehicle_use",\n  "EMER_USENAME": "emergency_motor_vehicle_use_name",\n  "ROLLOVER": "rollover",\n  "ROLLOVERNAME": "rollover_name",\n  "IMPACT1": "initial_contact_point",\n  "IMPACT1NAME": "initial_contact_point_name",\n  "FIRE_EXP": "fire_occurrence",\n  "FIRE_EXPNAME": "fire_occurrence_name",\n  "AGE": "age",\n  "AGENAME": "age_name",\n  "SEX": "sex",\n  "SEXNAME": "sex_name",\n  "PER_TYP": "person_type",\n  "PER_TYPNAME": "person_type_name",\n  "INJ_SEV": "injury_severity",\n  "INJ_SEVNAME": "injury_severity_name",\n  "SEAT_POS": "seating_position",\n  "SEAT_POSNAME": "seating_position_name",\n  "REST_USE": "restraint_system_helmet_use",\n  "REST_USENAME": "restraint_system_helmet_use_name",\n  "REST_MIS": "indication_of_misuse_of_restraint_system_helmet",\n  "REST_MISNAME": "indication_of_misuse_of_restraint_system_helmet_name",\n  "AIR_BAG": "air_bag_deployed",\n  "AIR_BAGNAME": "air_bag_deployed_name",\n  "EJECTION": "ejection",\n  "EJECTIONNAME": "ejection_name",\n  "EJ_PATH": "ejection_path",\n  "EJ_PATHNAME": "ejection_path_name",\n  "EXTRICAT": "extrication",\n  "EXTRICATNAME": "extrication_name",\n  "DRINKING": "police_reported_alcohol_involvement",\n  "DRINKINGNAME": "police_reported_alcohol_involvement_name",\n  "ALC_DET": "method_of_alcohol_determination_by_police",\n  "ALC_DETNAME": "method_of_alcohol_determination_by_police_name",\n  "ALC_STATUS": "alcohol_test_status",\n  "ALC_STATUSNAME": "alcohol_test_status_name",\n  "ATST_TYP": "alcohol_test_type",\n  "ATST_TYPNAME": "alcohol_test_type_name",\n  "ALC_RES": "alcohol_result",\n  "ALC_RESNAME": "alcohol_result_name",\n  "DRUGS": "police_reported_drug_involvement",\n  "DRUGSNAME": "police_reported_drug_involvement_name",\n  "DRUG_DET": "method_of_drug_determination_by_police",\n  "DRUG_DETNAME": "method_of_drug_determination_by_police_name",\n  "DSTATUS": "drug_test_status",\n  "DSTATUSNAME": "drug_test_status_name",\n  "HOSPITAL": "hospital",\n  "HOSPITALNAME": "hospital_name",\n  "DOA": "died_at_scene_en_route",\n  "DOANAME": "died_at_scene_en_route_name",\n  "DEATH_DA": "day_of_death",\n  "DEATH_DANAME": "day_of_death_name",\n  "DEATH_MO": "month_of_death",\n  "DEATH_MONAME": "month_of_death_name",\n  "DEATH_YR": "year_of_death",\n  "DEATH_YRNAME": "year_of_death_name",\n  "DEATH_HR": "hour_of_death",\n  "DEATH_HRNAME": "hour_of_death_name",\n  "DEATH_MN": "minute_of_death",\n  "DEATH_MNNAME": "minute_of_death_name",\n  "DEATH_TM": "death_time",\n  "DEATH_TMNAME": "death_time_name",\n  "LAG_HRS": "lag_hours",\n  "LAG_HRSNAME": "lag_hours_name",\n  "LAG_MINS": "lag_minutes",\n  "LAG_MINSNAME": "lag_minutes_name",\n  "P_SF1": "related_factors_person_level1",\n  "P_SF1NAME": "related_factors_person_level1_name",\n  "P_SF2": "related_factors_person_level2",\n  "P_SF2NAME": "related_factors_person_level2_name",\n  "P_SF3": "related_factors_person_level3",\n  "P_SF3NAME": "related_factors_person_level3_name",\n  "WORK_INJ": "fatal_injury_at_work",\n  "WORK_INJNAME": "fatal_injury_at_work_name",\n  "HISPANIC": "hispanic_origin",\n  "HISPANICNAME": "hispanic_origin_name",\n  "LOCATION": "non_motorist_location_at_time_of_crash",\n  "LOCATIONNAME": "non_motorist_location_at_time_of_crash_name",\n  "HELM_USE": "helm_use",\n  "HELM_USENAME": "helm_usename",\n  "HELM_MIS": "helm_mis",\n  "HELM_MISNAME": "helm_misname"\n}',
        },
        resources={
            "request_ephemeral_storage": "4G",
            "request_cpu": "1",
            "request_memory": "4G",
        },
    )

    # Run CSV transform within kubernetes pod for person pipelines
    person_2020_transform_csv = kubernetes_pod.KubernetesPodOperator(
        task_id="person_2020_transform_csv",
        startup_timeout_seconds=600,
        name="person",
        namespace="composer",
        service_account_name="datasets",
        image_pull_policy="Always",
        image="{{ var.json.nhtsa_traffic_fatalities.container_registry.run_csv_transform_kub }}",
        env_vars={
            "PIPELINE_NAME": "{{ var.json.nhtsa_traffic_fatalities.person_2020.pipeline_name }}",
            "SOURCE_URL": "{{ var.json.nhtsa_traffic_fatalities.person_2020.source_url }}",
            "CHUNKSIZE": "{{ var.json.nhtsa_traffic_fatalities.person_2020.chunksize }}",
            "SOURCE_ZIPFILE_EXTRACTED": "person.csv",
            "SOURCE_FILE": "{{ var.json.nhtsa_traffic_fatalities.person_2020.source_file }}",
            "PROJECT_ID": "{{ var.value.gcp_project }}",
            "DATASET_ID": "{{ var.json.nhtsa_traffic_fatalities.person_2020.dataset_id }}",
            "TABLE_ID": "{{ var.json.nhtsa_traffic_fatalities.person_2020.destination_table }}",
            "START_YEAR": "{{ var.json.nhtsa_traffic_fatalities.person_2020.start_year }}",
            "END_YEAR": "{{ var.json.nhtsa_traffic_fatalities.person_2020.end_year }}",
            "DROP_DEST_TABLE": "{{ var.json.nhtsa_traffic_fatalities.person_2020.drop_dest_table }}",
            "TARGET_GCS_BUCKET": "{{ var.value.composer_bucket }}",
            "TARGET_GCS_PATH": "{{ var.json.nhtsa_traffic_fatalities.person_2020.target_gcs_path }}",
            "SCHEMA_PATH": "{{ var.json.nhtsa_traffic_fatalities.person_2020.schema_path }}",
            "INPUT_CSV_HEADERS": '[\n  "state_number",\n  "state_name",\n  "consecutive_number",\n  "number_of_motor_vehicles_in_transport_mvit",\n  "vehicle_number",\n  "person_number",\n  "number_of_motor_vehicle_striking_non_motorist",\n  "county",\n  "day_of_crash",\n  "day_of_crash_name",\n  "month_of_crash",\n  "month_of_crash_name",\n  "hour_of_crash",\n  "hour_of_crash_name",\n  "minute_of_crash",\n  "minute_of_crash_name",\n  "land_use",\n  "land_use_name",\n  "functional_system",\n  "functional_system_name",\n  "first_harmful_event",\n  "first_harmful_event_name",\n  "manner_of_collision",\n  "manner_of_collision_name",\n  "school_bus_related",\n  "school_bus_related_name",\n  "vehicle_make",\n  "vehicle_make_name",\n  "make_model_combined",\n  "body_type",\n  "body_type_name",\n  "model_year",\n  "model_year_name",\n  "vehicle_trailing",\n  "vehicle_trailing_name",\n  "special_use",\n  "special_use_name",\n  "emergency_motor_vehicle_use",\n  "emergency_motor_vehicle_use_name",\n  "rollover",\n  "rollover_name",\n  "initial_contact_point",\n  "initial_contact_point_name",\n  "fire_occurrence",\n  "fire_occurrence_name",\n  "age",\n  "age_name",\n  "sex",\n  "sex_name",\n  "person_type",\n  "person_type_name",\n  "injury_severity",\n  "injury_severity_name",\n  "seating_position",\n  "seating_position_name",\n  "restraint_system_helmet_use",\n  "restraint_system_helmet_use_name",\n  "indication_of_misuse_of_restraint_system_helmet",\n  "indication_of_misuse_of_restraint_system_helmet_name",\n  "air_bag_deployed",\n  "air_bag_deployed_name",\n  "ejection",\n  "ejection_name",\n  "ejection_path",\n  "ejection_path_name",\n  "extrication",\n  "extrication_name",\n  "police_reported_alcohol_involvement",\n  "police_reported_alcohol_involvement_name",\n  "method_of_alcohol_determination_by_police",\n  "method_of_alcohol_determination_by_police_name",\n  "alcohol_test_status",\n  "alcohol_test_status_name",\n  "alcohol_test_type",\n  "alcohol_test_type_name",\n  "alcohol_result",\n  "alcohol_result_name",\n  "police_reported_drug_involvement",\n  "police_reported_drug_involvement_name",\n  "method_of_drug_determination_by_police",\n  "method_of_drug_determination_by_police_name",\n  "drug_test_status",\n  "drug_test_status_name",\n  "hospital",\n  "hospital_name",\n  "died_at_scene_en_route",\n  "died_at_scene_en_route_name",\n  "day_of_death",\n  "day_of_death_name",\n  "month_of_death",\n  "month_of_death_name",\n  "year_of_death",\n  "year_of_death_name",\n  "hour_of_death",\n  "hour_of_death_name",\n  "minute_of_death",\n  "minute_of_death_name",\n  "death_time",\n  "death_time_name",\n  "lag_hours",\n  "lag_hours_name",\n  "lag_minutes",\n  "lag_minutes_name",\n  "fatal_injury_at_work",\n  "fatal_injury_at_work_name",\n  "hispanic_origin",\n  "hispanic_origin_name",\n  "non_motorist_location_at_time_of_crash",\n  "non_motorist_location_at_time_of_crash_name",\n  "helm_use",\n  "helm_usename",\n  "helm_mis",\n  "helm_misname",\n  "vpic_make",\n  "vpic_make_name",\n  "vpic_model",\n  "vpic_model_name",\n  "vpic_body_class",\n  "vpic_body_class_name",\n  "icfinal_body",\n  "icfinalbody_name"\n]',
            "INPUT_DTYPES": '{\n  "state_number": "str",\n  "state_name": "str",\n  "consecutive_number": "str",\n  "number_of_motor_vehicles_in_transport_mvit": "str",\n  "vehicle_number": "str",\n  "person_number": "str",\n  "number_of_motor_vehicle_striking_non_motorist": "str",\n  "county": "str",\n  "day_of_crash": "str",\n  "day_of_crash_name": "str",\n  "month_of_crash": "str",\n  "month_of_crash_name": "str",\n  "hour_of_crash": "str",\n  "hour_of_crash_name": "str",\n  "minute_of_crash": "str",\n  "minute_of_crash_name": "str",\n  "land_use": "str",\n  "land_use_name": "str",\n  "functional_system": "str",\n  "functional_system_name": "str",\n  "first_harmful_event": "str",\n  "first_harmful_event_name": "str",\n  "manner_of_collision": "str",\n  "manner_of_collision_name": "str",\n  "school_bus_related": "str",\n  "school_bus_related_name": "str",\n  "vehicle_make": "str",\n  "vehicle_make_name": "str",\n  "make_model_combined": "str",\n  "body_type": "str",\n  "body_type_name": "str",\n  "model_year": "str",\n  "model_year_name": "str",\n  "vehicle_trailing": "str",\n  "vehicle_trailing_name": "str",\n  "special_use": "str",\n  "special_use_name": "str",\n  "emergency_motor_vehicle_use": "str",\n  "emergency_motor_vehicle_use_name": "str",\n  "rollover": "str",\n  "rollover_name": "str",\n  "initial_contact_point": "str",\n  "initial_contact_point_name": "str",\n  "fire_occurrence": "str",\n  "fire_occurrence_name": "str",\n  "age": "str",\n  "age_name": "str",\n  "sex": "str",\n  "sex_name": "str",\n  "person_type": "str",\n  "person_type_name": "str",\n  "injury_severity": "str",\n  "injury_severity_name": "str",\n  "seating_position": "str",\n  "seating_position_name": "str",\n  "restraint_system_helmet_use": "str",\n  "restraint_system_helmet_use_name": "str",\n  "indication_of_misuse_of_restraint_system_helmet": "str",\n  "indication_of_misuse_of_restraint_system_helmet_name": "str",\n  "air_bag_deployed": "str",\n  "air_bag_deployed_name": "str",\n  "ejection": "str",\n  "ejection_name": "str",\n  "ejection_path": "str",\n  "ejection_path_name": "str",\n  "extrication": "str",\n  "extrication_name": "str",\n  "police_reported_alcohol_involvement": "str",\n  "police_reported_alcohol_involvement_name": "str",\n  "method_of_alcohol_determination_by_police": "str",\n  "method_of_alcohol_determination_by_police_name": "str",\n  "alcohol_test_status": "str",\n  "alcohol_test_status_name": "str",\n  "alcohol_test_type": "str",\n  "alcohol_test_type_name": "str",\n  "alcohol_result": "str",\n  "alcohol_result_name": "str",\n  "police_reported_drug_involvement": "str",\n  "police_reported_drug_involvement_name": "str",\n  "method_of_drug_determination_by_police": "str",\n  "method_of_drug_determination_by_police_name": "str",\n  "drug_test_status": "str",\n  "drug_test_status_name": "str",\n  "hospital": "str",\n  "hospital_name": "str",\n  "died_at_scene_en_route": "str",\n  "died_at_scene_en_route_name": "str",\n  "day_of_death": "str",\n  "day_of_death_name": "str",\n  "month_of_death": "str",\n  "month_of_death_name": "str",\n  "year_of_death": "str",\n  "year_of_death_name": "str",\n  "hour_of_death": "str",\n  "hour_of_death_name": "str",\n  "minute_of_death": "str",\n  "minute_of_death_name": "str",\n  "death_time": "str",\n  "death_time_name": "str",\n  "lag_hours": "str",\n  "lag_hours_name": "str",\n  "lag_minutes": "str",\n  "lag_minutes_name": "str",\n  "fatal_injury_at_work": "str",\n  "fatal_injury_at_work_name": "str",\n  "hispanic_origin": "str",\n  "hispanic_origin_name": "str",\n  "non_motorist_location_at_time_of_crash": "str",\n  "non_motorist_location_at_time_of_crash_name": "str",\n  "helm_use": "str",\n  "helm_usename": "str",\n  "helm_mis": "str",\n  "helm_misname": "str",\n  "vpic_make": "str",\n  "vpic_make_name": "str",\n  "vpic_model": "str",\n  "vpic_model_name": "str",\n  "vpic_body_class": "str",\n  "vpic_body_class_name": "str",\n  "icfinal_body": "str",\n  "icfinalbody_name": "str"\n}',
            "RENAME_MAPPINGS_LIST": '{\n  "STATE": "state_number",\n  "STATENAME": "state_name",\n  "ST_CASE": "consecutive_number",\n  "VE_FORMS": "number_of_motor_vehicles_in_transport_mvit",\n  "VEH_NO": "vehicle_number",\n  "PER_NO": "person_number",\n  "STR_VEH": "number_of_motor_vehicle_striking_non_motorist",\n  "COUNTY": "county",\n  "DAY": "day_of_crash",\n  "DAYNAME": "day_of_crash_name",\n  "MONTH": "month_of_crash",\n  "MONTHNAME": "month_of_crash_name",\n  "HOUR": "hour_of_crash",\n  "HOURNAME": "hour_of_crash_name",\n  "MINUTE": "minute_of_crash",\n  "MINUTENAME": "minute_of_crash_name",\n  "RUR_URB": "land_use",\n  "RUR_URBNAME": "land_use_name",\n  "FUNC_SYS": "functional_system",\n  "FUNC_SYSNAME": "functional_system_name",\n  "HARM_EV": "first_harmful_event",\n  "HARM_EVNAME": "first_harmful_event_name",\n  "MAN_COLL": "manner_of_collision",\n  "MAN_COLLNAME": "manner_of_collision_name",\n  "SCH_BUS": "school_bus_related",\n  "SCH_BUSNAME": "school_bus_related_name",\n  "MAKE": "vehicle_make",\n  "MAKENAME": "vehicle_make_name",\n  "MAK_MOD": "make_model_combined",\n  "BODY_TYP": "body_type",\n  "BODY_TYPNAME": "body_type_name",\n  "MOD_YEAR": "model_year",\n  "MOD_YEARNAME": "model_year_name",\n  "TOW_VEH": "vehicle_trailing",\n  "TOW_VEHNAME": "vehicle_trailing_name",\n  "SPEC_USE": "special_use",\n  "SPEC_USENAME": "special_use_name",\n  "EMER_USE": "emergency_motor_vehicle_use",\n  "EMER_USENAME": "emergency_motor_vehicle_use_name",\n  "ROLLOVER": "rollover",\n  "ROLLOVERNAME": "rollover_name",\n  "IMPACT1": "initial_contact_point",\n  "IMPACT1NAME": "initial_contact_point_name",\n  "FIRE_EXP": "fire_occurrence",\n  "FIRE_EXPNAME": "fire_occurrence_name",\n  "AGE": "age",\n  "AGENAME": "age_name",\n  "SEX": "sex",\n  "SEXNAME": "sex_name",\n  "PER_TYP": "person_type",\n  "PER_TYPNAME": "person_type_name",\n  "INJ_SEV": "injury_severity",\n  "INJ_SEVNAME": "injury_severity_name",\n  "SEAT_POS": "seating_position",\n  "SEAT_POSNAME": "seating_position_name",\n  "REST_USE": "restraint_system_helmet_use",\n  "REST_USENAME": "restraint_system_helmet_use_name",\n  "REST_MIS": "indication_of_misuse_of_restraint_system_helmet",\n  "REST_MISNAME": "indication_of_misuse_of_restraint_system_helmet_name",\n  "AIR_BAG": "air_bag_deployed",\n  "AIR_BAGNAME": "air_bag_deployed_name",\n  "EJECTION": "ejection",\n  "EJECTIONNAME": "ejection_name",\n  "EJ_PATH": "ejection_path",\n  "EJ_PATHNAME": "ejection_path_name",\n  "EXTRICAT": "extrication",\n  "EXTRICATNAME": "extrication_name",\n  "DRINKING": "police_reported_alcohol_involvement",\n  "DRINKINGNAME": "police_reported_alcohol_involvement_name",\n  "ALC_DET": "method_of_alcohol_determination_by_police",\n  "ALC_DETNAME": "method_of_alcohol_determination_by_police_name",\n  "ALC_STATUS": "alcohol_test_status",\n  "ALC_STATUSNAME": "alcohol_test_status_name",\n  "ATST_TYP": "alcohol_test_type",\n  "ATST_TYPNAME": "alcohol_test_type_name",\n  "ALC_RES": "alcohol_result",\n  "ALC_RESNAME": "alcohol_result_name",\n  "DRUGS": "police_reported_drug_involvement",\n  "DRUGSNAME": "police_reported_drug_involvement_name",\n  "DRUG_DET": "method_of_drug_determination_by_police",\n  "DRUG_DETNAME": "method_of_drug_determination_by_police_name",\n  "DSTATUS": "drug_test_status",\n  "DSTATUSNAME": "drug_test_status_name",\n  "HOSPITAL": "hospital",\n  "HOSPITALNAME": "hospital_name",\n  "DOA": "died_at_scene_en_route",\n  "DOANAME": "died_at_scene_en_route_name",\n  "DEATH_DA": "day_of_death",\n  "DEATH_DANAME": "day_of_death_name",\n  "DEATH_MO": "month_of_death",\n  "DEATH_MONAME": "month_of_death_name",\n  "DEATH_YR": "year_of_death",\n  "DEATH_YRNAME": "year_of_death_name",\n  "DEATH_HR": "hour_of_death",\n  "DEATH_HRNAME": "hour_of_death_name",\n  "DEATH_MN": "minute_of_death",\n  "DEATH_MNNAME": "minute_of_death_name",\n  "DEATH_TM": "death_time",\n  "DEATH_TMNAME": "death_time_name",\n  "LAG_HRS": "lag_hours",\n  "LAG_HRSNAME": "lag_hours_name",\n  "LAG_MINS": "lag_minutes",\n  "LAG_MINSNAME": "lag_minutes_name",\n  "WORK_INJ": "fatal_injury_at_work",\n  "WORK_INJNAME": "fatal_injury_at_work_name",\n  "HISPANIC": "hispanic_origin",\n  "HISPANICNAME": "hispanic_origin_name",\n  "LOCATION": "non_motorist_location_at_time_of_crash",\n  "LOCATIONNAME": "non_motorist_location_at_time_of_crash_name",\n  "HELM_USE": "helm_use",\n  "HELM_USENAME": "helm_usename",\n  "HELM_MIS": "helm_mis",\n  "HELM_MISNAME": "helm_misname",\n  "VPICMAKE": "vpic_make",\n  "VPICMAKENAME": "vpic_make_name",\n  "VPICMODEL": "vpic_model",\n  "VPICMODELNAME": "vpic_model_name",\n  "VPICBODYCLASS": "vpic_body_class",\n  "VPICBODYCLASSNAME": "vpic_body_class_name",\n  "ICFINALBODY": "icfinal_body",\n  "ICFINALBODYNAME": "icfinalbody_name"\n}',
        },
        resources={
            "request_ephemeral_storage": "4G",
            "request_cpu": "1",
            "request_memory": "4G",
        },
    )

    # Run CSV transform within kubernetes pod for safetyeq pipelines
    safetyeq_2015_2016_transform_csv = kubernetes_pod.KubernetesPodOperator(
        task_id="safetyeq_2015_2016_transform_csv",
        startup_timeout_seconds=600,
        name="safetyeq",
        namespace="composer",
        service_account_name="datasets",
        image_pull_policy="Always",
        image="{{ var.json.nhtsa_traffic_fatalities.container_registry.run_csv_transform_kub }}",
        env_vars={
            "PIPELINE_NAME": "{{ var.json.nhtsa_traffic_fatalities.safetyeq_2015_2016.pipeline_name }}",
            "SOURCE_URL": "{{ var.json.nhtsa_traffic_fatalities.safetyeq_2015_2016.source_url }}",
            "CHUNKSIZE": "{{ var.json.nhtsa_traffic_fatalities.safetyeq_2015_2016.chunksize }}",
            "SOURCE_ZIPFILE_EXTRACTED": "safetyeq.csv",
            "SOURCE_FILE": "{{ var.json.nhtsa_traffic_fatalities.safetyeq_2015_2016.source_file }}",
            "PROJECT_ID": "{{ var.value.gcp_project }}",
            "DATASET_ID": "{{ var.json.nhtsa_traffic_fatalities.safetyeq_2015_2016.dataset_id }}",
            "TABLE_ID": "{{ var.json.nhtsa_traffic_fatalities.safetyeq_2015_2016.destination_table }}",
            "START_YEAR": "{{ var.json.nhtsa_traffic_fatalities.safetyeq_2015_2016.start_year }}",
            "END_YEAR": "{{ var.json.nhtsa_traffic_fatalities.safetyeq_2015_2016.end_year }}",
            "DROP_DEST_TABLE": "{{ var.json.nhtsa_traffic_fatalities.safetyeq_2015_2016.drop_dest_table }}",
            "TARGET_GCS_BUCKET": "{{ var.value.composer_bucket }}",
            "TARGET_GCS_PATH": "{{ var.json.nhtsa_traffic_fatalities.safetyeq_2015_2016.target_gcs_path }}",
            "SCHEMA_PATH": "{{ var.json.nhtsa_traffic_fatalities.safetyeq_2015_2016.schema_path }}",
            "INPUT_CSV_HEADERS": '[\n  "state_number",\n  "state_name",\n  "consecutive_number",\n  "vehicle_number",\n  "person_number",\n  "non_motorist_safety_equipment_use",\n  "non_motorist_safety_equipment_use_name"\n]',
            "INPUT_DTYPES": '{\n  "state_number": "str",\n  "state_name": "str",\n  "consecutive_number": "str",\n  "vehicle_number": "str",\n  "person_number": "str",\n  "non_motorist_safety_equipment_use": "str",\n  "non_motorist_safety_equipment_use_name": "str"\n}',
            "RENAME_MAPPINGS_LIST": '{\n  "STATE": "state_number",\n  "STATENAME": "state_name",\n  "ST_CASE": "consecutive_number",\n  "VEH_NO": "vehicle_number",\n  "PER_NO": "person_number",\n  "MSAFEQMT": "non_motorist_safety_equipment_use",\n  "MSAFEQMTNAME": "non_motorist_safety_equipment_use_name"\n}',
        },
        resources={
            "request_ephemeral_storage": "4G",
            "request_cpu": "1",
            "request_memory": "4G",
        },
    )

    # Run CSV transform within kubernetes pod for safetyeq pipelines
    safetyeq_2017_2020_transform_csv = kubernetes_pod.KubernetesPodOperator(
        task_id="safetyeq_2017_2020_transform_csv",
        startup_timeout_seconds=600,
        name="safetyeq",
        namespace="composer",
        service_account_name="datasets",
        image_pull_policy="Always",
        image="{{ var.json.nhtsa_traffic_fatalities.container_registry.run_csv_transform_kub }}",
        env_vars={
            "PIPELINE_NAME": "{{ var.json.nhtsa_traffic_fatalities.safetyeq_2017_2020.pipeline_name }}",
            "SOURCE_URL": "{{ var.json.nhtsa_traffic_fatalities.safetyeq_2017_2020.source_url }}",
            "CHUNKSIZE": "{{ var.json.nhtsa_traffic_fatalities.safetyeq_2017_2020.chunksize }}",
            "SOURCE_ZIPFILE_EXTRACTED": "safetyeq.csv",
            "SOURCE_FILE": "{{ var.json.nhtsa_traffic_fatalities.safetyeq_2017_2020.source_file }}",
            "PROJECT_ID": "{{ var.value.gcp_project }}",
            "DATASET_ID": "{{ var.json.nhtsa_traffic_fatalities.safetyeq_2017_2020.dataset_id }}",
            "TABLE_ID": "{{ var.json.nhtsa_traffic_fatalities.safetyeq_2017_2020.destination_table }}",
            "START_YEAR": "{{ var.json.nhtsa_traffic_fatalities.safetyeq_2017_2020.start_year }}",
            "END_YEAR": "{{ var.json.nhtsa_traffic_fatalities.safetyeq_2017_2020.end_year }}",
            "DROP_DEST_TABLE": "{{ var.json.nhtsa_traffic_fatalities.safetyeq_2017_2020.drop_dest_table }}",
            "TARGET_GCS_BUCKET": "{{ var.value.composer_bucket }}",
            "TARGET_GCS_PATH": "{{ var.json.nhtsa_traffic_fatalities.safetyeq_2017_2020.target_gcs_path }}",
            "SCHEMA_PATH": "{{ var.json.nhtsa_traffic_fatalities.safetyeq_2017_2020.schema_path }}",
            "INPUT_CSV_HEADERS": '[\n  "state_number",\n  "state_name",\n  "consecutive_number",\n  "vehicle_number",\n  "person_number",\n  "nm_helmet",\n  "nm_helmet_name",\n  "nm_propad",\n  "nm_propad_name",\n  "nm_othpro",\n  "nm_othpro_name",\n  "nm_refclo",\n  "nm_refclo_name",\n  "nm_light",\n  "nm_light_name",\n  "nm_othpre",\n  "nm_othpre_name"\n]',
            "INPUT_DTYPES": '{\n  "state_number": "str",\n  "state_name": "str",\n  "consecutive_number": "str",\n  "vehicle_number": "str",\n  "person_number": "str",\n  "nm_helmet": "str",\n  "nm_helmet_name": "str",\n  "nm_propad": "str",\n  "nm_propad_name": "str",\n  "nm_othpro": "str",\n  "nm_othpro_name": "str",\n  "nm_refclo": "str",\n  "nm_refclo_name": "str",\n  "nm_light": "str",\n  "nm_light_name": "str",\n  "nm_othpre": "str",\n  "nm_othpre_name": "str"\n}',
            "RENAME_MAPPINGS_LIST": '{\n  "STATE": "state_number",\n  "STATENAME": "state_name",\n  "ST_CASE": "consecutive_number",\n  "VEH_NO": "vehicle_number",\n  "PER_NO": "person_number",\n  "NMHELMET": "nm_helmet",\n  "NMHELMETNAME": "nm_helmet_name",\n  "NMPROPAD": "nm_propad",\n  "NMPROPADNAME": "nm_propad_name",\n  "NMOTHPRO": "nm_othpro",\n  "NMOTHPRONAME": "nm_othpro_name",\n  "NMREFCLO": "nm_refclo",\n  "NMREFCLONAME": "nm_refclo_name",\n  "NMLIGHT": "nm_light",\n  "NMLIGHTNAME": "nm_light_name",\n  "NMOTHPRE": "nm_othpre",\n  "NMOTHPRENAME": "nm_othpre_name"\n}',
        },
        resources={
            "request_ephemeral_storage": "4G",
            "request_cpu": "1",
            "request_memory": "4G",
        },
    )

    # Run CSV transform within kubernetes pod for vehicle pipelines
    vehicle_2015_transform_csv = kubernetes_pod.KubernetesPodOperator(
        task_id="vehicle_2015_transform_csv",
        startup_timeout_seconds=600,
        name="vehicle",
        namespace="composer",
        service_account_name="datasets",
        image_pull_policy="Always",
        image="{{ var.json.nhtsa_traffic_fatalities.container_registry.run_csv_transform_kub }}",
        env_vars={
            "PIPELINE_NAME": "{{ var.json.nhtsa_traffic_fatalities.vehicle_2015.pipeline_name }}",
            "SOURCE_URL": "{{ var.json.nhtsa_traffic_fatalities.vehicle_2015.source_url }}",
            "CHUNKSIZE": "{{ var.json.nhtsa_traffic_fatalities.vehicle_2015.chunksize }}",
            "SOURCE_ZIPFILE_EXTRACTED": "vehicle.csv",
            "SOURCE_FILE": "{{ var.json.nhtsa_traffic_fatalities.vehicle_2015.source_file }}",
            "PROJECT_ID": "{{ var.value.gcp_project }}",
            "DATASET_ID": "{{ var.json.nhtsa_traffic_fatalities.vehicle_2015.dataset_id }}",
            "TABLE_ID": "{{ var.json.nhtsa_traffic_fatalities.vehicle_2015.destination_table }}",
            "START_YEAR": "{{ var.json.nhtsa_traffic_fatalities.vehicle_2015.start_year }}",
            "END_YEAR": "{{ var.json.nhtsa_traffic_fatalities.vehicle_2015.end_year }}",
            "DROP_DEST_TABLE": "{{ var.json.nhtsa_traffic_fatalities.vehicle_2015.drop_dest_table }}",
            "TARGET_GCS_BUCKET": "{{ var.value.composer_bucket }}",
            "TARGET_GCS_PATH": "{{ var.json.nhtsa_traffic_fatalities.vehicle_2015.target_gcs_path }}",
            "SCHEMA_PATH": "{{ var.json.nhtsa_traffic_fatalities.vehicle_2015.schema_path }}",
            "INPUT_CSV_HEADERS": '[\n  "state_number",\n  "state_name",\n  "consecutive_number",\n  "vehicle_number",\n  "number_of_motor_vehicles_in_transport_mvit",\n  "number_of_occupants",\n  "number_of_occupants_name",\n  "day_of_crash",\n  "day_of_crash_name",\n  "month_of_crash",\n  "month_of_crash_name",\n  "hour_of_crash",\n  "hour_of_crash_name",\n  "minute_of_crash",\n  "minute_of_crash_name",\n  "first_harmful_event",\n  "first_harmful_event_name",\n  "manner_of_collision",\n  "manner_of_collision_name",\n  "unit_type",\n  "unit_type_name",\n  "hit_and_run",\n  "hit_and_run_name",\n  "registration_state",\n  "registration_state_name",\n  "registered_vehicle_owner",\n  "registered_vehicle_owner_name",\n  "vehicle_make",\n  "vehicle_make_name",\n  "vehicle_model",\n  "make_model_combined",\n  "make_model_combined_name",\n  "body_type",\n  "body_type_name",\n  "vehicle_model_year",\n  "vehicle_model_year_name",\n  "vehicle_identification_number_vin",\n  "vehicle_identification_number_vin_name",\n  "vin_character_1",\n  "vin_character_2",\n  "vin_character_3",\n  "vin_character_4",\n  "vin_character_5",\n  "vin_character_6",\n  "vin_character_7",\n  "vin_character_8",\n  "vin_character_9",\n  "vin_character_10",\n  "vin_character_11",\n  "vin_character_12",\n  "vehicle_trailing",\n  "vehicle_trailing_name",\n  "jackknife",\n  "jackknife_name",\n  "mcid_issuing_authority",\n  "mcid_issuing_authority_name",\n  "mcid_identification_number",\n  "mcid_identification_number_name",\n  "motor_carrier_identification_number_mcid",\n  "motor_carrier_identification_number_mcid_name",\n  "gross_vehicle_weight_rating",\n  "gross_vehicle_weight_rating_name",\n  "vehicle_configuration",\n  "vehicle_configuration_name",\n  "cargo_body_type",\n  "cargo_body_type_name",\n  "hazardous_material_involvement",\n  "hazardous_material_involvement_name",\n  "hazardous_material_placard",\n  "hazardous_material_placard_name",\n  "hazardous_material_identification_number",\n  "hazardous_material_identification_number_name",\n  "hazardous_material_class_number",\n  "hazardous_material_class_number_name",\n  "release_of_hazardous_material_from_the_cargo_compartment",\n  "release_of_hazardous_material_from_the_cargo_compartment_name",\n  "bus_use",\n  "bus_use_name",\n  "special_use",\n  "special_use_name",\n  "emergency_motor_vehicle_use",\n  "emergency_motor_vehicle_use_name",\n  "travel_speed",\n  "travel_speed_name",\n  "underride_override",\n  "underride_override_name",\n  "rollover",\n  "rollover_name",\n  "location_of_rollover",\n  "location_of_rollover_name",\n  "initial_contact_point",\n  "initial_contact_point_name",\n  "extent_of_damage",\n  "extent_of_damage_name",\n  "vehicle_removal",\n  "vehicle_removal_name",\n  "most_harmful_event",\n  "most_harmful_event_name",\n  "related_factors_vehicle_level_1",\n  "related_factors_vehicle_level_1_name",\n  "related_factors_vehicle_level_2",\n  "related_factors_vehicle_level_2_name",\n  "fire_occurrence",\n  "fire_occurrence_name",\n  "driver_presence",\n  "driver_presence_name",\n  "drivers_license_state",\n  "drivers_license_state_name",\n  "drivers_zip_code",\n  "drivers_zip_code_name",\n  "non_cdl_license_status",\n  "non_cdl_license_status_name",\n  "non_cdl_license_type",\n  "non_cdl_license_type_name",\n  "commercial_motor_vehicle_license_status",\n  "commercial_motor_vehicle_license_status_name",\n  "compliance_with_cdl_endorsements",\n  "compliance_with_cdl_endorsements_name",\n  "license_compliance_with_class_of_vehicle",\n  "license_compliance_with_class_of_vehicle_name",\n  "compliance_with_license_restrictions",\n  "compliance_with_license_restrictions_name",\n  "driver_height",\n  "driver_height_name",\n  "driver_weight",\n  "driver_weight_name",\n  "previous_recorded_crashes",\n  "previous_recorded_crashes_name",\n  "previous_recorded_suspensions_and_revocations",\n  "previous_recorded_suspensions_and_revocations_name",\n  "previous_dwi_convictions",\n  "previous_dwi_convictions_name",\n  "previous_speeding_convictions",\n  "previous_speeding_convictions_name",\n  "previous_other_moving_violation_convictions",\n  "previous_other_moving_violation_convictions_name",\n  "month_of_first_crash_suspension_or_conviction",\n  "month_of_first_crash_suspension_or_conviction_name",\n  "year_of_first_crash_suspension_or_conviction",\n  "year_of_first_crash_suspension_or_conviction_name",\n  "month_of_last_crash_suspension_or_conviction",\n  "month_of_last_crash_suspension_or_conviction_name",\n  "year_of_last_crash_suspension_or_conviction",\n  "year_of_last_crash_suspension_or_conviction_name",\n  "speeding_related",\n  "speeding_related_name",\n  "related_factors_driver_level_1",\n  "related_factors_driver_level_1_name",\n  "related_factors_driver_level_2",\n  "related_factors_driver_level_2_name",\n  "related_factors_driver_level_3",\n  "related_factors_driver_level_3_name",\n  "related_factors_driver_level_4",\n  "related_factors_driver_level_4_name",\n  "trafficway_description",\n  "trafficway_description_name",\n  "total_lanes_in_roadway",\n  "total_lanes_in_roadway_name",\n  "speed_limit",\n  "speed_limit_name",\n  "roadway_alignment",\n  "roadway_alignment_name",\n  "roadway_grade",\n  "roadway_grade_name",\n  "roadway_surface_type",\n  "roadway_surface_type_name",\n  "roadway_surface_condition",\n  "roadway_surface_condition_name",\n  "traffic_control_device",\n  "traffic_control_device_name",\n  "traffic_control_device_functioning",\n  "traffic_control_device_functioning_name",\n  "pre_event_movement_prior_to_recognition_of_critical_event",\n  "pre_event_movement_prior_to_recognition_of_critical_event_name",\n  "critical_event_precrash",\n  "critical_event_precrash_name",\n  "attempted_avoidance_maneuver",\n  "attempted_avoidance_maneuver_name",\n  "pre_impact_stability",\n  "pre_impact_stability_name",\n  "pre_impact_location",\n  "pre_impact_location_name",\n  "crash_type",\n  "crash_type_name",\n  "fatalities_in_vehicle",\n  "driver_drinking",\n  "driver_drinking_name"\n]',
            "INPUT_DTYPES": '{\n  "state_number": "str",\n  "state_name": "str",\n  "consecutive_number": "str",\n  "vehicle_number": "str",\n  "number_of_motor_vehicles_in_transport_mvit": "str",\n  "number_of_occupants": "str",\n  "number_of_occupants_name": "str",\n  "day_of_crash": "str",\n  "day_of_crash_name": "str",\n  "month_of_crash": "str",\n  "month_of_crash_name": "str",\n  "hour_of_crash": "str",\n  "hour_of_crash_name": "str",\n  "minute_of_crash": "str",\n  "minute_of_crash_name": "str",\n  "first_harmful_event": "str",\n  "first_harmful_event_name": "str",\n  "manner_of_collision": "str",\n  "manner_of_collision_name": "str",\n  "unit_type": "str",\n  "unit_type_name": "str",\n  "hit_and_run": "str",\n  "hit_and_run_name": "str",\n  "registration_state": "str",\n  "registration_state_name": "str",\n  "registered_vehicle_owner": "str",\n  "registered_vehicle_owner_name": "str",\n  "vehicle_make": "str",\n  "vehicle_make_name": "str",\n  "vehicle_model": "str",\n  "make_model_combined": "str",\n  "make_model_combined_name": "str",\n  "body_type": "str",\n  "body_type_name": "str",\n  "vehicle_model_year": "str",\n  "vehicle_model_year_name": "str",\n  "vehicle_identification_number_vin": "str",\n  "vehicle_identification_number_vin_name": "str",\n  "vin_character_1": "str",\n  "vin_character_2": "str",\n  "vin_character_3": "str",\n  "vin_character_4": "str",\n  "vin_character_5": "str",\n  "vin_character_6": "str",\n  "vin_character_7": "str",\n  "vin_character_8": "str",\n  "vin_character_9": "str",\n  "vin_character_10": "str",\n  "vin_character_11": "str",\n  "vin_character_12": "str",\n  "vehicle_trailing": "str",\n  "vehicle_trailing_name": "str",\n  "jackknife": "str",\n  "jackknife_name": "str",\n  "mcid_issuing_authority": "str",\n  "mcid_issuing_authority_name": "str",\n  "mcid_identification_number": "str",\n  "mcid_identification_number_name": "str",\n  "motor_carrier_identification_number_mcid": "str",\n  "motor_carrier_identification_number_mcid_name": "str",\n  "gross_vehicle_weight_rating": "str",\n  "gross_vehicle_weight_rating_name": "str",\n  "vehicle_configuration": "str",\n  "vehicle_configuration_name": "str",\n  "cargo_body_type": "str",\n  "cargo_body_type_name": "str",\n  "hazardous_material_involvement": "str",\n  "hazardous_material_involvement_name": "str",\n  "hazardous_material_placard": "str",\n  "hazardous_material_placard_name": "str",\n  "hazardous_material_identification_number": "str",\n  "hazardous_material_identification_number_name": "str",\n  "hazardous_material_class_number": "str",\n  "hazardous_material_class_number_name": "str",\n  "release_of_hazardous_material_from_the_cargo_compartment": "str",\n  "release_of_hazardous_material_from_the_cargo_compartment_name": "str",\n  "bus_use": "str",\n  "bus_use_name": "str",\n  "special_use": "str",\n  "special_use_name": "str",\n  "emergency_motor_vehicle_use": "str",\n  "emergency_motor_vehicle_use_name": "str",\n  "travel_speed": "str",\n  "travel_speed_name": "str",\n  "underride_override": "str",\n  "underride_override_name": "str",\n  "rollover": "str",\n  "rollover_name": "str",\n  "location_of_rollover": "str",\n  "location_of_rollover_name": "str",\n  "initial_contact_point": "str",\n  "initial_contact_point_name": "str",\n  "extent_of_damage": "str",\n  "extent_of_damage_name": "str",\n  "vehicle_removal": "str",\n  "vehicle_removal_name": "str",\n  "most_harmful_event": "str",\n  "most_harmful_event_name": "str",\n  "related_factors_vehicle_level_1": "str",\n  "related_factors_vehicle_level_1_name": "str",\n  "related_factors_vehicle_level_2": "str",\n  "related_factors_vehicle_level_2_name": "str",\n  "fire_occurrence": "str",\n  "fire_occurrence_name": "str",\n  "driver_presence": "str",\n  "driver_presence_name": "str",\n  "drivers_license_state": "str",\n  "drivers_license_state_name": "str",\n  "drivers_zip_code": "str",\n  "drivers_zip_code_name": "str",\n  "non_cdl_license_status": "str",\n  "non_cdl_license_status_name": "str",\n  "non_cdl_license_type": "str",\n  "non_cdl_license_type_name": "str",\n  "commercial_motor_vehicle_license_status": "str",\n  "commercial_motor_vehicle_license_status_name": "str",\n  "compliance_with_cdl_endorsements": "str",\n  "compliance_with_cdl_endorsements_name": "str",\n  "license_compliance_with_class_of_vehicle": "str",\n  "license_compliance_with_class_of_vehicle_name": "str",\n  "compliance_with_license_restrictions": "str",\n  "compliance_with_license_restrictions_name": "str",\n  "driver_height": "str",\n  "driver_height_name": "str",\n  "driver_weight": "str",\n  "driver_weight_name": "str",\n  "previous_recorded_crashes": "str",\n  "previous_recorded_crashes_name": "str",\n  "previous_recorded_suspensions_and_revocations": "str",\n  "previous_recorded_suspensions_and_revocations_name": "str",\n  "previous_dwi_convictions": "str",\n  "previous_dwi_convictions_name": "str",\n  "previous_speeding_convictions": "str",\n  "previous_speeding_convictions_name": "str",\n  "previous_other_moving_violation_convictions": "str",\n  "previous_other_moving_violation_convictions_name": "str",\n  "month_of_first_crash_suspension_or_conviction": "str",\n  "month_of_first_crash_suspension_or_conviction_name": "str",\n  "year_of_first_crash_suspension_or_conviction": "str",\n  "year_of_first_crash_suspension_or_conviction_name": "str",\n  "month_of_last_crash_suspension_or_conviction": "str",\n  "month_of_last_crash_suspension_or_conviction_name": "str",\n  "year_of_last_crash_suspension_or_conviction": "str",\n  "year_of_last_crash_suspension_or_conviction_name": "str",\n  "speeding_related": "str",\n  "speeding_related_name": "str",\n  "related_factors_driver_level_1": "str",\n  "related_factors_driver_level_1_name": "str",\n  "related_factors_driver_level_2": "str",\n  "related_factors_driver_level_2_name": "str",\n  "related_factors_driver_level_3": "str",\n  "related_factors_driver_level_3_name": "str",\n  "related_factors_driver_level_4": "str",\n  "related_factors_driver_level_4_name": "str",\n  "trafficway_description": "str",\n  "trafficway_description_name": "str",\n  "total_lanes_in_roadway": "str",\n  "total_lanes_in_roadway_name": "str",\n  "speed_limit": "str",\n  "speed_limit_name": "str",\n  "roadway_alignment": "str",\n  "roadway_alignment_name": "str",\n  "roadway_grade": "str",\n  "roadway_grade_name": "str",\n  "roadway_surface_type": "str",\n  "roadway_surface_type_name": "str",\n  "roadway_surface_condition": "str",\n  "roadway_surface_condition_name": "str",\n  "traffic_control_device": "str",\n  "traffic_control_device_name": "str",\n  "traffic_control_device_functioning": "str",\n  "traffic_control_device_functioning_name": "str",\n  "pre_event_movement_prior_to_recognition_of_critical_event": "str",\n  "pre_event_movement_prior_to_recognition_of_critical_event_name": "str",\n  "critical_event_precrash": "str",\n  "critical_event_precrash_name": "str",\n  "attempted_avoidance_maneuver": "str",\n  "attempted_avoidance_maneuver_name": "str",\n  "pre_impact_stability": "str",\n  "pre_impact_stability_name": "str",\n  "pre_impact_location": "str",\n  "pre_impact_location_name": "str",\n  "crash_type": "str",\n  "crash_type_name": "str",\n  "fatalities_in_vehicle": "str",\n  "driver_drinking": "str",\n  "driver_drinking_name": "str"\n}',
            "RENAME_MAPPINGS_LIST": '{\n  "STATE": "state_number",\n  "STATENAME": "state_name",\n  "ST_CASE": "consecutive_number",\n  "VEH_NO": "vehicle_number",\n  "VE_FORMS": "number_of_motor_vehicles_in_transport_mvit",\n  "NUMOCCS": "number_of_occupants",\n  "NUMOCCSNAME": "number_of_occupants_name",\n  "DAY": "day_of_crash",\n  "DAYNAME": "day_of_crash_name",\n  "MONTH": "month_of_crash",\n  "MONTHNAME": "month_of_crash_name",\n  "HOUR": "hour_of_crash",\n  "HOURNAME": "hour_of_crash_name",\n  "MINUTE": "minute_of_crash",\n  "MINUTENAME": "minute_of_crash_name",\n  "HARM_EV": "first_harmful_event",\n  "HARM_EVNAME": "first_harmful_event_name",\n  "MAN_COLL": "manner_of_collision",\n  "MAN_COLLNAME": "manner_of_collision_name",\n  "UNITTYPE": "unit_type",\n  "UNITTYPENAME": "unit_type_name",\n  "HIT_RUN": "hit_and_run",\n  "HIT_RUNNAME": "hit_and_run_name",\n  "REG_STAT": "registration_state",\n  "REG_STATNAME": "registration_state_name",\n  "OWNER": "registered_vehicle_owner",\n  "OWNERNAME": "registered_vehicle_owner_name",\n  "MAKE": "vehicle_make",\n  "MAKENAME": "vehicle_make_name",\n  "MODEL": "vehicle_model",\n  "MAK_MOD": "make_model_combined",\n  "MAK_MODNAME": "make_model_combined_name",\n  "BODY_TYP": "body_type",\n  "BODY_TYPNAME": "body_type_name",\n  "MOD_YEAR": "vehicle_model_year",\n  "MOD_YEARNAME": "vehicle_model_year_name",\n  "VIN": "vehicle_identification_number_vin",\n  "VINNAME": "vehicle_identification_number_vin_name",\n  "VIN_1": "vin_character_1",\n  "VIN_2": "vin_character_2",\n  "VIN_3": "vin_character_3",\n  "VIN_4": "vin_character_4",\n  "VIN_5": "vin_character_5",\n  "VIN_6": "vin_character_6",\n  "VIN_7": "vin_character_7",\n  "VIN_8": "vin_character_8",\n  "VIN_9": "vin_character_9",\n  "VIN_10": "vin_character_10",\n  "VIN_11": "vin_character_11",\n  "VIN_12": "vin_character_12",\n  "TOW_VEH": "vehicle_trailing",\n  "TOW_VEHNAME": "vehicle_trailing_name",\n  "J_KNIFE": "jackknife",\n  "J_KNIFENAME": "jackknife_name",\n  "MCARR_I1": "mcid_issuing_authority",\n  "MCARR_I1NAME": "mcid_issuing_authority_name",\n  "MCARR_I2": "mcid_identification_number",\n  "MCARR_I2NAME": "mcid_identification_number_name",\n  "MCARR_ID": "motor_carrier_identification_number_mcid",\n  "MCARR_IDNAME": "motor_carrier_identification_number_mcid_name",\n  "GVWR": "gross_vehicle_weight_rating",\n  "GVWRNAME": "gross_vehicle_weight_rating_name",\n  "V_CONFIG": "vehicle_configuration",\n  "V_CONFIGNAME": "vehicle_configuration_name",\n  "CARGO_BT": "cargo_body_type",\n  "CARGO_BTNAME": "cargo_body_type_name",\n  "HAZ_INV": "hazardous_material_involvement",\n  "HAZ_INVNAME": "hazardous_material_involvement_name",\n  "HAZ_PLAC": "hazardous_material_placard",\n  "HAZ_PLACNAME": "hazardous_material_placard_name",\n  "HAZ_ID": "hazardous_material_identification_number",\n  "HAZ_IDNAME": "hazardous_material_identification_number_name",\n  "HAZ_CNO": "hazardous_material_class_number",\n  "HAZ_CNONAME": "hazardous_material_class_number_name",\n  "HAZ_REL": "release_of_hazardous_material_from_the_cargo_compartment",\n  "HAZ_RELNAME": "release_of_hazardous_material_from_the_cargo_compartment_name",\n  "BUS_USE": "bus_use",\n  "BUS_USENAME": "bus_use_name",\n  "SPEC_USE": "special_use",\n  "SPEC_USENAME": "special_use_name",\n  "EMER_USE": "emergency_motor_vehicle_use",\n  "EMER_USENAME": "emergency_motor_vehicle_use_name",\n  "TRAV_SP": "travel_speed",\n  "TRAV_SPNAME": "travel_speed_name",\n  "UNDERIDE": "underride_override",\n  "UNDERIDENAME": "underride_override_name",\n  "ROLLOVER": "rollover",\n  "ROLLOVERNAME": "rollover_name",\n  "ROLINLOC": "location_of_rollover",\n  "ROLINLOCNAME": "location_of_rollover_name",\n  "IMPACT1": "initial_contact_point",\n  "IMPACT1NAME": "initial_contact_point_name",\n  "DEFORMED": "extent_of_damage",\n  "DEFORMEDNAME": "extent_of_damage_name",\n  "TOWED": "vehicle_removal",\n  "TOWEDNAME": "vehicle_removal_name",\n  "M_HARM": "most_harmful_event",\n  "M_HARMNAME": "most_harmful_event_name",\n  "VEH_SC1": "related_factors_vehicle_level_1",\n  "VEH_SC1NAME": "related_factors_vehicle_level_1_name",\n  "VEH_SC2": "related_factors_vehicle_level_2",\n  "VEH_SC2NAME": "related_factors_vehicle_level_2_name",\n  "FIRE_EXP": "fire_occurrence",\n  "FIRE_EXPNAME": "fire_occurrence_name",\n  "DR_PRES": "driver_presence",\n  "DR_PRESNAME": "driver_presence_name",\n  "L_STATE": "drivers_license_state",\n  "L_STATENAME": "drivers_license_state_name",\n  "DR_ZIP": "drivers_zip_code",\n  "DR_ZIPNAME": "drivers_zip_code_name",\n  "L_STATUS": "non_cdl_license_status",\n  "L_STATUSNAME": "non_cdl_license_status_name",\n  "L_TYPE": "non_cdl_license_type",\n  "L_TYPENAME": "non_cdl_license_type_name",\n  "CDL_STAT": "commercial_motor_vehicle_license_status",\n  "CDL_STATNAME": "commercial_motor_vehicle_license_status_name",\n  "L_ENDORS": "compliance_with_cdl_endorsements",\n  "L_ENDORSNAME": "compliance_with_cdl_endorsements_name",\n  "L_COMPL": "license_compliance_with_class_of_vehicle",\n  "L_COMPLNAME": "license_compliance_with_class_of_vehicle_name",\n  "L_RESTRI": "compliance_with_license_restrictions",\n  "L_RESTRINAME": "compliance_with_license_restrictions_name",\n  "DR_HGT": "driver_height",\n  "DR_HGTNAME": "driver_height_name",\n  "DR_WGT": "driver_weight",\n  "DR_WGTNAME": "driver_weight_name",\n  "PREV_ACC": "previous_recorded_crashes",\n  "PREV_ACCNAME": "previous_recorded_crashes_name",\n  "PREV_SUS": "previous_recorded_suspensions_and_revocations",\n  "PREV_SUSNAME": "previous_recorded_suspensions_and_revocations_name",\n  "PREV_DWI": "previous_dwi_convictions",\n  "PREV_DWINAME": "previous_dwi_convictions_name",\n  "PREV_SPD": "previous_speeding_convictions",\n  "PREV_SPDNAME": "previous_speeding_convictions_name",\n  "PREV_OTH": "previous_other_moving_violation_convictions",\n  "PREV_OTHNAME": "previous_other_moving_violation_convictions_name",\n  "FIRST_MO": "month_of_first_crash_suspension_or_conviction",\n  "FIRST_MONAME": "month_of_first_crash_suspension_or_conviction_name",\n  "FIRST_YR": "year_of_first_crash_suspension_or_conviction",\n  "FIRST_YRNAME": "year_of_first_crash_suspension_or_conviction_name",\n  "LAST_MO": "month_of_last_crash_suspension_or_conviction",\n  "LAST_MONAME": "month_of_last_crash_suspension_or_conviction_name",\n  "LAST_YR": "year_of_last_crash_suspension_or_conviction",\n  "LAST_YRNAME": "year_of_last_crash_suspension_or_conviction_name",\n  "SPEEDREL": "speeding_related",\n  "SPEEDRELNAME": "speeding_related_name",\n  "DR_SF1": "related_factors_driver_level_1",\n  "DR_SF1NAME": "related_factors_driver_level_1_name",\n  "DR_SF2": "related_factors_driver_level_2",\n  "DR_SF2NAME": "related_factors_driver_level_2_name",\n  "DR_SF3": "related_factors_driver_level_3",\n  "DR_SF3NAME": "related_factors_driver_level_3_name",\n  "DR_SF4": "related_factors_driver_level_4",\n  "DR_SF4NAME": "related_factors_driver_level_4_name",\n  "VTRAFWAY": "trafficway_description",\n  "VTRAFWAYNAME": "trafficway_description_name",\n  "VNUM_LAN": "total_lanes_in_roadway",\n  "VNUM_LANNAME": "total_lanes_in_roadway_name",\n  "VSPD_LIM": "speed_limit",\n  "VSPD_LIMNAME": "speed_limit_name",\n  "VALIGN": "roadway_alignment",\n  "VALIGNNAME": "roadway_alignment_name",\n  "VPROFILE": "roadway_grade",\n  "VPROFILENAME": "roadway_grade_name",\n  "VPAVETYP": "roadway_surface_type",\n  "VPAVETYPNAME": "roadway_surface_type_name",\n  "VSURCOND": "roadway_surface_condition",\n  "VSURCONDNAME": "roadway_surface_condition_name",\n  "VTRAFCON": "traffic_control_device",\n  "VTRAFCONNAME": "traffic_control_device_name",\n  "VTCONT_F": "traffic_control_device_functioning",\n  "VTCONT_FNAME": "traffic_control_device_functioning_name",\n  "P_CRASH1": "pre_event_movement_prior_to_recognition_of_critical_event",\n  "P_CRASH1NAME": "pre_event_movement_prior_to_recognition_of_critical_event_name",\n  "P_CRASH2": "critical_event_precrash",\n  "P_CRASH2NAME": "critical_event_precrash_name",\n  "P_CRASH3": "attempted_avoidance_maneuver",\n  "P_CRASH3NAME": "attempted_avoidance_maneuver_name",\n  "PCRASH4": "pre_impact_stability",\n  "PCRASH4NAME": "pre_impact_stability_name",\n  "PCRASH5": "pre_impact_location",\n  "PCRASH5NAME": "pre_impact_location_name",\n  "ACC_TYPE": "crash_type",\n  "ACC_TYPENAME": "crash_type_name",\n  "DEATHS": "fatalities_in_vehicle",\n  "DR_DRINK": "driver_drinking",\n  "DR_DRINKNAME": "driver_drinking_name"\n}',
        },
        resources={
            "request_ephemeral_storage": "4G",
            "request_cpu": "1",
            "request_memory": "4G",
        },
    )

    # Run CSV transform within kubernetes pod for vehicle pipelines
    vehicle_2016_2017_transform_csv = kubernetes_pod.KubernetesPodOperator(
        task_id="vehicle_2016_2017_transform_csv",
        startup_timeout_seconds=600,
        name="vehicle",
        namespace="composer",
        service_account_name="datasets",
        image_pull_policy="Always",
        image="{{ var.json.nhtsa_traffic_fatalities.container_registry.run_csv_transform_kub }}",
        env_vars={
            "PIPELINE_NAME": "{{ var.json.nhtsa_traffic_fatalities.vehicle_2016_2017.pipeline_name }}",
            "SOURCE_URL": "{{ var.json.nhtsa_traffic_fatalities.vehicle_2016_2017.source_url }}",
            "CHUNKSIZE": "{{ var.json.nhtsa_traffic_fatalities.vehicle_2016_2017.chunksize }}",
            "SOURCE_ZIPFILE_EXTRACTED": "vehicle.csv",
            "SOURCE_FILE": "{{ var.json.nhtsa_traffic_fatalities.vehicle_2016_2017.source_file }}",
            "PROJECT_ID": "{{ var.value.gcp_project }}",
            "DATASET_ID": "{{ var.json.nhtsa_traffic_fatalities.vehicle_2016_2017.dataset_id }}",
            "TABLE_ID": "{{ var.json.nhtsa_traffic_fatalities.vehicle_2016_2017.destination_table }}",
            "START_YEAR": "{{ var.json.nhtsa_traffic_fatalities.vehicle_2016_2017.start_year }}",
            "END_YEAR": "{{ var.json.nhtsa_traffic_fatalities.vehicle_2016_2017.end_year }}",
            "DROP_DEST_TABLE": "{{ var.json.nhtsa_traffic_fatalities.vehicle_2016_2017.drop_dest_table }}",
            "TARGET_GCS_BUCKET": "{{ var.value.composer_bucket }}",
            "TARGET_GCS_PATH": "{{ var.json.nhtsa_traffic_fatalities.vehicle_2016_2017.target_gcs_path }}",
            "SCHEMA_PATH": "{{ var.json.nhtsa_traffic_fatalities.vehicle_2016_2017.schema_path }}",
            "INPUT_CSV_HEADERS": '[\n  "state_number",\n  "state_name",\n  "consecutive_number",\n  "vehicle_number",\n  "number_of_motor_vehicles_in_transport_mvit",\n  "number_of_occupants",\n  "number_of_occupants_name",\n  "day_of_crash",\n  "day_of_crash_name",\n  "month_of_crash",\n  "month_of_crash_name",\n  "hour_of_crash",\n  "hour_of_crash_name",\n  "minute_of_crash",\n  "minute_of_crash_name",\n  "first_harmful_event",\n  "first_harmful_event_name",\n  "manner_of_collision",\n  "manner_of_collision_name",\n  "unit_type",\n  "unit_type_name",\n  "hit_and_run",\n  "hit_and_run_name",\n  "registration_state",\n  "registration_state_name",\n  "registered_vehicle_owner",\n  "registered_vehicle_owner_name",\n  "vehicle_make",\n  "vehicle_make_name",\n  "vehicle_model",\n  "make_model_combined",\n  "make_model_combined_name",\n  "body_type",\n  "body_type_name",\n  "vehicle_model_year",\n  "vehicle_model_year_name",\n  "vehicle_identification_number_vin",\n  "vehicle_identification_number_vin_name",\n  "vin_character_1",\n  "vin_character_2",\n  "vin_character_3",\n  "vin_character_4",\n  "vin_character_5",\n  "vin_character_6",\n  "vin_character_7",\n  "vin_character_8",\n  "vin_character_9",\n  "vin_character_10",\n  "vin_character_11",\n  "vin_character_12",\n  "vehicle_trailing",\n  "vehicle_trailing_name",\n  "jackknife",\n  "jackknife_name",\n  "mcid_issuing_authority",\n  "mcid_issuing_authority_name",\n  "mcid_identification_number",\n  "mcid_identification_number_name",\n  "motor_carrier_identification_number_mcid",\n  "motor_carrier_identification_number_mcid_name",\n  "gross_vehicle_weight_rating",\n  "gross_vehicle_weight_rating_name",\n  "vehicle_configuration",\n  "vehicle_configuration_name",\n  "cargo_body_type",\n  "cargo_body_type_name",\n  "hazardous_material_involvement",\n  "hazardous_material_involvement_name",\n  "hazardous_material_placard",\n  "hazardous_material_placard_name",\n  "hazardous_material_identification_number",\n  "hazardous_material_identification_number_name",\n  "hazardous_material_class_number",\n  "hazardous_material_class_number_name",\n  "release_of_hazardous_material_from_the_cargo_compartment",\n  "release_of_hazardous_material_from_the_cargo_compartment_name",\n  "bus_use",\n  "bus_use_name",\n  "special_use",\n  "special_use_name",\n  "emergency_motor_vehicle_use",\n  "emergency_motor_vehicle_use_name",\n  "travel_speed",\n  "travel_speed_name",\n  "underride_override",\n  "underride_override_name",\n  "rollover",\n  "rollover_name",\n  "location_of_rollover",\n  "location_of_rollover_name",\n  "initial_contact_point",\n  "initial_contact_point_name",\n  "extent_of_damage",\n  "extent_of_damage_name",\n  "vehicle_removal",\n  "vehicle_removal_name",\n  "most_harmful_event",\n  "most_harmful_event_name",\n  "related_factors_vehicle_level_1",\n  "related_factors_vehicle_level_1_name",\n  "related_factors_vehicle_level_2",\n  "related_factors_vehicle_level_2_name",\n  "fire_occurrence",\n  "fire_occurrence_name",\n  "driver_presence",\n  "driver_presence_name",\n  "drivers_license_state",\n  "drivers_license_state_name",\n  "drivers_zip_code",\n  "drivers_zip_code_name",\n  "non_cdl_license_status",\n  "non_cdl_license_status_name",\n  "non_cdl_license_type",\n  "non_cdl_license_type_name",\n  "commercial_motor_vehicle_license_status",\n  "commercial_motor_vehicle_license_status_name",\n  "compliance_with_cdl_endorsements",\n  "compliance_with_cdl_endorsements_name",\n  "license_compliance_with_class_of_vehicle",\n  "license_compliance_with_class_of_vehicle_name",\n  "compliance_with_license_restrictions",\n  "compliance_with_license_restrictions_name",\n  "driver_height",\n  "driver_height_name",\n  "driver_weight",\n  "driver_weight_name",\n  "previous_recorded_crashes",\n  "previous_recorded_crashes_name",\n  "previous_recorded_suspensions_and_revocations",\n  "previous_recorded_suspensions_and_revocations_name",\n  "previous_dwi_convictions",\n  "previous_dwi_convictions_name",\n  "previous_speeding_convictions",\n  "previous_speeding_convictions_name",\n  "previous_other_moving_violation_convictions",\n  "previous_other_moving_violation_convictions_name",\n  "month_of_first_crash_suspension_or_conviction",\n  "month_of_first_crash_suspension_or_conviction_name",\n  "year_of_first_crash_suspension_or_conviction",\n  "year_of_first_crash_suspension_or_conviction_name",\n  "month_of_last_crash_suspension_or_conviction",\n  "month_of_last_crash_suspension_or_conviction_name",\n  "year_of_last_crash_suspension_or_conviction",\n  "year_of_last_crash_suspension_or_conviction_name",\n  "speeding_related",\n  "speeding_related_name",\n  "related_factors_driver_level_1",\n  "related_factors_driver_level_1_name",\n  "related_factors_driver_level_2",\n  "related_factors_driver_level_2_name",\n  "related_factors_driver_level_3",\n  "related_factors_driver_level_3_name",\n  "related_factors_driver_level_4",\n  "related_factors_driver_level_4_name",\n  "trafficway_description",\n  "trafficway_description_name",\n  "total_lanes_in_roadway",\n  "total_lanes_in_roadway_name",\n  "speed_limit",\n  "speed_limit_name",\n  "roadway_alignment",\n  "roadway_alignment_name",\n  "roadway_grade",\n  "roadway_grade_name",\n  "roadway_surface_type",\n  "roadway_surface_type_name",\n  "roadway_surface_condition",\n  "roadway_surface_condition_name",\n  "traffic_control_device",\n  "traffic_control_device_name",\n  "traffic_control_device_functioning",\n  "traffic_control_device_functioning_name",\n  "pre_event_movement_prior_to_recognition_of_critical_event",\n  "pre_event_movement_prior_to_recognition_of_critical_event_name",\n  "critical_event_precrash",\n  "critical_event_precrash_name",\n  "attempted_avoidance_maneuver",\n  "attempted_avoidance_maneuver_name",\n  "pre_impact_stability",\n  "pre_impact_stability_name",\n  "pre_impact_location",\n  "pre_impact_location_name",\n  "crash_type",\n  "crash_type_name",\n  "fatalities_in_vehicle",\n  "driver_drinking",\n  "driver_drinking_name",\n  "trailer_vehicle_identification_number_1",\n  "trailer_vehicle_identification_number_1_name",\n  "trailer_vehicle_identification_number_2",\n  "trailer_vehicle_identification_number_2_name",\n  "trailer_vehicle_identification_number_3",\n  "trailer_vehicle_identification_number_3_name"\n]',
            "INPUT_DTYPES": '{\n  "state_number": "str",\n  "state_name": "str",\n  "consecutive_number": "str",\n  "vehicle_number": "str",\n  "number_of_motor_vehicles_in_transport_mvit": "str",\n  "number_of_occupants": "str",\n  "number_of_occupants_name": "str",\n  "day_of_crash": "str",\n  "day_of_crash_name": "str",\n  "month_of_crash": "str",\n  "month_of_crash_name": "str",\n  "hour_of_crash": "str",\n  "hour_of_crash_name": "str",\n  "minute_of_crash": "str",\n  "minute_of_crash_name": "str",\n  "first_harmful_event": "str",\n  "first_harmful_event_name": "str",\n  "manner_of_collision": "str",\n  "manner_of_collision_name": "str",\n  "unit_type": "str",\n  "unit_type_name": "str",\n  "hit_and_run": "str",\n  "hit_and_run_name": "str",\n  "registration_state": "str",\n  "registration_state_name": "str",\n  "registered_vehicle_owner": "str",\n  "registered_vehicle_owner_name": "str",\n  "vehicle_make": "str",\n  "vehicle_make_name": "str",\n  "vehicle_model": "str",\n  "make_model_combined": "str",\n  "make_model_combined_name": "str",\n  "body_type": "str",\n  "body_type_name": "str",\n  "vehicle_model_year": "str",\n  "vehicle_model_year_name": "str",\n  "vehicle_identification_number_vin": "str",\n  "vehicle_identification_number_vin_name": "str",\n  "vin_character_1": "str",\n  "vin_character_2": "str",\n  "vin_character_3": "str",\n  "vin_character_4": "str",\n  "vin_character_5": "str",\n  "vin_character_6": "str",\n  "vin_character_7": "str",\n  "vin_character_8": "str",\n  "vin_character_9": "str",\n  "vin_character_10": "str",\n  "vin_character_11": "str",\n  "vin_character_12": "str",\n  "vehicle_trailing": "str",\n  "vehicle_trailing_name": "str",\n  "jackknife": "str",\n  "jackknife_name": "str",\n  "mcid_issuing_authority": "str",\n  "mcid_issuing_authority_name": "str",\n  "mcid_identification_number": "str",\n  "mcid_identification_number_name": "str",\n  "motor_carrier_identification_number_mcid": "str",\n  "motor_carrier_identification_number_mcid_name": "str",\n  "gross_vehicle_weight_rating": "str",\n  "gross_vehicle_weight_rating_name": "str",\n  "vehicle_configuration": "str",\n  "vehicle_configuration_name": "str",\n  "cargo_body_type": "str",\n  "cargo_body_type_name": "str",\n  "hazardous_material_involvement": "str",\n  "hazardous_material_involvement_name": "str",\n  "hazardous_material_placard": "str",\n  "hazardous_material_placard_name": "str",\n  "hazardous_material_identification_number": "str",\n  "hazardous_material_identification_number_name": "str",\n  "hazardous_material_class_number": "str",\n  "hazardous_material_class_number_name": "str",\n  "release_of_hazardous_material_from_the_cargo_compartment": "str",\n  "release_of_hazardous_material_from_the_cargo_compartment_name": "str",\n  "bus_use": "str",\n  "bus_use_name": "str",\n  "special_use": "str",\n  "special_use_name": "str",\n  "emergency_motor_vehicle_use": "str",\n  "emergency_motor_vehicle_use_name": "str",\n  "travel_speed": "str",\n  "travel_speed_name": "str",\n  "underride_override": "str",\n  "underride_override_name": "str",\n  "rollover": "str",\n  "rollover_name": "str",\n  "location_of_rollover": "str",\n  "location_of_rollover_name": "str",\n  "initial_contact_point": "str",\n  "initial_contact_point_name": "str",\n  "extent_of_damage": "str",\n  "extent_of_damage_name": "str",\n  "vehicle_removal": "str",\n  "vehicle_removal_name": "str",\n  "most_harmful_event": "str",\n  "most_harmful_event_name": "str",\n  "related_factors_vehicle_level_1": "str",\n  "related_factors_vehicle_level_1_name": "str",\n  "related_factors_vehicle_level_2": "str",\n  "related_factors_vehicle_level_2_name": "str",\n  "fire_occurrence": "str",\n  "fire_occurrence_name": "str",\n  "driver_presence": "str",\n  "driver_presence_name": "str",\n  "drivers_license_state": "str",\n  "drivers_license_state_name": "str",\n  "drivers_zip_code": "str",\n  "drivers_zip_code_name": "str",\n  "non_cdl_license_status": "str",\n  "non_cdl_license_status_name": "str",\n  "non_cdl_license_type": "str",\n  "non_cdl_license_type_name": "str",\n  "commercial_motor_vehicle_license_status": "str",\n  "commercial_motor_vehicle_license_status_name": "str",\n  "compliance_with_cdl_endorsements": "str",\n  "compliance_with_cdl_endorsements_name": "str",\n  "license_compliance_with_class_of_vehicle": "str",\n  "license_compliance_with_class_of_vehicle_name": "str",\n  "compliance_with_license_restrictions": "str",\n  "compliance_with_license_restrictions_name": "str",\n  "driver_height": "str",\n  "driver_height_name": "str",\n  "driver_weight": "str",\n  "driver_weight_name": "str",\n  "previous_recorded_crashes": "str",\n  "previous_recorded_crashes_name": "str",\n  "previous_recorded_suspensions_and_revocations": "str",\n  "previous_recorded_suspensions_and_revocations_name": "str",\n  "previous_dwi_convictions": "str",\n  "previous_dwi_convictions_name": "str",\n  "previous_speeding_convictions": "str",\n  "previous_speeding_convictions_name": "str",\n  "previous_other_moving_violation_convictions": "str",\n  "previous_other_moving_violation_convictions_name": "str",\n  "month_of_first_crash_suspension_or_conviction": "str",\n  "month_of_first_crash_suspension_or_conviction_name": "str",\n  "year_of_first_crash_suspension_or_conviction": "str",\n  "year_of_first_crash_suspension_or_conviction_name": "str",\n  "month_of_last_crash_suspension_or_conviction": "str",\n  "month_of_last_crash_suspension_or_conviction_name": "str",\n  "year_of_last_crash_suspension_or_conviction": "str",\n  "year_of_last_crash_suspension_or_conviction_name": "str",\n  "speeding_related": "str",\n  "speeding_related_name": "str",\n  "related_factors_driver_level_1": "str",\n  "related_factors_driver_level_1_name": "str",\n  "related_factors_driver_level_2": "str",\n  "related_factors_driver_level_2_name": "str",\n  "related_factors_driver_level_3": "str",\n  "related_factors_driver_level_3_name": "str",\n  "related_factors_driver_level_4": "str",\n  "related_factors_driver_level_4_name": "str",\n  "trafficway_description": "str",\n  "trafficway_description_name": "str",\n  "total_lanes_in_roadway": "str",\n  "total_lanes_in_roadway_name": "str",\n  "speed_limit": "str",\n  "speed_limit_name": "str",\n  "roadway_alignment": "str",\n  "roadway_alignment_name": "str",\n  "roadway_grade": "str",\n  "roadway_grade_name": "str",\n  "roadway_surface_type": "str",\n  "roadway_surface_type_name": "str",\n  "roadway_surface_condition": "str",\n  "roadway_surface_condition_name": "str",\n  "traffic_control_device": "str",\n  "traffic_control_device_name": "str",\n  "traffic_control_device_functioning": "str",\n  "traffic_control_device_functioning_name": "str",\n  "pre_event_movement_prior_to_recognition_of_critical_event": "str",\n  "pre_event_movement_prior_to_recognition_of_critical_event_name": "str",\n  "critical_event_precrash": "str",\n  "critical_event_precrash_name": "str",\n  "attempted_avoidance_maneuver": "str",\n  "attempted_avoidance_maneuver_name": "str",\n  "pre_impact_stability": "str",\n  "pre_impact_stability_name": "str",\n  "pre_impact_location": "str",\n  "pre_impact_location_name": "str",\n  "crash_type": "str",\n  "crash_type_name": "str",\n  "fatalities_in_vehicle": "str",\n  "driver_drinking": "str",\n  "driver_drinking_name": "str",\n  "trailer_vehicle_identification_number_1": "str",\n  "trailer_vehicle_identification_number_1_name": "str",\n  "trailer_vehicle_identification_number_2": "str",\n  "trailer_vehicle_identification_number_2_name": "str",\n  "trailer_vehicle_identification_number_3": "str",\n  "trailer_vehicle_identification_number_3_name": "str"\n}',
            "RENAME_MAPPINGS_LIST": '{\n  "STATE": "state_number",\n  "STATENAME": "state_name",\n  "ST_CASE": "consecutive_number",\n  "VEH_NO": "vehicle_number",\n  "VE_FORMS": "number_of_motor_vehicles_in_transport_mvit",\n  "NUMOCCS": "number_of_occupants",\n  "NUMOCCSNAME": "number_of_occupants_name",\n  "DAY": "day_of_crash",\n  "DAYNAME": "day_of_crash_name",\n  "MONTH": "month_of_crash",\n  "MONTHNAME": "month_of_crash_name",\n  "HOUR": "hour_of_crash",\n  "HOURNAME": "hour_of_crash_name",\n  "MINUTE": "minute_of_crash",\n  "MINUTENAME": "minute_of_crash_name",\n  "HARM_EV": "first_harmful_event",\n  "HARM_EVNAME": "first_harmful_event_name",\n  "MAN_COLL": "manner_of_collision",\n  "MAN_COLLNAME": "manner_of_collision_name",\n  "UNITTYPE": "unit_type",\n  "UNITTYPENAME": "unit_type_name",\n  "HIT_RUN": "hit_and_run",\n  "HIT_RUNNAME": "hit_and_run_name",\n  "REG_STAT": "registration_state",\n  "REG_STATNAME": "registration_state_name",\n  "OWNER": "registered_vehicle_owner",\n  "OWNERNAME": "registered_vehicle_owner_name",\n  "MAKE": "vehicle_make",\n  "MAKENAME": "vehicle_make_name",\n  "MODEL": "vehicle_model",\n  "MAK_MOD": "make_model_combined",\n  "MAK_MODNAME": "make_model_combined_name",\n  "BODY_TYP": "body_type",\n  "BODY_TYPNAME": "body_type_name",\n  "MOD_YEAR": "vehicle_model_year",\n  "MOD_YEARNAME": "vehicle_model_year_name",\n  "VIN": "vehicle_identification_number_vin",\n  "VINNAME": "vehicle_identification_number_vin_name",\n  "VIN_1": "vin_character_1",\n  "VIN_2": "vin_character_2",\n  "VIN_3": "vin_character_3",\n  "VIN_4": "vin_character_4",\n  "VIN_5": "vin_character_5",\n  "VIN_6": "vin_character_6",\n  "VIN_7": "vin_character_7",\n  "VIN_8": "vin_character_8",\n  "VIN_9": "vin_character_9",\n  "VIN_10": "vin_character_10",\n  "VIN_11": "vin_character_11",\n  "VIN_12": "vin_character_12",\n  "TOW_VEH": "vehicle_trailing",\n  "TOW_VEHNAME": "vehicle_trailing_name",\n  "J_KNIFE": "jackknife",\n  "J_KNIFENAME": "jackknife_name",\n  "MCARR_I1": "mcid_issuing_authority",\n  "MCARR_I1NAME": "mcid_issuing_authority_name",\n  "MCARR_I2": "mcid_identification_number",\n  "MCARR_I2NAME": "mcid_identification_number_name",\n  "MCARR_ID": "motor_carrier_identification_number_mcid",\n  "MCARR_IDNAME": "motor_carrier_identification_number_mcid_name",\n  "GVWR": "gross_vehicle_weight_rating",\n  "GVWRNAME": "gross_vehicle_weight_rating_name",\n  "V_CONFIG": "vehicle_configuration",\n  "V_CONFIGNAME": "vehicle_configuration_name",\n  "CARGO_BT": "cargo_body_type",\n  "CARGO_BTNAME": "cargo_body_type_name",\n  "HAZ_INV": "hazardous_material_involvement",\n  "HAZ_INVNAME": "hazardous_material_involvement_name",\n  "HAZ_PLAC": "hazardous_material_placard",\n  "HAZ_PLACNAME": "hazardous_material_placard_name",\n  "HAZ_ID": "hazardous_material_identification_number",\n  "HAZ_IDNAME": "hazardous_material_identification_number_name",\n  "HAZ_CNO": "hazardous_material_class_number",\n  "HAZ_CNONAME": "hazardous_material_class_number_name",\n  "HAZ_REL": "release_of_hazardous_material_from_the_cargo_compartment",\n  "HAZ_RELNAME": "release_of_hazardous_material_from_the_cargo_compartment_name",\n  "BUS_USE": "bus_use",\n  "BUS_USENAME": "bus_use_name",\n  "SPEC_USE": "special_use",\n  "SPEC_USENAME": "special_use_name",\n  "EMER_USE": "emergency_motor_vehicle_use",\n  "EMER_USENAME": "emergency_motor_vehicle_use_name",\n  "TRAV_SP": "travel_speed",\n  "TRAV_SPNAME": "travel_speed_name",\n  "UNDERIDE": "underride_override",\n  "UNDERIDENAME": "underride_override_name",\n  "ROLLOVER": "rollover",\n  "ROLLOVERNAME": "rollover_name",\n  "ROLINLOC": "location_of_rollover",\n  "ROLINLOCNAME": "location_of_rollover_name",\n  "IMPACT1": "initial_contact_point",\n  "IMPACT1NAME": "initial_contact_point_name",\n  "DEFORMED": "extent_of_damage",\n  "DEFORMEDNAME": "extent_of_damage_name",\n  "TOWED": "vehicle_removal",\n  "TOWEDNAME": "vehicle_removal_name",\n  "M_HARM": "most_harmful_event",\n  "M_HARMNAME": "most_harmful_event_name",\n  "VEH_SC1": "related_factors_vehicle_level_1",\n  "VEH_SC1NAME": "related_factors_vehicle_level_1_name",\n  "VEH_SC2": "related_factors_vehicle_level_2",\n  "VEH_SC2NAME": "related_factors_vehicle_level_2_name",\n  "FIRE_EXP": "fire_occurrence",\n  "FIRE_EXPNAME": "fire_occurrence_name",\n  "DR_PRES": "driver_presence",\n  "DR_PRESNAME": "driver_presence_name",\n  "L_STATE": "drivers_license_state",\n  "L_STATENAME": "drivers_license_state_name",\n  "DR_ZIP": "drivers_zip_code",\n  "DR_ZIPNAME": "drivers_zip_code_name",\n  "L_STATUS": "non_cdl_license_status",\n  "L_STATUSNAME": "non_cdl_license_status_name",\n  "L_TYPE": "non_cdl_license_type",\n  "L_TYPENAME": "non_cdl_license_type_name",\n  "CDL_STAT": "commercial_motor_vehicle_license_status",\n  "CDL_STATNAME": "commercial_motor_vehicle_license_status_name",\n  "L_ENDORS": "compliance_with_cdl_endorsements",\n  "L_ENDORSNAME": "compliance_with_cdl_endorsements_name",\n  "L_COMPL": "license_compliance_with_class_of_vehicle",\n  "L_COMPLNAME": "license_compliance_with_class_of_vehicle_name",\n  "L_RESTRI": "compliance_with_license_restrictions",\n  "L_RESTRINAME": "compliance_with_license_restrictions_name",\n  "DR_HGT": "driver_height",\n  "DR_HGTNAME": "driver_height_name",\n  "DR_WGT": "driver_weight",\n  "DR_WGTNAME": "driver_weight_name",\n  "PREV_ACC": "previous_recorded_crashes",\n  "PREV_ACCNAME": "previous_recorded_crashes_name",\n  "PREV_SUS": "previous_recorded_suspensions_and_revocations",\n  "PREV_SUSNAME": "previous_recorded_suspensions_and_revocations_name",\n  "PREV_DWI": "previous_dwi_convictions",\n  "PREV_DWINAME": "previous_dwi_convictions_name",\n  "PREV_SPD": "previous_speeding_convictions",\n  "PREV_SPDNAME": "previous_speeding_convictions_name",\n  "PREV_OTH": "previous_other_moving_violation_convictions",\n  "PREV_OTHNAME": "previous_other_moving_violation_convictions_name",\n  "FIRST_MO": "month_of_first_crash_suspension_or_conviction",\n  "FIRST_MONAME": "month_of_first_crash_suspension_or_conviction_name",\n  "FIRST_YR": "year_of_first_crash_suspension_or_conviction",\n  "FIRST_YRNAME": "year_of_first_crash_suspension_or_conviction_name",\n  "LAST_MO": "month_of_last_crash_suspension_or_conviction",\n  "LAST_MONAME": "month_of_last_crash_suspension_or_conviction_name",\n  "LAST_YR": "year_of_last_crash_suspension_or_conviction",\n  "LAST_YRNAME": "year_of_last_crash_suspension_or_conviction_name",\n  "SPEEDREL": "speeding_related",\n  "SPEEDRELNAME": "speeding_related_name",\n  "DR_SF1": "related_factors_driver_level_1",\n  "DR_SF1NAME": "related_factors_driver_level_1_name",\n  "DR_SF2": "related_factors_driver_level_2",\n  "DR_SF2NAME": "related_factors_driver_level_2_name",\n  "DR_SF3": "related_factors_driver_level_3",\n  "DR_SF3NAME": "related_factors_driver_level_3_name",\n  "DR_SF4": "related_factors_driver_level_4",\n  "DR_SF4NAME": "related_factors_driver_level_4_name",\n  "VTRAFWAY": "trafficway_description",\n  "VTRAFWAYNAME": "trafficway_description_name",\n  "VNUM_LAN": "total_lanes_in_roadway",\n  "VNUM_LANNAME": "total_lanes_in_roadway_name",\n  "VSPD_LIM": "speed_limit",\n  "VSPD_LIMNAME": "speed_limit_name",\n  "VALIGN": "roadway_alignment",\n  "VALIGNNAME": "roadway_alignment_name",\n  "VPROFILE": "roadway_grade",\n  "VPROFILENAME": "roadway_grade_name",\n  "VPAVETYP": "roadway_surface_type",\n  "VPAVETYPNAME": "roadway_surface_type_name",\n  "VSURCOND": "roadway_surface_condition",\n  "VSURCONDNAME": "roadway_surface_condition_name",\n  "VTRAFCON": "traffic_control_device",\n  "VTRAFCONNAME": "traffic_control_device_name",\n  "VTCONT_F": "traffic_control_device_functioning",\n  "VTCONT_FNAME": "traffic_control_device_functioning_name",\n  "P_CRASH1": "pre_event_movement_prior_to_recognition_of_critical_event",\n  "P_CRASH1NAME": "pre_event_movement_prior_to_recognition_of_critical_event_name",\n  "P_CRASH2": "critical_event_precrash",\n  "P_CRASH2NAME": "critical_event_precrash_name",\n  "P_CRASH3": "attempted_avoidance_maneuver",\n  "P_CRASH3NAME": "attempted_avoidance_maneuver_name",\n  "PCRASH4": "pre_impact_stability",\n  "PCRASH4NAME": "pre_impact_stability_name",\n  "PCRASH5": "pre_impact_location",\n  "PCRASH5NAME": "pre_impact_location_name",\n  "ACC_TYPE": "crash_type",\n  "ACC_TYPENAME": "crash_type_name",\n  "DEATHS": "fatalities_in_vehicle",\n  "DR_DRINK": "driver_drinking",\n  "DR_DRINKNAME": "driver_drinking_name",\n  "TRLR1VIN": "trailer_vehicle_identification_number_1",\n  "TRLR1VINNAME": "trailer_vehicle_identification_number_1_name",\n  "TRLR2VIN": "trailer_vehicle_identification_number_2",\n  "TRLR2VINNAME": "trailer_vehicle_identification_number_2_name",\n  "TRLR3VIN": "trailer_vehicle_identification_number_3",\n  "TRLR3VINNAME": "trailer_vehicle_identification_number_3_name"\n}',
        },
        resources={
            "request_ephemeral_storage": "4G",
            "request_cpu": "1",
            "request_memory": "4G",
        },
    )

    # Run CSV transform within kubernetes pod for vehicle pipelines
    vehicle_2018_2019_transform_csv = kubernetes_pod.KubernetesPodOperator(
        task_id="vehicle_2018_2019_transform_csv",
        startup_timeout_seconds=600,
        name="vehicle",
        namespace="composer",
        service_account_name="datasets",
        image_pull_policy="Always",
        image="{{ var.json.nhtsa_traffic_fatalities.container_registry.run_csv_transform_kub }}",
        env_vars={
            "PIPELINE_NAME": "{{ var.json.nhtsa_traffic_fatalities.vehicle_2018_2019.pipeline_name }}",
            "SOURCE_URL": "{{ var.json.nhtsa_traffic_fatalities.vehicle_2018_2019.source_url }}",
            "CHUNKSIZE": "{{ var.json.nhtsa_traffic_fatalities.vehicle_2018_2019.chunksize }}",
            "SOURCE_ZIPFILE_EXTRACTED": "vehicle.csv",
            "SOURCE_FILE": "{{ var.json.nhtsa_traffic_fatalities.vehicle_2018_2019.source_file }}",
            "PROJECT_ID": "{{ var.value.gcp_project }}",
            "DATASET_ID": "{{ var.json.nhtsa_traffic_fatalities.vehicle_2018_2019.dataset_id }}",
            "TABLE_ID": "{{ var.json.nhtsa_traffic_fatalities.vehicle_2018_2019.destination_table }}",
            "START_YEAR": "{{ var.json.nhtsa_traffic_fatalities.vehicle_2018_2019.start_year }}",
            "END_YEAR": "{{ var.json.nhtsa_traffic_fatalities.vehicle_2018_2019.end_year }}",
            "DROP_DEST_TABLE": "{{ var.json.nhtsa_traffic_fatalities.vehicle_2018_2019.drop_dest_table }}",
            "TARGET_GCS_BUCKET": "{{ var.value.composer_bucket }}",
            "TARGET_GCS_PATH": "{{ var.json.nhtsa_traffic_fatalities.vehicle_2018_2019.target_gcs_path }}",
            "SCHEMA_PATH": "{{ var.json.nhtsa_traffic_fatalities.vehicle_2018_2019.schema_path }}",
            "INPUT_CSV_HEADERS": '[\n  "state_number",\n  "state_name",\n  "consecutive_number",\n  "vehicle_number",\n  "number_of_motor_vehicles_in_transport_mvit",\n  "number_of_occupants",\n  "number_of_occupants_name",\n  "day_of_crash",\n  "day_of_crash_name",\n  "month_of_crash",\n  "month_of_crash_name",\n  "hour_of_crash",\n  "hour_of_crash_name",\n  "minute_of_crash",\n  "minute_of_crash_name",\n  "first_harmful_event",\n  "first_harmful_event_name",\n  "manner_of_collision",\n  "manner_of_collision_name",\n  "unit_type",\n  "unit_type_name",\n  "hit_and_run",\n  "hit_and_run_name",\n  "registration_state",\n  "registration_state_name",\n  "registered_vehicle_owner",\n  "registered_vehicle_owner_name",\n  "vehicle_make",\n  "vehicle_make_name",\n  "vehicle_model",\n  "make_model_combined",\n  "make_model_combined_name",\n  "body_type",\n  "body_type_name",\n  "vehicle_model_year",\n  "vehicle_model_year_name",\n  "vehicle_identification_number_vin",\n  "vehicle_identification_number_vin_name",\n  "vin_character_1",\n  "vin_character_2",\n  "vin_character_3",\n  "vin_character_4",\n  "vin_character_5",\n  "vin_character_6",\n  "vin_character_7",\n  "vin_character_8",\n  "vin_character_9",\n  "vin_character_10",\n  "vin_character_11",\n  "vin_character_12",\n  "vehicle_trailing",\n  "vehicle_trailing_name",\n  "jackknife",\n  "jackknife_name",\n  "mcid_issuing_authority",\n  "mcid_issuing_authority_name",\n  "mcid_identification_number",\n  "mcid_identification_number_name",\n  "motor_carrier_identification_number_mcid",\n  "motor_carrier_identification_number_mcid_name",\n  "gross_vehicle_weight_rating",\n  "gross_vehicle_weight_rating_name",\n  "vehicle_configuration",\n  "vehicle_configuration_name",\n  "cargo_body_type",\n  "cargo_body_type_name",\n  "hazardous_material_involvement",\n  "hazardous_material_involvement_name",\n  "hazardous_material_placard",\n  "hazardous_material_placard_name",\n  "hazardous_material_identification_number",\n  "hazardous_material_identification_number_name",\n  "hazardous_material_class_number",\n  "hazardous_material_class_number_name",\n  "release_of_hazardous_material_from_the_cargo_compartment",\n  "release_of_hazardous_material_from_the_cargo_compartment_name",\n  "bus_use",\n  "bus_use_name",\n  "special_use",\n  "special_use_name",\n  "emergency_motor_vehicle_use",\n  "emergency_motor_vehicle_use_name",\n  "travel_speed",\n  "travel_speed_name",\n  "underride_override",\n  "underride_override_name",\n  "rollover",\n  "rollover_name",\n  "location_of_rollover",\n  "location_of_rollover_name",\n  "initial_contact_point",\n  "initial_contact_point_name",\n  "extent_of_damage",\n  "extent_of_damage_name",\n  "vehicle_removal",\n  "vehicle_removal_name",\n  "most_harmful_event",\n  "most_harmful_event_name",\n  "related_factors_vehicle_level_1",\n  "related_factors_vehicle_level_1_name",\n  "related_factors_vehicle_level_2",\n  "related_factors_vehicle_level_2_name",\n  "fire_occurrence",\n  "fire_occurrence_name",\n  "driver_presence",\n  "driver_presence_name",\n  "drivers_license_state",\n  "drivers_license_state_name",\n  "drivers_zip_code",\n  "drivers_zip_code_name",\n  "non_cdl_license_status",\n  "non_cdl_license_status_name",\n  "non_cdl_license_type",\n  "non_cdl_license_type_name",\n  "commercial_motor_vehicle_license_status",\n  "commercial_motor_vehicle_license_status_name",\n  "compliance_with_cdl_endorsements",\n  "compliance_with_cdl_endorsements_name",\n  "license_compliance_with_class_of_vehicle",\n  "license_compliance_with_class_of_vehicle_name",\n  "compliance_with_license_restrictions",\n  "compliance_with_license_restrictions_name",\n  "driver_height",\n  "driver_height_name",\n  "driver_weight",\n  "driver_weight_name",\n  "previous_recorded_crashes",\n  "previous_recorded_crashes_name",\n  "previous_recorded_suspensions_and_revocations1",\n  "previous_recorded_suspensions_and_revocations1_name",\n  "previous_recorded_suspensions_and_revocations2",\n  "previous_recorded_suspensions_and_revocations2_name",\n  "previous_recorded_suspensions_and_revocations3",\n  "previous_recorded_suspensions_and_revocations3_name",\n  "previous_dwi_convictions",\n  "previous_dwi_convictions_name",\n  "previous_speeding_convictions",\n  "previous_speeding_convictions_name",\n  "previous_other_moving_violation_convictions",\n  "previous_other_moving_violation_convictions_name",\n  "month_of_first_crash_suspension_or_conviction",\n  "month_of_first_crash_suspension_or_conviction_name",\n  "year_of_first_crash_suspension_or_conviction",\n  "year_of_first_crash_suspension_or_conviction_name",\n  "month_of_last_crash_suspension_or_conviction",\n  "month_of_last_crash_suspension_or_conviction_name",\n  "year_of_last_crash_suspension_or_conviction",\n  "year_of_last_crash_suspension_or_conviction_name",\n  "speeding_related",\n  "speeding_related_name",\n  "related_factors_driver_level_1",\n  "related_factors_driver_level_1_name",\n  "related_factors_driver_level_2",\n  "related_factors_driver_level_2_name",\n  "related_factors_driver_level_3",\n  "related_factors_driver_level_3_name",\n  "related_factors_driver_level_4",\n  "related_factors_driver_level_4_name",\n  "trafficway_description",\n  "trafficway_description_name",\n  "total_lanes_in_roadway",\n  "total_lanes_in_roadway_name",\n  "speed_limit",\n  "speed_limit_name",\n  "roadway_alignment",\n  "roadway_alignment_name",\n  "roadway_grade",\n  "roadway_grade_name",\n  "roadway_surface_type",\n  "roadway_surface_type_name",\n  "roadway_surface_condition",\n  "roadway_surface_condition_name",\n  "traffic_control_device",\n  "traffic_control_device_name",\n  "traffic_control_device_functioning",\n  "traffic_control_device_functioning_name",\n  "pre_event_movement_prior_to_recognition_of_critical_event",\n  "pre_event_movement_prior_to_recognition_of_critical_event_name",\n  "critical_event_precrash",\n  "critical_event_precrash_name",\n  "attempted_avoidance_maneuver",\n  "attempted_avoidance_maneuver_name",\n  "pre_impact_stability",\n  "pre_impact_stability_name",\n  "pre_impact_location",\n  "pre_impact_location_name",\n  "crash_type",\n  "crash_type_name",\n  "fatalities_in_vehicle",\n  "driver_drinking",\n  "driver_drinking_name",\n  "trailer_vehicle_identification_number_1",\n  "trailer_vehicle_identification_number_1_name",\n  "trailer_vehicle_identification_number_2",\n  "trailer_vehicle_identification_number_2_name",\n  "trailer_vehicle_identification_number_3",\n  "trailer_vehicle_identification_number_3_name"\n]',
            "INPUT_DTYPES": '{\n  "state_number": "str",\n  "state_name": "str",\n  "consecutive_number": "str",\n  "vehicle_number": "str",\n  "number_of_motor_vehicles_in_transport_mvit": "str",\n  "number_of_occupants": "str",\n  "number_of_occupants_name": "str",\n  "day_of_crash": "str",\n  "day_of_crash_name": "str",\n  "month_of_crash": "str",\n  "month_of_crash_name": "str",\n  "hour_of_crash": "str",\n  "hour_of_crash_name": "str",\n  "minute_of_crash": "str",\n  "minute_of_crash_name": "str",\n  "first_harmful_event": "str",\n  "first_harmful_event_name": "str",\n  "manner_of_collision": "str",\n  "manner_of_collision_name": "str",\n  "unit_type": "str",\n  "unit_type_name": "str",\n  "hit_and_run": "str",\n  "hit_and_run_name": "str",\n  "registration_state": "str",\n  "registration_state_name": "str",\n  "registered_vehicle_owner": "str",\n  "registered_vehicle_owner_name": "str",\n  "vehicle_make": "str",\n  "vehicle_make_name": "str",\n  "vehicle_model": "str",\n  "make_model_combined": "str",\n  "make_model_combined_name": "str",\n  "body_type": "str",\n  "body_type_name": "str",\n  "vehicle_model_year": "str",\n  "vehicle_model_year_name": "str",\n  "vehicle_identification_number_vin": "str",\n  "vehicle_identification_number_vin_name": "str",\n  "vin_character_1": "str",\n  "vin_character_2": "str",\n  "vin_character_3": "str",\n  "vin_character_4": "str",\n  "vin_character_5": "str",\n  "vin_character_6": "str",\n  "vin_character_7": "str",\n  "vin_character_8": "str",\n  "vin_character_9": "str",\n  "vin_character_10": "str",\n  "vin_character_11": "str",\n  "vin_character_12": "str",\n  "vehicle_trailing": "str",\n  "vehicle_trailing_name": "str",\n  "jackknife": "str",\n  "jackknife_name": "str",\n  "mcid_issuing_authority": "str",\n  "mcid_issuing_authority_name": "str",\n  "mcid_identification_number": "str",\n  "mcid_identification_number_name": "str",\n  "motor_carrier_identification_number_mcid": "str",\n  "motor_carrier_identification_number_mcid_name": "str",\n  "gross_vehicle_weight_rating": "str",\n  "gross_vehicle_weight_rating_name": "str",\n  "vehicle_configuration": "str",\n  "vehicle_configuration_name": "str",\n  "cargo_body_type": "str",\n  "cargo_body_type_name": "str",\n  "hazardous_material_involvement": "str",\n  "hazardous_material_involvement_name": "str",\n  "hazardous_material_placard": "str",\n  "hazardous_material_placard_name": "str",\n  "hazardous_material_identification_number": "str",\n  "hazardous_material_identification_number_name": "str",\n  "hazardous_material_class_number": "str",\n  "hazardous_material_class_number_name": "str",\n  "release_of_hazardous_material_from_the_cargo_compartment": "str",\n  "release_of_hazardous_material_from_the_cargo_compartment_name": "str",\n  "bus_use": "str",\n  "bus_use_name": "str",\n  "special_use": "str",\n  "special_use_name": "str",\n  "emergency_motor_vehicle_use": "str",\n  "emergency_motor_vehicle_use_name": "str",\n  "travel_speed": "str",\n  "travel_speed_name": "str",\n  "underride_override": "str",\n  "underride_override_name": "str",\n  "rollover": "str",\n  "rollover_name": "str",\n  "location_of_rollover": "str",\n  "location_of_rollover_name": "str",\n  "initial_contact_point": "str",\n  "initial_contact_point_name": "str",\n  "extent_of_damage": "str",\n  "extent_of_damage_name": "str",\n  "vehicle_removal": "str",\n  "vehicle_removal_name": "str",\n  "most_harmful_event": "str",\n  "most_harmful_event_name": "str",\n  "related_factors_vehicle_level_1": "str",\n  "related_factors_vehicle_level_1_name": "str",\n  "related_factors_vehicle_level_2": "str",\n  "related_factors_vehicle_level_2_name": "str",\n  "fire_occurrence": "str",\n  "fire_occurrence_name": "str",\n  "driver_presence": "str",\n  "driver_presence_name": "str",\n  "drivers_license_state": "str",\n  "drivers_license_state_name": "str",\n  "drivers_zip_code": "str",\n  "drivers_zip_code_name": "str",\n  "non_cdl_license_status": "str",\n  "non_cdl_license_status_name": "str",\n  "non_cdl_license_type": "str",\n  "non_cdl_license_type_name": "str",\n  "commercial_motor_vehicle_license_status": "str",\n  "commercial_motor_vehicle_license_status_name": "str",\n  "compliance_with_cdl_endorsements": "str",\n  "compliance_with_cdl_endorsements_name": "str",\n  "license_compliance_with_class_of_vehicle": "str",\n  "license_compliance_with_class_of_vehicle_name": "str",\n  "compliance_with_license_restrictions": "str",\n  "compliance_with_license_restrictions_name": "str",\n  "driver_height": "str",\n  "driver_height_name": "str",\n  "driver_weight": "str",\n  "driver_weight_name": "str",\n  "previous_recorded_crashes": "str",\n  "previous_recorded_crashes_name": "str",\n  "previous_recorded_suspensions_and_revocations1": "str",\n  "previous_recorded_suspensions_and_revocations1_name": "str",\n  "previous_recorded_suspensions_and_revocations2": "str",\n  "previous_recorded_suspensions_and_revocations2_name": "str",\n  "previous_recorded_suspensions_and_revocations3": "str",\n  "previous_recorded_suspensions_and_revocations3_name": "str",\n  "previous_dwi_convictions": "str",\n  "previous_dwi_convictions_name": "str",\n  "previous_speeding_convictions": "str",\n  "previous_speeding_convictions_name": "str",\n  "previous_other_moving_violation_convictions": "str",\n  "previous_other_moving_violation_convictions_name": "str",\n  "month_of_first_crash_suspension_or_conviction": "str",\n  "month_of_first_crash_suspension_or_conviction_name": "str",\n  "year_of_first_crash_suspension_or_conviction": "str",\n  "year_of_first_crash_suspension_or_conviction_name": "str",\n  "month_of_last_crash_suspension_or_conviction": "str",\n  "month_of_last_crash_suspension_or_conviction_name": "str",\n  "year_of_last_crash_suspension_or_conviction": "str",\n  "year_of_last_crash_suspension_or_conviction_name": "str",\n  "speeding_related": "str",\n  "speeding_related_name": "str",\n  "related_factors_driver_level_1": "str",\n  "related_factors_driver_level_1_name": "str",\n  "related_factors_driver_level_2": "str",\n  "related_factors_driver_level_2_name": "str",\n  "related_factors_driver_level_3": "str",\n  "related_factors_driver_level_3_name": "str",\n  "related_factors_driver_level_4": "str",\n  "related_factors_driver_level_4_name": "str",\n  "trafficway_description": "str",\n  "trafficway_description_name": "str",\n  "total_lanes_in_roadway": "str",\n  "total_lanes_in_roadway_name": "str",\n  "speed_limit": "str",\n  "speed_limit_name": "str",\n  "roadway_alignment": "str",\n  "roadway_alignment_name": "str",\n  "roadway_grade": "str",\n  "roadway_grade_name": "str",\n  "roadway_surface_type": "str",\n  "roadway_surface_type_name": "str",\n  "roadway_surface_condition": "str",\n  "roadway_surface_condition_name": "str",\n  "traffic_control_device": "str",\n  "traffic_control_device_name": "str",\n  "traffic_control_device_functioning": "str",\n  "traffic_control_device_functioning_name": "str",\n  "pre_event_movement_prior_to_recognition_of_critical_event": "str",\n  "pre_event_movement_prior_to_recognition_of_critical_event_name": "str",\n  "critical_event_precrash": "str",\n  "critical_event_precrash_name": "str",\n  "attempted_avoidance_maneuver": "str",\n  "attempted_avoidance_maneuver_name": "str",\n  "pre_impact_stability": "str",\n  "pre_impact_stability_name": "str",\n  "pre_impact_location": "str",\n  "pre_impact_location_name": "str",\n  "crash_type": "str",\n  "crash_type_name": "str",\n  "fatalities_in_vehicle": "str",\n  "driver_drinking": "str",\n  "driver_drinking_name": "str",\n  "trailer_vehicle_identification_number_1": "str",\n  "trailer_vehicle_identification_number_1_name": "str",\n  "trailer_vehicle_identification_number_2": "str",\n  "trailer_vehicle_identification_number_2_name": "str",\n  "trailer_vehicle_identification_number_3": "str",\n  "trailer_vehicle_identification_number_3_name": "str"\n}',
            "RENAME_MAPPINGS_LIST": '{\n  "STATE": "state_number",\n  "STATENAME": "state_name",\n  "ST_CASE": "consecutive_number",\n  "VEH_NO": "vehicle_number",\n  "VE_FORMS": "number_of_motor_vehicles_in_transport_mvit",\n  "NUMOCCS": "number_of_occupants",\n  "NUMOCCSNAME": "number_of_occupants_name",\n  "DAY": "day_of_crash",\n  "DAYNAME": "day_of_crash_name",\n  "MONTH": "month_of_crash",\n  "MONTHNAME": "month_of_crash_name",\n  "HOUR": "hour_of_crash",\n  "HOURNAME": "hour_of_crash_name",\n  "MINUTE": "minute_of_crash",\n  "MINUTENAME": "minute_of_crash_name",\n  "HARM_EV": "first_harmful_event",\n  "HARM_EVNAME": "first_harmful_event_name",\n  "MAN_COLL": "manner_of_collision",\n  "MAN_COLLNAME": "manner_of_collision_name",\n  "UNITTYPE": "unit_type",\n  "UNITTYPENAME": "unit_type_name",\n  "HIT_RUN": "hit_and_run",\n  "HIT_RUNNAME": "hit_and_run_name",\n  "REG_STAT": "registration_state",\n  "REG_STATNAME": "registration_state_name",\n  "OWNER": "registered_vehicle_owner",\n  "OWNERNAME": "registered_vehicle_owner_name",\n  "MAKE": "vehicle_make",\n  "MAKENAME": "vehicle_make_name",\n  "MODEL": "vehicle_model",\n  "MAK_MOD": "make_model_combined",\n  "MAK_MODNAME": "make_model_combined_name",\n  "BODY_TYP": "body_type",\n  "BODY_TYPNAME": "body_type_name",\n  "MOD_YEAR": "vehicle_model_year",\n  "MOD_YEARNAME": "vehicle_model_year_name",\n  "VIN": "vehicle_identification_number_vin",\n  "VINNAME": "vehicle_identification_number_vin_name",\n  "VIN_1": "vin_character_1",\n  "VIN_2": "vin_character_2",\n  "VIN_3": "vin_character_3",\n  "VIN_4": "vin_character_4",\n  "VIN_5": "vin_character_5",\n  "VIN_6": "vin_character_6",\n  "VIN_7": "vin_character_7",\n  "VIN_8": "vin_character_8",\n  "VIN_9": "vin_character_9",\n  "VIN_10": "vin_character_10",\n  "VIN_11": "vin_character_11",\n  "VIN_12": "vin_character_12",\n  "TOW_VEH": "vehicle_trailing",\n  "TOW_VEHNAME": "vehicle_trailing_name",\n  "J_KNIFE": "jackknife",\n  "J_KNIFENAME": "jackknife_name",\n  "MCARR_I1": "mcid_issuing_authority",\n  "MCARR_I1NAME": "mcid_issuing_authority_name",\n  "MCARR_I2": "mcid_identification_number",\n  "MCARR_I2NAME": "mcid_identification_number_name",\n  "MCARR_ID": "motor_carrier_identification_number_mcid",\n  "MCARR_IDNAME": "motor_carrier_identification_number_mcid_name",\n  "GVWR": "gross_vehicle_weight_rating",\n  "GVWRNAME": "gross_vehicle_weight_rating_name",\n  "V_CONFIG": "vehicle_configuration",\n  "V_CONFIGNAME": "vehicle_configuration_name",\n  "CARGO_BT": "cargo_body_type",\n  "CARGO_BTNAME": "cargo_body_type_name",\n  "HAZ_INV": "hazardous_material_involvement",\n  "HAZ_INVNAME": "hazardous_material_involvement_name",\n  "HAZ_PLAC": "hazardous_material_placard",\n  "HAZ_PLACNAME": "hazardous_material_placard_name",\n  "HAZ_ID": "hazardous_material_identification_number",\n  "HAZ_IDNAME": "hazardous_material_identification_number_name",\n  "HAZ_CNO": "hazardous_material_class_number",\n  "HAZ_CNONAME": "hazardous_material_class_number_name",\n  "HAZ_REL": "release_of_hazardous_material_from_the_cargo_compartment",\n  "HAZ_RELNAME": "release_of_hazardous_material_from_the_cargo_compartment_name",\n  "BUS_USE": "bus_use",\n  "BUS_USENAME": "bus_use_name",\n  "SPEC_USE": "special_use",\n  "SPEC_USENAME": "special_use_name",\n  "EMER_USE": "emergency_motor_vehicle_use",\n  "EMER_USENAME": "emergency_motor_vehicle_use_name",\n  "TRAV_SP": "travel_speed",\n  "TRAV_SPNAME": "travel_speed_name",\n  "UNDERIDE": "underride_override",\n  "UNDERIDENAME": "underride_override_name",\n  "ROLLOVER": "rollover",\n  "ROLLOVERNAME": "rollover_name",\n  "ROLINLOC": "location_of_rollover",\n  "ROLINLOCNAME": "location_of_rollover_name",\n  "IMPACT1": "initial_contact_point",\n  "IMPACT1NAME": "initial_contact_point_name",\n  "DEFORMED": "extent_of_damage",\n  "DEFORMEDNAME": "extent_of_damage_name",\n  "TOWED": "vehicle_removal",\n  "TOWEDNAME": "vehicle_removal_name",\n  "M_HARM": "most_harmful_event",\n  "M_HARMNAME": "most_harmful_event_name",\n  "VEH_SC1": "related_factors_vehicle_level_1",\n  "VEH_SC1NAME": "related_factors_vehicle_level_1_name",\n  "VEH_SC2": "related_factors_vehicle_level_2",\n  "VEH_SC2NAME": "related_factors_vehicle_level_2_name",\n  "FIRE_EXP": "fire_occurrence",\n  "FIRE_EXPNAME": "fire_occurrence_name",\n  "DR_PRES": "driver_presence",\n  "DR_PRESNAME": "driver_presence_name",\n  "L_STATE": "drivers_license_state",\n  "L_STATENAME": "drivers_license_state_name",\n  "DR_ZIP": "drivers_zip_code",\n  "DR_ZIPNAME": "drivers_zip_code_name",\n  "L_STATUS": "non_cdl_license_status",\n  "L_STATUSNAME": "non_cdl_license_status_name",\n  "L_TYPE": "non_cdl_license_type",\n  "L_TYPENAME": "non_cdl_license_type_name",\n  "CDL_STAT": "commercial_motor_vehicle_license_status",\n  "CDL_STATNAME": "commercial_motor_vehicle_license_status_name",\n  "L_ENDORS": "compliance_with_cdl_endorsements",\n  "L_ENDORSNAME": "compliance_with_cdl_endorsements_name",\n  "L_COMPL": "license_compliance_with_class_of_vehicle",\n  "L_COMPLNAME": "license_compliance_with_class_of_vehicle_name",\n  "L_RESTRI": "compliance_with_license_restrictions",\n  "L_RESTRINAME": "compliance_with_license_restrictions_name",\n  "DR_HGT": "driver_height",\n  "DR_HGTNAME": "driver_height_name",\n  "DR_WGT": "driver_weight",\n  "DR_WGTNAME": "driver_weight_name",\n  "PREV_ACC": "previous_recorded_crashes",\n  "PREV_ACCNAME": "previous_recorded_crashes_name",\n  "PREV_SUS1": "previous_recorded_suspensions_and_revocations1",\n  "PREV_SUS1NAME": "previous_recorded_suspensions_and_revocations1_name",\n  "PREV_SUS2": "previous_recorded_suspensions_and_revocations2",\n  "PREV_SUS2NAME": "previous_recorded_suspensions_and_revocations2_name",\n  "PREV_SUS3": "previous_recorded_suspensions_and_revocations3",\n  "PREV_SUS3NAME": "previous_recorded_suspensions_and_revocations3_name",\n  "PREV_DWI": "previous_dwi_convictions",\n  "PREV_DWINAME": "previous_dwi_convictions_name",\n  "PREV_SPD": "previous_speeding_convictions",\n  "PREV_SPDNAME": "previous_speeding_convictions_name",\n  "PREV_OTH": "previous_other_moving_violation_convictions",\n  "PREV_OTHNAME": "previous_other_moving_violation_convictions_name",\n  "FIRST_MO": "month_of_first_crash_suspension_or_conviction",\n  "FIRST_MONAME": "month_of_first_crash_suspension_or_conviction_name",\n  "FIRST_YR": "year_of_first_crash_suspension_or_conviction",\n  "FIRST_YRNAME": "year_of_first_crash_suspension_or_conviction_name",\n  "LAST_MO": "month_of_last_crash_suspension_or_conviction",\n  "LAST_MONAME": "month_of_last_crash_suspension_or_conviction_name",\n  "LAST_YR": "year_of_last_crash_suspension_or_conviction",\n  "LAST_YRNAME": "year_of_last_crash_suspension_or_conviction_name",\n  "SPEEDREL": "speeding_related",\n  "SPEEDRELNAME": "speeding_related_name",\n  "DR_SF1": "related_factors_driver_level_1",\n  "DR_SF1NAME": "related_factors_driver_level_1_name",\n  "DR_SF2": "related_factors_driver_level_2",\n  "DR_SF2NAME": "related_factors_driver_level_2_name",\n  "DR_SF3": "related_factors_driver_level_3",\n  "DR_SF3NAME": "related_factors_driver_level_3_name",\n  "DR_SF4": "related_factors_driver_level_4",\n  "DR_SF4NAME": "related_factors_driver_level_4_name",\n  "VTRAFWAY": "trafficway_description",\n  "VTRAFWAYNAME": "trafficway_description_name",\n  "VNUM_LAN": "total_lanes_in_roadway",\n  "VNUM_LANNAME": "total_lanes_in_roadway_name",\n  "VSPD_LIM": "speed_limit",\n  "VSPD_LIMNAME": "speed_limit_name",\n  "VALIGN": "roadway_alignment",\n  "VALIGNNAME": "roadway_alignment_name",\n  "VPROFILE": "roadway_grade",\n  "VPROFILENAME": "roadway_grade_name",\n  "VPAVETYP": "roadway_surface_type",\n  "VPAVETYPNAME": "roadway_surface_type_name",\n  "VSURCOND": "roadway_surface_condition",\n  "VSURCONDNAME": "roadway_surface_condition_name",\n  "VTRAFCON": "traffic_control_device",\n  "VTRAFCONNAME": "traffic_control_device_name",\n  "VTCONT_F": "traffic_control_device_functioning",\n  "VTCONT_FNAME": "traffic_control_device_functioning_name",\n  "P_CRASH1": "pre_event_movement_prior_to_recognition_of_critical_event",\n  "P_CRASH1NAME": "pre_event_movement_prior_to_recognition_of_critical_event_name",\n  "P_CRASH2": "critical_event_precrash",\n  "P_CRASH2NAME": "critical_event_precrash_name",\n  "P_CRASH3": "attempted_avoidance_maneuver",\n  "P_CRASH3NAME": "attempted_avoidance_maneuver_name",\n  "PCRASH4": "pre_impact_stability",\n  "PCRASH4NAME": "pre_impact_stability_name",\n  "PCRASH5": "pre_impact_location",\n  "PCRASH5NAME": "pre_impact_location_name",\n  "ACC_TYPE": "crash_type",\n  "ACC_TYPENAME": "crash_type_name",\n  "DEATHS": "fatalities_in_vehicle",\n  "DR_DRINK": "driver_drinking",\n  "DR_DRINKNAME": "driver_drinking_name",\n  "TRLR1VIN": "trailer_vehicle_identification_number_1",\n  "TRLR1VINNAME": "trailer_vehicle_identification_number_1_name",\n  "TRLR2VIN": "trailer_vehicle_identification_number_2",\n  "TRLR2VINNAME": "trailer_vehicle_identification_number_2_name",\n  "TRLR3VIN": "trailer_vehicle_identification_number_3",\n  "TRLR3VINNAME": "trailer_vehicle_identification_number_3_name"\n}',
        },
        resources={
            "request_ephemeral_storage": "4G",
            "request_cpu": "1",
            "request_memory": "4G",
        },
    )

    # Run CSV transform within kubernetes pod for vehicle pipelines
    vehicle_2020_transform_csv = kubernetes_pod.KubernetesPodOperator(
        task_id="vehicle_2020_transform_csv",
        startup_timeout_seconds=600,
        name="vehicle",
        namespace="composer",
        service_account_name="datasets",
        image_pull_policy="Always",
        image="{{ var.json.nhtsa_traffic_fatalities.container_registry.run_csv_transform_kub }}",
        env_vars={
            "PIPELINE_NAME": "{{ var.json.nhtsa_traffic_fatalities.vehicle_2020.pipeline_name }}",
            "SOURCE_URL": "{{ var.json.nhtsa_traffic_fatalities.vehicle_2020.source_url }}",
            "CHUNKSIZE": "{{ var.json.nhtsa_traffic_fatalities.vehicle_2020.chunksize }}",
            "SOURCE_ZIPFILE_EXTRACTED": "vehicle.csv",
            "SOURCE_FILE": "{{ var.json.nhtsa_traffic_fatalities.vehicle_2020.source_file }}",
            "PROJECT_ID": "{{ var.value.gcp_project }}",
            "DATASET_ID": "{{ var.json.nhtsa_traffic_fatalities.vehicle_2020.dataset_id }}",
            "TABLE_ID": "{{ var.json.nhtsa_traffic_fatalities.vehicle_2020.destination_table }}",
            "START_YEAR": "{{ var.json.nhtsa_traffic_fatalities.vehicle_2020.start_year }}",
            "END_YEAR": "{{ var.json.nhtsa_traffic_fatalities.vehicle_2020.end_year }}",
            "DROP_DEST_TABLE": "{{ var.json.nhtsa_traffic_fatalities.vehicle_2020.drop_dest_table }}",
            "TARGET_GCS_BUCKET": "{{ var.value.composer_bucket }}",
            "TARGET_GCS_PATH": "{{ var.json.nhtsa_traffic_fatalities.vehicle_2020.target_gcs_path }}",
            "SCHEMA_PATH": "{{ var.json.nhtsa_traffic_fatalities.vehicle_2020.schema_path }}",
            "INPUT_CSV_HEADERS": '[\n  "state_number",\n  "state_name",\n  "consecutive_number",\n  "vehicle_number",\n  "number_of_motor_vehicles_in_transport_mvit",\n  "number_of_occupants",\n  "number_of_occupants_name",\n  "day_of_crash",\n  "day_of_crash_name",\n  "month_of_crash",\n  "month_of_crash_name",\n  "hour_of_crash",\n  "hour_of_crash_name",\n  "minute_of_crash",\n  "minute_of_crash_name",\n  "first_harmful_event",\n  "first_harmful_event_name",\n  "manner_of_collision",\n  "manner_of_collision_name",\n  "unit_type",\n  "unit_type_name",\n  "hit_and_run",\n  "hit_and_run_name",\n  "registration_state",\n  "registration_state_name",\n  "registered_vehicle_owner",\n  "registered_vehicle_owner_name",\n  "vehicle_make",\n  "vehicle_make_name",\n  "vehicle_model",\n  "make_model_combined",\n  "make_model_combined_name",\n  "body_type",\n  "body_type_name",\n  "vehicle_model_year",\n  "vehicle_model_year_name",\n  "vehicle_identification_number_vin",\n  "vehicle_identification_number_vin_name",\n  "vin_character_1",\n  "vin_character_2",\n  "vin_character_3",\n  "vin_character_4",\n  "vin_character_5",\n  "vin_character_6",\n  "vin_character_7",\n  "vin_character_8",\n  "vin_character_9",\n  "vin_character_10",\n  "vin_character_11",\n  "vin_character_12",\n  "vehicle_trailing",\n  "vehicle_trailing_name",\n  "jackknife",\n  "jackknife_name",\n  "mcid_issuing_authority",\n  "mcid_issuing_authority_name",\n  "mcid_identification_number",\n  "mcid_identification_number_name",\n  "motor_carrier_identification_number_mcid",\n  "motor_carrier_identification_number_mcid_name",\n  "vehicle_configuration",\n  "vehicle_configuration_name",\n  "cargo_body_type",\n  "cargo_body_type_name",\n  "hazardous_material_involvement",\n  "hazardous_material_involvement_name",\n  "hazardous_material_placard",\n  "hazardous_material_placard_name",\n  "hazardous_material_identification_number",\n  "hazardous_material_identification_number_name",\n  "hazardous_material_class_number",\n  "hazardous_material_class_number_name",\n  "release_of_hazardous_material_from_the_cargo_compartment",\n  "release_of_hazardous_material_from_the_cargo_compartment_name",\n  "bus_use",\n  "bus_use_name",\n  "special_use",\n  "special_use_name",\n  "emergency_motor_vehicle_use",\n  "emergency_motor_vehicle_use_name",\n  "travel_speed",\n  "travel_speed_name",\n  "underride_override",\n  "underride_override_name",\n  "rollover",\n  "rollover_name",\n  "location_of_rollover",\n  "location_of_rollover_name",\n  "initial_contact_point",\n  "initial_contact_point_name",\n  "extent_of_damage",\n  "extent_of_damage_name",\n  "vehicle_removal",\n  "vehicle_removal_name",\n  "most_harmful_event",\n  "most_harmful_event_name",\n  "fire_occurrence",\n  "fire_occurrence_name",\n  "driver_presence",\n  "driver_presence_name",\n  "drivers_license_state",\n  "drivers_license_state_name",\n  "drivers_zip_code",\n  "drivers_zip_code_name",\n  "non_cdl_license_status",\n  "non_cdl_license_status_name",\n  "non_cdl_license_type",\n  "non_cdl_license_type_name",\n  "commercial_motor_vehicle_license_status",\n  "commercial_motor_vehicle_license_status_name",\n  "compliance_with_cdl_endorsements",\n  "compliance_with_cdl_endorsements_name",\n  "license_compliance_with_class_of_vehicle",\n  "license_compliance_with_class_of_vehicle_name",\n  "compliance_with_license_restrictions",\n  "compliance_with_license_restrictions_name",\n  "driver_height",\n  "driver_height_name",\n  "driver_weight",\n  "driver_weight_name",\n  "previous_recorded_crashes",\n  "previous_recorded_crashes_name",\n  "previous_recorded_suspensions_and_revocations1",\n  "previous_recorded_suspensions_and_revocations1_name",\n  "previous_recorded_suspensions_and_revocations2",\n  "previous_recorded_suspensions_and_revocations2_name",\n  "previous_recorded_suspensions_and_revocations3",\n  "previous_recorded_suspensions_and_revocations3_name",\n  "previous_dwi_convictions",\n  "previous_dwi_convictions_name",\n  "previous_speeding_convictions",\n  "previous_speeding_convictions_name",\n  "previous_other_moving_violation_convictions",\n  "previous_other_moving_violation_convictions_name",\n  "month_of_first_crash_suspension_or_conviction",\n  "month_of_first_crash_suspension_or_conviction_name",\n  "year_of_first_crash_suspension_or_conviction",\n  "year_of_first_crash_suspension_or_conviction_name",\n  "month_of_last_crash_suspension_or_conviction",\n  "month_of_last_crash_suspension_or_conviction_name",\n  "year_of_last_crash_suspension_or_conviction",\n  "year_of_last_crash_suspension_or_conviction_name",\n  "speeding_related",\n  "speeding_related_name",\n  "trafficway_description",\n  "trafficway_description_name",\n  "total_lanes_in_roadway",\n  "total_lanes_in_roadway_name",\n  "speed_limit",\n  "speed_limit_name",\n  "roadway_alignment",\n  "roadway_alignment_name",\n  "roadway_grade",\n  "roadway_grade_name",\n  "roadway_surface_type",\n  "roadway_surface_type_name",\n  "roadway_surface_condition",\n  "roadway_surface_condition_name",\n  "traffic_control_device",\n  "traffic_control_device_name",\n  "traffic_control_device_functioning",\n  "traffic_control_device_functioning_name",\n  "pre_event_movement_prior_to_recognition_of_critical_event",\n  "pre_event_movement_prior_to_recognition_of_critical_event_name",\n  "critical_event_precrash",\n  "critical_event_precrash_name",\n  "attempted_avoidance_maneuver",\n  "attempted_avoidance_maneuver_name",\n  "pre_impact_stability",\n  "pre_impact_stability_name",\n  "pre_impact_location",\n  "pre_impact_location_name",\n  "crash_type",\n  "crash_type_name",\n  "fatalities_in_vehicle",\n  "driver_drinking",\n  "driver_drinking_name",\n  "trailer_vehicle_identification_number_1",\n  "trailer_vehicle_identification_number_1_name",\n  "trailer_vehicle_identification_number_2",\n  "trailer_vehicle_identification_number_2_name",\n  "trailer_vehicle_identification_number_3",\n  "trailer_vehicle_identification_number_3_name",\n  "vpic_make",\n  "vpic_make_name",\n  "vpic_model",\n  "vpic_model_name",\n  "vpic_body_class",\n  "vpic_body_class_name",\n  "final_stage_body_class",\n  "final_stage_body_class_name",\n  "power_unit_gross_vehicle_weight_rating_from",\n  "power_unit_gross_vehicle_weight_rating_from_name",\n  "power_unit_gross_vehicle_weight_rating_to",\n  "power_unit_gross_vehicle_weight_rating_to_name",\n  "trailer_gross_vehicle_weight_rating_1",\n  "trailer_gross_vehicle_weight_rating_1_name",\n  "trailer_gross_vehicle_weight_rating_2",\n  "trailer_gross_vehicle_weight_rating_2_name",\n  "trailer_gross_vehicle_weight_rating_3",\n  "trailer_gross_vehicle_weight_rating_3_name"\n]',
            "INPUT_DTYPES": '{\n  "state_number": "str",\n  "state_name": "str",\n  "consecutive_number": "str",\n  "vehicle_number": "str",\n  "number_of_motor_vehicles_in_transport_mvit": "str",\n  "number_of_occupants": "str",\n  "number_of_occupants_name": "str",\n  "day_of_crash": "str",\n  "day_of_crash_name": "str",\n  "month_of_crash": "str",\n  "month_of_crash_name": "str",\n  "hour_of_crash": "str",\n  "hour_of_crash_name": "str",\n  "minute_of_crash": "str",\n  "minute_of_crash_name": "str",\n  "first_harmful_event": "str",\n  "first_harmful_event_name": "str",\n  "manner_of_collision": "str",\n  "manner_of_collision_name": "str",\n  "unit_type": "str",\n  "unit_type_name": "str",\n  "hit_and_run": "str",\n  "hit_and_run_name": "str",\n  "registration_state": "str",\n  "registration_state_name": "str",\n  "registered_vehicle_owner": "str",\n  "registered_vehicle_owner_name": "str",\n  "vehicle_make": "str",\n  "vehicle_make_name": "str",\n  "vehicle_model": "str",\n  "make_model_combined": "str",\n  "make_model_combined_name": "str",\n  "body_type": "str",\n  "body_type_name": "str",\n  "vehicle_model_year": "str",\n  "vehicle_model_year_name": "str",\n  "vehicle_identification_number_vin": "str",\n  "vehicle_identification_number_vin_name": "str",\n  "vin_character_1": "str",\n  "vin_character_2": "str",\n  "vin_character_3": "str",\n  "vin_character_4": "str",\n  "vin_character_5": "str",\n  "vin_character_6": "str",\n  "vin_character_7": "str",\n  "vin_character_8": "str",\n  "vin_character_9": "str",\n  "vin_character_10": "str",\n  "vin_character_11": "str",\n  "vin_character_12": "str",\n  "vehicle_trailing": "str",\n  "vehicle_trailing_name": "str",\n  "jackknife": "str",\n  "jackknife_name": "str",\n  "mcid_issuing_authority": "str",\n  "mcid_issuing_authority_name": "str",\n  "mcid_identification_number": "str",\n  "mcid_identification_number_name": "str",\n  "motor_carrier_identification_number_mcid": "str",\n  "motor_carrier_identification_number_mcid_name": "str",\n  "vehicle_configuration": "str",\n  "vehicle_configuration_name": "str",\n  "cargo_body_type": "str",\n  "cargo_body_type_name": "str",\n  "hazardous_material_involvement": "str",\n  "hazardous_material_involvement_name": "str",\n  "hazardous_material_placard": "str",\n  "hazardous_material_placard_name": "str",\n  "hazardous_material_identification_number": "str",\n  "hazardous_material_identification_number_name": "str",\n  "hazardous_material_class_number": "str",\n  "hazardous_material_class_number_name": "str",\n  "release_of_hazardous_material_from_the_cargo_compartment": "str",\n  "release_of_hazardous_material_from_the_cargo_compartment_name": "str",\n  "bus_use": "str",\n  "bus_use_name": "str",\n  "special_use": "str",\n  "special_use_name": "str",\n  "emergency_motor_vehicle_use": "str",\n  "emergency_motor_vehicle_use_name": "str",\n  "travel_speed": "str",\n  "travel_speed_name": "str",\n  "underride_override": "str",\n  "underride_override_name": "str",\n  "rollover": "str",\n  "rollover_name": "str",\n  "location_of_rollover": "str",\n  "location_of_rollover_name": "str",\n  "initial_contact_point": "str",\n  "initial_contact_point_name": "str",\n  "extent_of_damage": "str",\n  "extent_of_damage_name": "str",\n  "vehicle_removal": "str",\n  "vehicle_removal_name": "str",\n  "most_harmful_event": "str",\n  "most_harmful_event_name": "str",\n  "fire_occurrence": "str",\n  "fire_occurrence_name": "str",\n  "driver_presence": "str",\n  "driver_presence_name": "str",\n  "drivers_license_state": "str",\n  "drivers_license_state_name": "str",\n  "drivers_zip_code": "str",\n  "drivers_zip_code_name": "str",\n  "non_cdl_license_status": "str",\n  "non_cdl_license_status_name": "str",\n  "non_cdl_license_type": "str",\n  "non_cdl_license_type_name": "str",\n  "commercial_motor_vehicle_license_status": "str",\n  "commercial_motor_vehicle_license_status_name": "str",\n  "compliance_with_cdl_endorsements": "str",\n  "compliance_with_cdl_endorsements_name": "str",\n  "license_compliance_with_class_of_vehicle": "str",\n  "license_compliance_with_class_of_vehicle_name": "str",\n  "compliance_with_license_restrictions": "str",\n  "compliance_with_license_restrictions_name": "str",\n  "driver_height": "str",\n  "driver_height_name": "str",\n  "driver_weight": "str",\n  "driver_weight_name": "str",\n  "previous_recorded_crashes": "str",\n  "previous_recorded_crashes_name": "str",\n  "previous_recorded_suspensions_and_revocations1": "str",\n  "previous_recorded_suspensions_and_revocations1_name": "str",\n  "previous_recorded_suspensions_and_revocations2": "str",\n  "previous_recorded_suspensions_and_revocations2_name": "str",\n  "previous_recorded_suspensions_and_revocations3": "str",\n  "previous_recorded_suspensions_and_revocations3_name": "str",\n  "previous_dwi_convictions": "str",\n  "previous_dwi_convictions_name": "str",\n  "previous_speeding_convictions": "str",\n  "previous_speeding_convictions_name": "str",\n  "previous_other_moving_violation_convictions": "str",\n  "previous_other_moving_violation_convictions_name": "str",\n  "month_of_first_crash_suspension_or_conviction": "str",\n  "month_of_first_crash_suspension_or_conviction_name": "str",\n  "year_of_first_crash_suspension_or_conviction": "str",\n  "year_of_first_crash_suspension_or_conviction_name": "str",\n  "month_of_last_crash_suspension_or_conviction": "str",\n  "month_of_last_crash_suspension_or_conviction_name": "str",\n  "year_of_last_crash_suspension_or_conviction": "str",\n  "year_of_last_crash_suspension_or_conviction_name": "str",\n  "speeding_related": "str",\n  "speeding_related_name": "str",\n  "trafficway_description": "str",\n  "trafficway_description_name": "str",\n  "total_lanes_in_roadway": "str",\n  "total_lanes_in_roadway_name": "str",\n  "speed_limit": "str",\n  "speed_limit_name": "str",\n  "roadway_alignment": "str",\n  "roadway_alignment_name": "str",\n  "roadway_grade": "str",\n  "roadway_grade_name": "str",\n  "roadway_surface_type": "str",\n  "roadway_surface_type_name": "str",\n  "roadway_surface_condition": "str",\n  "roadway_surface_condition_name": "str",\n  "traffic_control_device": "str",\n  "traffic_control_device_name": "str",\n  "traffic_control_device_functioning": "str",\n  "traffic_control_device_functioning_name": "str",\n  "pre_event_movement_prior_to_recognition_of_critical_event": "str",\n  "pre_event_movement_prior_to_recognition_of_critical_event_name": "str",\n  "critical_event_precrash": "str",\n  "critical_event_precrash_name": "str",\n  "attempted_avoidance_maneuver": "str",\n  "attempted_avoidance_maneuver_name": "str",\n  "pre_impact_stability": "str",\n  "pre_impact_stability_name": "str",\n  "pre_impact_location": "str",\n  "pre_impact_location_name": "str",\n  "crash_type": "str",\n  "crash_type_name": "str",\n  "fatalities_in_vehicle": "str",\n  "driver_drinking": "str",\n  "driver_drinking_name": "str",\n  "trailer_vehicle_identification_number_1": "str",\n  "trailer_vehicle_identification_number_1_name": "str",\n  "trailer_vehicle_identification_number_2": "str",\n  "trailer_vehicle_identification_number_2_name": "str",\n  "trailer_vehicle_identification_number_3": "str",\n  "trailer_vehicle_identification_number_3_name": "str",\n  "vpic_make": "str",\n  "vpic_make_name": "str",\n  "vpic_model": "str",\n  "vpic_model_name": "str",\n  "vpic_body_class": "str",\n  "vpic_body_class_name": "str",\n  "final_stage_body_class": "str",\n  "final_stage_body_class_name": "str",\n  "power_unit_gross_vehicle_weight_rating_from": "str",\n  "power_unit_gross_vehicle_weight_rating_from_name": "str",\n  "power_unit_gross_vehicle_weight_rating_to": "str",\n  "power_unit_gross_vehicle_weight_rating_to_name": "str",\n  "trailer_gross_vehicle_weight_rating_1": "str",\n  "trailer_gross_vehicle_weight_rating_1_name": "str",\n  "trailer_gross_vehicle_weight_rating_2": "str",\n  "trailer_gross_vehicle_weight_rating_2_name": "str",\n  "trailer_gross_vehicle_weight_rating_3": "str",\n  "trailer_gross_vehicle_weight_rating_3_name": "str"\n}',
            "RENAME_MAPPINGS_LIST": '{\n  "STATE": "state_number",\n  "STATENAME": "state_name",\n  "ST_CASE": "consecutive_number",\n  "VEH_NO": "vehicle_number",\n  "VE_FORMS": "number_of_motor_vehicles_in_transport_mvit",\n  "NUMOCCS": "number_of_occupants",\n  "NUMOCCSNAME": "number_of_occupants_name",\n  "DAY": "day_of_crash",\n  "DAYNAME": "day_of_crash_name",\n  "MONTH": "month_of_crash",\n  "MONTHNAME": "month_of_crash_name",\n  "HOUR": "hour_of_crash",\n  "HOURNAME": "hour_of_crash_name",\n  "MINUTE": "minute_of_crash",\n  "MINUTENAME": "minute_of_crash_name",\n  "HARM_EV": "first_harmful_event",\n  "HARM_EVNAME": "first_harmful_event_name",\n  "MAN_COLL": "manner_of_collision",\n  "MAN_COLLNAME": "manner_of_collision_name",\n  "UNITTYPE": "unit_type",\n  "UNITTYPENAME": "unit_type_name",\n  "HIT_RUN": "hit_and_run",\n  "HIT_RUNNAME": "hit_and_run_name",\n  "REG_STAT": "registration_state",\n  "REG_STATNAME": "registration_state_name",\n  "OWNER": "registered_vehicle_owner",\n  "OWNERNAME": "registered_vehicle_owner_name",\n  "MAKE": "vehicle_make",\n  "MAKENAME": "vehicle_make_name",\n  "MODEL": "vehicle_model",\n  "MAK_MOD": "make_model_combined",\n  "MAK_MODNAME": "make_model_combined_name",\n  "BODY_TYP": "body_type",\n  "BODY_TYPNAME": "body_type_name",\n  "MOD_YEAR": "vehicle_model_year",\n  "MOD_YEARNAME": "vehicle_model_year_name",\n  "VIN": "vehicle_identification_number_vin",\n  "VINNAME": "vehicle_identification_number_vin_name",\n  "VIN_1": "vin_character_1",\n  "VIN_2": "vin_character_2",\n  "VIN_3": "vin_character_3",\n  "VIN_4": "vin_character_4",\n  "VIN_5": "vin_character_5",\n  "VIN_6": "vin_character_6",\n  "VIN_7": "vin_character_7",\n  "VIN_8": "vin_character_8",\n  "VIN_9": "vin_character_9",\n  "VIN_10": "vin_character_10",\n  "VIN_11": "vin_character_11",\n  "VIN_12": "vin_character_12",\n  "TOW_VEH": "vehicle_trailing",\n  "TOW_VEHNAME": "vehicle_trailing_name",\n  "J_KNIFE": "jackknife",\n  "J_KNIFENAME": "jackknife_name",\n  "MCARR_I1": "mcid_issuing_authority",\n  "MCARR_I1NAME": "mcid_issuing_authority_name",\n  "MCARR_I2": "mcid_identification_number",\n  "MCARR_I2NAME": "mcid_identification_number_name",\n  "MCARR_ID": "motor_carrier_identification_number_mcid",\n  "MCARR_IDNAME": "motor_carrier_identification_number_mcid_name",\n  "V_CONFIG": "vehicle_configuration",\n  "V_CONFIGNAME": "vehicle_configuration_name",\n  "CARGO_BT": "cargo_body_type",\n  "CARGO_BTNAME": "cargo_body_type_name",\n  "HAZ_INV": "hazardous_material_involvement",\n  "HAZ_INVNAME": "hazardous_material_involvement_name",\n  "HAZ_PLAC": "hazardous_material_placard",\n  "HAZ_PLACNAME": "hazardous_material_placard_name",\n  "HAZ_ID": "hazardous_material_identification_number",\n  "HAZ_IDNAME": "hazardous_material_identification_number_name",\n  "HAZ_CNO": "hazardous_material_class_number",\n  "HAZ_CNONAME": "hazardous_material_class_number_name",\n  "HAZ_REL": "release_of_hazardous_material_from_the_cargo_compartment",\n  "HAZ_RELNAME": "release_of_hazardous_material_from_the_cargo_compartment_name",\n  "BUS_USE": "bus_use",\n  "BUS_USENAME": "bus_use_name",\n  "SPEC_USE": "special_use",\n  "SPEC_USENAME": "special_use_name",\n  "EMER_USE": "emergency_motor_vehicle_use",\n  "EMER_USENAME": "emergency_motor_vehicle_use_name",\n  "TRAV_SP": "travel_speed",\n  "TRAV_SPNAME": "travel_speed_name",\n  "UNDERIDE": "underride_override",\n  "UNDERIDENAME": "underride_override_name",\n  "ROLLOVER": "rollover",\n  "ROLLOVERNAME": "rollover_name",\n  "ROLINLOC": "location_of_rollover",\n  "ROLINLOCNAME": "location_of_rollover_name",\n  "IMPACT1": "initial_contact_point",\n  "IMPACT1NAME": "initial_contact_point_name",\n  "DEFORMED": "extent_of_damage",\n  "DEFORMEDNAME": "extent_of_damage_name",\n  "TOWED": "vehicle_removal",\n  "TOWEDNAME": "vehicle_removal_name",\n  "M_HARM": "most_harmful_event",\n  "M_HARMNAME": "most_harmful_event_name",\n  "FIRE_EXP": "fire_occurrence",\n  "FIRE_EXPNAME": "fire_occurrence_name",\n  "DR_PRES": "driver_presence",\n  "DR_PRESNAME": "driver_presence_name",\n  "L_STATE": "drivers_license_state",\n  "L_STATENAME": "drivers_license_state_name",\n  "DR_ZIP": "drivers_zip_code",\n  "DR_ZIPNAME": "drivers_zip_code_name",\n  "L_STATUS": "non_cdl_license_status",\n  "L_STATUSNAME": "non_cdl_license_status_name",\n  "L_TYPE": "non_cdl_license_type",\n  "L_TYPENAME": "non_cdl_license_type_name",\n  "CDL_STAT": "commercial_motor_vehicle_license_status",\n  "CDL_STATNAME": "commercial_motor_vehicle_license_status_name",\n  "L_ENDORS": "compliance_with_cdl_endorsements",\n  "L_ENDORSNAME": "compliance_with_cdl_endorsements_name",\n  "L_COMPL": "license_compliance_with_class_of_vehicle",\n  "L_COMPLNAME": "license_compliance_with_class_of_vehicle_name",\n  "L_RESTRI": "compliance_with_license_restrictions",\n  "L_RESTRINAME": "compliance_with_license_restrictions_name",\n  "DR_HGT": "driver_height",\n  "DR_HGTNAME": "driver_height_name",\n  "DR_WGT": "driver_weight",\n  "DR_WGTNAME": "driver_weight_name",\n  "PREV_ACC": "previous_recorded_crashes",\n  "PREV_ACCNAME": "previous_recorded_crashes_name",\n  "PREV_SUS1": "previous_recorded_suspensions_and_revocations1",\n  "PREV_SUS1NAME": "previous_recorded_suspensions_and_revocations1_name",\n  "PREV_SUS2": "previous_recorded_suspensions_and_revocations2",\n  "PREV_SUS2NAME": "previous_recorded_suspensions_and_revocations2_name",\n  "PREV_SUS3": "previous_recorded_suspensions_and_revocations3",\n  "PREV_SUS3NAME": "previous_recorded_suspensions_and_revocations3_name",\n  "PREV_DWI": "previous_dwi_convictions",\n  "PREV_DWINAME": "previous_dwi_convictions_name",\n  "PREV_SPD": "previous_speeding_convictions",\n  "PREV_SPDNAME": "previous_speeding_convictions_name",\n  "PREV_OTH": "previous_other_moving_violation_convictions",\n  "PREV_OTHNAME": "previous_other_moving_violation_convictions_name",\n  "FIRST_MO": "month_of_first_crash_suspension_or_conviction",\n  "FIRST_MONAME": "month_of_first_crash_suspension_or_conviction_name",\n  "FIRST_YR": "year_of_first_crash_suspension_or_conviction",\n  "FIRST_YRNAME": "year_of_first_crash_suspension_or_conviction_name",\n  "LAST_MO": "month_of_last_crash_suspension_or_conviction",\n  "LAST_MONAME": "month_of_last_crash_suspension_or_conviction_name",\n  "LAST_YR": "year_of_last_crash_suspension_or_conviction",\n  "LAST_YRNAME": "year_of_last_crash_suspension_or_conviction_name",\n  "SPEEDREL": "speeding_related",\n  "SPEEDRELNAME": "speeding_related_name",\n  "VTRAFWAY": "trafficway_description",\n  "VTRAFWAYNAME": "trafficway_description_name",\n  "VNUM_LAN": "total_lanes_in_roadway",\n  "VNUM_LANNAME": "total_lanes_in_roadway_name",\n  "VSPD_LIM": "speed_limit",\n  "VSPD_LIMNAME": "speed_limit_name",\n  "VALIGN": "roadway_alignment",\n  "VALIGNNAME": "roadway_alignment_name",\n  "VPROFILE": "roadway_grade",\n  "VPROFILENAME": "roadway_grade_name",\n  "VPAVETYP": "roadway_surface_type",\n  "VPAVETYPNAME": "roadway_surface_type_name",\n  "VSURCOND": "roadway_surface_condition",\n  "VSURCONDNAME": "roadway_surface_condition_name",\n  "VTRAFCON": "traffic_control_device",\n  "VTRAFCONNAME": "traffic_control_device_name",\n  "VTCONT_F": "traffic_control_device_functioning",\n  "VTCONT_FNAME": "traffic_control_device_functioning_name",\n  "P_CRASH1": "pre_event_movement_prior_to_recognition_of_critical_event",\n  "P_CRASH1NAME": "pre_event_movement_prior_to_recognition_of_critical_event_name",\n  "P_CRASH2": "critical_event_precrash",\n  "P_CRASH2NAME": "critical_event_precrash_name",\n  "P_CRASH3": "attempted_avoidance_maneuver",\n  "P_CRASH3NAME": "attempted_avoidance_maneuver_name",\n  "PCRASH4": "pre_impact_stability",\n  "PCRASH4NAME": "pre_impact_stability_name",\n  "PCRASH5": "pre_impact_location",\n  "PCRASH5NAME": "pre_impact_location_name",\n  "ACC_TYPE": "crash_type",\n  "ACC_TYPENAME": "crash_type_name",\n  "DEATHS": "fatalities_in_vehicle",\n  "DR_DRINK": "driver_drinking",\n  "DR_DRINKNAME": "driver_drinking_name",\n  "TRLR1VIN": "trailer_vehicle_identification_number_1",\n  "TRLR1VINNAME": "trailer_vehicle_identification_number_1_name",\n  "TRLR2VIN": "trailer_vehicle_identification_number_2",\n  "TRLR2VINNAME": "trailer_vehicle_identification_number_2_name",\n  "TRLR3VIN": "trailer_vehicle_identification_number_3",\n  "TRLR3VINNAME": "trailer_vehicle_identification_number_3_name",\n  "VPICMAKE": "vpic_make",\n  "VPICMAKENAME": "vpic_make_name",\n  "VPICMODEL": "vpic_model",\n  "VPICMODELNAME": "vpic_model_name",\n  "VPICBODYCLASS": "vpic_body_class",\n  "VPICBODYCLASSNAME": "vpic_body_class_name",\n  "ICFINALBODY": "final_stage_body_class",\n  "ICFINALBODYNAME": "final_stage_body_class_name",\n  "GVWR_FROM": "power_unit_gross_vehicle_weight_rating_from",\n  "GVWR_FROMNAME": "power_unit_gross_vehicle_weight_rating_from_name",\n  "GVWR_TO": "power_unit_gross_vehicle_weight_rating_to",\n  "GVWR_TONAME": "power_unit_gross_vehicle_weight_rating_to_name",\n  "TRLR1GVWR": "trailer_gross_vehicle_weight_rating_1",\n  "TRLR1GVWRNAME": "trailer_gross_vehicle_weight_rating_1_name",\n  "TRLR2GVWR": "trailer_gross_vehicle_weight_rating_2",\n  "TRLR2GVWRNAME": "trailer_gross_vehicle_weight_rating_2_name",\n  "TRLR3GVWR": "trailer_gross_vehicle_weight_rating_3",\n  "TRLR3GVWRNAME": "trailer_gross_vehicle_weight_rating_3_name"\n}',
        },
        resources={
            "request_ephemeral_storage": "4G",
            "request_cpu": "1",
            "request_memory": "4G",
        },
    )

    # Run CSV transform within kubernetes pod for vevent pipelines
    vevent_2015_2020_transform_csv = kubernetes_pod.KubernetesPodOperator(
        task_id="vevent_2015_2020_transform_csv",
        startup_timeout_seconds=600,
        name="vevent",
        namespace="composer",
        service_account_name="datasets",
        image_pull_policy="Always",
        image="{{ var.json.nhtsa_traffic_fatalities.container_registry.run_csv_transform_kub }}",
        env_vars={
            "PIPELINE_NAME": "{{ var.json.nhtsa_traffic_fatalities.vevent_2015_2020.pipeline_name }}",
            "SOURCE_URL": "{{ var.json.nhtsa_traffic_fatalities.vevent_2015_2020.source_url }}",
            "CHUNKSIZE": "{{ var.json.nhtsa_traffic_fatalities.vevent_2015_2020.chunksize }}",
            "SOURCE_ZIPFILE_EXTRACTED": "vevent.csv",
            "SOURCE_FILE": "{{ var.json.nhtsa_traffic_fatalities.vevent_2015_2020.source_file }}",
            "PROJECT_ID": "{{ var.value.gcp_project }}",
            "DATASET_ID": "{{ var.json.nhtsa_traffic_fatalities.vevent_2015_2020.dataset_id }}",
            "TABLE_ID": "{{ var.json.nhtsa_traffic_fatalities.vevent_2015_2020.destination_table }}",
            "START_YEAR": "{{ var.json.nhtsa_traffic_fatalities.vevent_2015_2020.start_year }}",
            "END_YEAR": "{{ var.json.nhtsa_traffic_fatalities.vevent_2015_2020.end_year }}",
            "DROP_DEST_TABLE": "{{ var.json.nhtsa_traffic_fatalities.vevent_2015_2020.drop_dest_table }}",
            "TARGET_GCS_BUCKET": "{{ var.value.composer_bucket }}",
            "TARGET_GCS_PATH": "{{ var.json.nhtsa_traffic_fatalities.vevent_2015_2020.target_gcs_path }}",
            "SCHEMA_PATH": "{{ var.json.nhtsa_traffic_fatalities.vevent_2015_2020.schema_path }}",
            "INPUT_CSV_HEADERS": '[\n  "state_number",\n  "state_name",\n  "consecutive_number",\n  "event_number",\n  "vehicle_number",\n  "vehicle_event_number",\n  "vehicle_number_this_vehicle",\n  "area_of_impact_this_vehicle",\n  "area_of_impact_this_vehicle_name",\n  "sequence_of_events",\n  "sequence_of_events_name",\n  "vehicle_number_other_vehicle",\n  "vehicle_number_other_vehicle_name",\n  "area_of_impact_other_vehicle",\n  "area_of_impact_other_vehicle_name"\n]',
            "INPUT_DTYPES": '{\n  "state_number": "str",\n  "state_name": "str",\n  "consecutive_number": "str",\n  "event_number": "str",\n  "vehicle_number": "str",\n  "vehicle_event_number": "str",\n  "vehicle_number_this_vehicle": "str",\n  "area_of_impact_this_vehicle": "str",\n  "area_of_impact_this_vehicle_name": "str",\n  "sequence_of_events": "str",\n  "sequence_of_events_name": "str",\n  "vehicle_number_other_vehicle": "str",\n  "vehicle_number_other_vehicle_name": "str",\n  "area_of_impact_other_vehicle": "str",\n  "area_of_impact_other_vehicle_name": "str"\n}',
            "RENAME_MAPPINGS_LIST": '{\n  "STATE": "state_number",\n  "STATENAME": "state_name",\n  "ST_CASE": "consecutive_number",\n  "EVENTNUM": "event_number",\n  "VEH_NO": "vehicle_number",\n  "VEVENTNUM": "vehicle_event_number",\n  "VNUMBER1": "vehicle_number_this_vehicle",\n  "AOI1": "area_of_impact_this_vehicle",\n  "AOI1NAME": "area_of_impact_this_vehicle_name",\n  "SOE": "sequence_of_events",\n  "SOENAME": "sequence_of_events_name",\n  "VNUMBER2": "vehicle_number_other_vehicle",\n  "VNUMBER2NAME": "vehicle_number_other_vehicle_name",\n  "AOI2": "area_of_impact_other_vehicle",\n  "AOI2NAME": "area_of_impact_other_vehicle_name"\n}',
        },
        resources={
            "request_ephemeral_storage": "4G",
            "request_cpu": "1",
            "request_memory": "4G",
        },
    )

    # Run CSV transform within kubernetes pod for vindecode pipelines
    vindecode_2015_transform_csv = kubernetes_pod.KubernetesPodOperator(
        task_id="vindecode_2015_transform_csv",
        startup_timeout_seconds=600,
        name="vindecode",
        namespace="composer",
        service_account_name="datasets",
        image_pull_policy="Always",
        image="{{ var.json.nhtsa_traffic_fatalities.container_registry.run_csv_transform_kub }}",
        env_vars={
            "PIPELINE_NAME": "{{ var.json.nhtsa_traffic_fatalities.vindecode_2015.pipeline_name }}",
            "SOURCE_URL": "{{ var.json.nhtsa_traffic_fatalities.vindecode_2015.source_url }}",
            "CHUNKSIZE": "{{ var.json.nhtsa_traffic_fatalities.vindecode_2015.chunksize }}",
            "SOURCE_ZIPFILE_EXTRACTED": "vindecode.csv",
            "SOURCE_FILE": "{{ var.json.nhtsa_traffic_fatalities.vindecode_2015.source_file }}",
            "PROJECT_ID": "{{ var.value.gcp_project }}",
            "DATASET_ID": "{{ var.json.nhtsa_traffic_fatalities.vindecode_2015.dataset_id }}",
            "TABLE_ID": "{{ var.json.nhtsa_traffic_fatalities.vindecode_2015.destination_table }}",
            "START_YEAR": "{{ var.json.nhtsa_traffic_fatalities.vindecode_2015.start_year }}",
            "END_YEAR": "{{ var.json.nhtsa_traffic_fatalities.vindecode_2015.end_year }}",
            "DROP_DEST_TABLE": "{{ var.json.nhtsa_traffic_fatalities.vindecode_2015.drop_dest_table }}",
            "TARGET_GCS_BUCKET": "{{ var.value.composer_bucket }}",
            "TARGET_GCS_PATH": "{{ var.json.nhtsa_traffic_fatalities.vindecode_2015.target_gcs_path }}",
            "SCHEMA_PATH": "{{ var.json.nhtsa_traffic_fatalities.vindecode_2015.schema_path }}",
            "INPUT_CSV_HEADERS": '[\n  "state_number",\n  "state_name",\n  "consecutive_number",\n  "vehicle_number",\n  "vehicle_make",\n  "marketing_year",\n  "vehicle_type_code",\n  "vehicle_type",\n  "make_name",\n  "model_code",\n  "vehicle_trim",\n  "vehicle_trim_1",\n  "vehicle_trim_2",\n  "vehicle_trim_3",\n  "vehicle_trim_4",\n  "body_style_code",\n  "body_style",\n  "num_of_doors",\n  "number_of_wheels",\n  "num_of_wheels_by_power_train",\n  "vehicle_manufacturer_code",\n  "vehicle_manufacturer_name",\n  "displacement_cid",\n  "displacement_cc",\n  "cylinder_count_code",\n  "cycle_count",\n  "fuel_code",\n  "fuel",\n  "type_of_fuel_code",\n  "type_of_fuel",\n  "carburetion_types_code",\n  "carburetion_types",\n  "num_of_barrels",\n  "gross_vehicle_weights_range_code",\n  "gross_vehicle_weights_range",\n  "distance_between_axles_for_base_model",\n  "distance_between_axles_for_particular_series",\n  "front_tire",\n  "front_tire_pressure",\n  "front_tire_size_code",\n  "front_tire_size",\n  "rear_tire",\n  "rear_tire_pressure",\n  "rear_tire_size_code",\n  "rear_tire_size",\n  "tonnage_rating",\n  "shipping_weight",\n  "base_price",\n  "drive_type_1",\n  "drive_type_2",\n  "country_sold_code",\n  "country_sold",\n  "brakes_abs_code",\n  "brakes_abs_description",\n  "security_type_code",\n  "security_type",\n  "daytime_running_lights_1",\n  "daytime_running_lights_2",\n  "restraint_type_code",\n  "restraint_type",\n  "cab_configuration_code",\n  "cab_configuration",\n  "axle_type_front_axle_code",\n  "axle_type_front_axle",\n  "axle_type_rear_axle_code",\n  "axle_type_rear_axle",\n  "brake_type_code",\n  "brake_type",\n  "engine_manufacture_code",\n  "engine_manufacture",\n  "engine_model",\n  "duty_type_code",\n  "duty_type",\n  "bed_length_code",\n  "bed_length",\n  "standard_segmentation_code",\n  "standard_segmentation",\n  "plant_code",\n  "plant_country",\n  "plant_city",\n  "plant_country_code",\n  "plant_state_code",\n  "plant_state",\n  "origin_code",\n  "origin",\n  "displacement_liters",\n  "block_type",\n  "head_configuration_1",\n  "head_configuration_2",\n  "valves_per_cylinder",\n  "valves_total",\n  "engine_code",\n  "is_incomplete",\n  "battery_type_code",\n  "battery_type",\n  "total_battery_power",\n  "battery_voltage",\n  "supercharge_flag",\n  "supercharge_flag_description",\n  "turbocharger_flag",\n  "turbocharger_flag_description",\n  "variable_valve_timing_flag",\n  "motorcycles_body_style_code",\n  "motorcycles_body_style"\n]',
            "INPUT_DTYPES": '{\n  "state_number": "str",\n  "state_name": "str",\n  "consecutive_number": "str",\n  "vehicle_number": "str",\n  "vehicle_make": "str",\n  "marketing_year": "str",\n  "vehicle_type_code": "str",\n  "vehicle_type": "str",\n  "make_name": "str",\n  "model_code": "str",\n  "vehicle_trim": "str",\n  "vehicle_trim_1": "str",\n  "vehicle_trim_2": "str",\n  "vehicle_trim_3": "str",\n  "vehicle_trim_4": "str",\n  "body_style_code": "str",\n  "body_style": "str",\n  "num_of_doors": "str",\n  "number_of_wheels": "str",\n  "num_of_wheels_by_power_train": "str",\n  "vehicle_manufacturer_code": "str",\n  "vehicle_manufacturer_name": "str",\n  "displacement_cid": "str",\n  "displacement_cc": "str",\n  "cylinder_count_code": "str",\n  "cycle_count": "str",\n  "fuel_code": "str",\n  "fuel": "str",\n  "type_of_fuel_code": "str",\n  "type_of_fuel": "str",\n  "carburetion_types_code": "str",\n  "carburetion_types": "str",\n  "num_of_barrels": "str",\n  "gross_vehicle_weights_range_code": "str",\n  "gross_vehicle_weights_range": "str",\n  "distance_between_axles_for_base_model": "str",\n  "distance_between_axles_for_particular_series": "str",\n  "front_tire": "str",\n  "front_tire_pressure": "str",\n  "front_tire_size_code": "str",\n  "front_tire_size": "str",\n  "rear_tire": "str",\n  "rear_tire_pressure": "str",\n  "rear_tire_size_code": "str",\n  "rear_tire_size": "str",\n  "tonnage_rating": "str",\n  "shipping_weight": "str",\n  "base_price": "str",\n  "drive_type_1": "str",\n  "drive_type_2": "str",\n  "country_sold_code": "str",\n  "country_sold": "str",\n  "brakes_abs_code": "str",\n  "brakes_abs_description": "str",\n  "security_type_code": "str",\n  "security_type": "str",\n  "daytime_running_lights_1": "str",\n  "daytime_running_lights_2": "str",\n  "restraint_type_code": "str",\n  "restraint_type": "str",\n  "cab_configuration_code": "str",\n  "cab_configuration": "str",\n  "axle_type_front_axle_code": "str",\n  "axle_type_front_axle": "str",\n  "axle_type_rear_axle_code": "str",\n  "axle_type_rear_axle": "str",\n  "brake_type_code": "str",\n  "brake_type": "str",\n  "engine_manufacture_code": "str",\n  "engine_manufacture": "str",\n  "engine_model": "str",\n  "duty_type_code": "str",\n  "duty_type": "str",\n  "bed_length_code": "str",\n  "bed_length": "str",\n  "standard_segmentation_code": "str",\n  "standard_segmentation": "str",\n  "plant_code": "str",\n  "plant_country": "str",\n  "plant_city": "str",\n  "plant_country_code": "str",\n  "plant_state_code": "str",\n  "plant_state": "str",\n  "origin_code": "str",\n  "origin": "str",\n  "displacement_liters": "str",\n  "block_type": "str",\n  "head_configuration_1": "str",\n  "head_configuration_2": "str",\n  "valves_per_cylinder": "str",\n  "valves_total": "str",\n  "engine_code": "str",\n  "is_incomplete": "str",\n  "battery_type_code": "str",\n  "battery_type": "str",\n  "total_battery_power": "str",\n  "battery_voltage": "str",\n  "supercharge_flag": "str",\n  "supercharge_flag_description": "str",\n  "turbocharger_flag": "str",\n  "turbocharger_flag_description": "str",\n  "variable_valve_timing_flag": "str",\n  "motorcycles_body_style_code": "str",\n  "motorcycles_body_style": "str"\n}',
            "RENAME_MAPPINGS_LIST": '{\n  "STATE": "state_number",\n  "STATENAME": "state_name",\n  "ST_CASE": "consecutive_number",\n  "VEH_NO": "vehicle_number",\n  "NCICMAKE": "vehicle_make",\n  "VINYEAR": "marketing_year",\n  "VEHTYPE": "vehicle_type_code",\n  "VEHTYPE_T": "vehicle_type",\n  "VINMAKE_T": "make_name",\n  "VINMODEL_T": "model_code",\n  "VINTRIM_T": "vehicle_trim",\n  "VINTRIM1_T": "vehicle_trim_1",\n  "VINTRIM2_T": "vehicle_trim_2",\n  "VINTRIM3_T": "vehicle_trim_3",\n  "VINTRIM4_T": "vehicle_trim_4",\n  "BODYSTYL": "body_style_code",\n  "BODYSTYL_T": "body_style",\n  "DOORS": "num_of_doors",\n  "WHEELS": "number_of_wheels",\n  "DRIVWHLS": "num_of_wheels_by_power_train",\n  "MFG": "vehicle_manufacturer_code",\n  "MFG_T": "vehicle_manufacturer_name",\n  "DISPLCI": "displacement_cid",\n  "DISPLCC": "displacement_cc",\n  "CYLNDRS": "cylinder_count_code",\n  "CYCLES": "cycle_count",\n  "FUEL": "fuel_code",\n  "FUEL_T": "fuel",\n  "FUELINJ": "type_of_fuel_code",\n  "FUELINJ_T": "type_of_fuel",\n  "CARBTYPE": "carburetion_types_code",\n  "CARBTYPE_T": "carburetion_types",\n  "CARBBRLS": "num_of_barrels",\n  "GVWRANGE": "gross_vehicle_weights_range_code",\n  "GVWRANGE_T": "gross_vehicle_weights_range",\n  "WHLBSH": "distance_between_axles_for_base_model",\n  "WHLBLG": "distance_between_axles_for_particular_series",\n  "TIREDESC_F": "front_tire",\n  "PSI_F": "front_tire_pressure",\n  "TIRESZ_F": "front_tire_size_code",\n  "TIRESZ_F_T": "front_tire_size",\n  "TIREDESC_R": "rear_tire",\n  "PSI_R": "rear_tire_pressure",\n  "REARSIZE": "rear_tire_size_code",\n  "REARSIZE_T": "rear_tire_size",\n  "TONRATING": "tonnage_rating",\n  "SHIPWEIGHT": "shipping_weight",\n  "MSRP": "base_price",\n  "DRIVETYP": "drive_type_1",\n  "DRIVETYP_T": "drive_type_2",\n  "SALECTRY": "country_sold_code",\n  "SALECTRY_T": "country_sold",\n  "ABS": "brakes_abs_code",\n  "ABS_T": "brakes_abs_description",\n  "SECURITY": "security_type_code",\n  "SECURITY_T": "security_type",\n  "DRL": "daytime_running_lights_1",\n  "DRL_T": "daytime_running_lights_2",\n  "RSTRNT": "restraint_type_code",\n  "RSTRNT_T": "restraint_type",\n  "TKCAB": "cab_configuration_code",\n  "TKCAB_T": "cab_configuration",\n  "TKAXLEF": "axle_type_front_axle_code",\n  "TKAXLEF_T": "axle_type_front_axle",\n  "TKAXLER": "axle_type_rear_axle_code",\n  "TKAXLER_T": "axle_type_rear_axle",\n  "TKBRAK": "brake_type_code",\n  "TKBRAK_T": "brake_type",\n  "ENGMFG": "engine_manufacture_code",\n  "ENGMFG_T": "engine_manufacture",\n  "ENGMODEL": "engine_model",\n  "TKDUTY": "duty_type_code",\n  "TKDUTY_T": "duty_type",\n  "TKBEDL": "bed_length_code",\n  "TKBEDL_T": "bed_length",\n  "SEGMNT": "standard_segmentation_code",\n  "SEGMNT_T": "standard_segmentation",\n  "PLANT": "plant_code",\n  "PLNTCTRY_T": "plant_country",\n  "PLNTCITY": "plant_city",\n  "PLNTCTRY": "plant_country_code",\n  "PLNTSTAT": "plant_state_code",\n  "PLNTSTAT_T": "plant_state",\n  "ORIGIN": "origin_code",\n  "ORIGIN_T": "origin",\n  "DISPCLMT": "displacement_liters",\n  "BLOCKTYPE": "block_type",\n  "ENGHEAD": "head_configuration_1",\n  "ENGHEAD_T": "head_configuration_2",\n  "VLVCLNDR": "valves_per_cylinder",\n  "VLVTOTAL": "valves_total",\n  "ENGVINCD": "engine_code",\n  "INCOMPLT": "is_incomplete",\n  "BATTYP": "battery_type_code",\n  "BATTYP_T": "battery_type",\n  "BATKWRTG": "total_battery_power",\n  "BATVOLT": "battery_voltage",\n  "SUPCHRGR": "supercharge_flag",\n  "SUPCHRGR_T": "supercharge_flag_description",\n  "TURBO": "turbocharger_flag",\n  "TURBO_T": "turbocharger_flag_description",\n  "ENGVVT": "variable_valve_timing_flag",\n  "MCYUSAGE": "motorcycles_body_style_code",\n  "MCYUSAGE_T": "motorcycles_body_style"\n}',
        },
        resources={
            "request_ephemeral_storage": "4G",
            "request_cpu": "1",
            "request_memory": "4G",
        },
    )

    # Run CSV transform within kubernetes pod for violatn pipelines
    violatn_2015_2020_transform_csv = kubernetes_pod.KubernetesPodOperator(
        task_id="violatn_2015_2020_transform_csv",
        startup_timeout_seconds=600,
        name="violatn",
        namespace="composer",
        service_account_name="datasets",
        image_pull_policy="Always",
        image="{{ var.json.nhtsa_traffic_fatalities.container_registry.run_csv_transform_kub }}",
        env_vars={
            "PIPELINE_NAME": "{{ var.json.nhtsa_traffic_fatalities.violatn_2015_2020.pipeline_name }}",
            "SOURCE_URL": "{{ var.json.nhtsa_traffic_fatalities.violatn_2015_2020.source_url }}",
            "CHUNKSIZE": "{{ var.json.nhtsa_traffic_fatalities.violatn_2015_2020.chunksize }}",
            "SOURCE_ZIPFILE_EXTRACTED": "violatn.csv",
            "SOURCE_FILE": "{{ var.json.nhtsa_traffic_fatalities.violatn_2015_2020.source_file }}",
            "PROJECT_ID": "{{ var.value.gcp_project }}",
            "DATASET_ID": "{{ var.json.nhtsa_traffic_fatalities.violatn_2015_2020.dataset_id }}",
            "TABLE_ID": "{{ var.json.nhtsa_traffic_fatalities.violatn_2015_2020.destination_table }}",
            "START_YEAR": "{{ var.json.nhtsa_traffic_fatalities.violatn_2015_2020.start_year }}",
            "END_YEAR": "{{ var.json.nhtsa_traffic_fatalities.violatn_2015_2020.end_year }}",
            "DROP_DEST_TABLE": "{{ var.json.nhtsa_traffic_fatalities.violatn_2015_2020.drop_dest_table }}",
            "TARGET_GCS_BUCKET": "{{ var.value.composer_bucket }}",
            "TARGET_GCS_PATH": "{{ var.json.nhtsa_traffic_fatalities.violatn_2015_2020.target_gcs_path }}",
            "SCHEMA_PATH": "{{ var.json.nhtsa_traffic_fatalities.violatn_2015_2020.schema_path }}",
            "INPUT_CSV_HEADERS": '[\n  "state_number",\n  "state_name",\n  "consecutive_number",\n  "vehicle_number",\n  "violations_charged",\n  "violations_charged_name"\n]',
            "INPUT_DTYPES": '{\n  "state_number": "str",\n  "state_name": "str",\n  "consecutive_number": "str",\n  "vehicle_number": "str",\n  "violations_charged": "str",\n  "violations_charged_name": "str"\n}',
            "RENAME_MAPPINGS_LIST": '{\n  "STATE": "state_number",\n  "STATENAME": "state_name",\n  "ST_CASE": "consecutive_number",\n  "VEH_NO": "vehicle_number",\n  "MVIOLATN": "violations_charged",\n  "MVIOLATNNAME": "violations_charged_name"\n}',
        },
        resources={
            "request_ephemeral_storage": "4G",
            "request_cpu": "1",
            "request_memory": "4G",
        },
    )

    # Run CSV transform within kubernetes pod for vision pipelines
    vision_2015_2020_transform_csv = kubernetes_pod.KubernetesPodOperator(
        task_id="vision_2015_2020_transform_csv",
        startup_timeout_seconds=600,
        name="vision",
        namespace="composer",
        service_account_name="datasets",
        image_pull_policy="Always",
        image="{{ var.json.nhtsa_traffic_fatalities.container_registry.run_csv_transform_kub }}",
        env_vars={
            "PIPELINE_NAME": "{{ var.json.nhtsa_traffic_fatalities.vision_2015_2020.pipeline_name }}",
            "SOURCE_URL": "{{ var.json.nhtsa_traffic_fatalities.vision_2015_2020.source_url }}",
            "CHUNKSIZE": "{{ var.json.nhtsa_traffic_fatalities.vision_2015_2020.chunksize }}",
            "SOURCE_ZIPFILE_EXTRACTED": "vision.csv",
            "SOURCE_FILE": "{{ var.json.nhtsa_traffic_fatalities.vision_2015_2020.source_file }}",
            "PROJECT_ID": "{{ var.value.gcp_project }}",
            "DATASET_ID": "{{ var.json.nhtsa_traffic_fatalities.vision_2015_2020.dataset_id }}",
            "TABLE_ID": "{{ var.json.nhtsa_traffic_fatalities.vision_2015_2020.destination_table }}",
            "START_YEAR": "{{ var.json.nhtsa_traffic_fatalities.vision_2015_2020.start_year }}",
            "END_YEAR": "{{ var.json.nhtsa_traffic_fatalities.vision_2015_2020.end_year }}",
            "DROP_DEST_TABLE": "{{ var.json.nhtsa_traffic_fatalities.vision_2015_2020.drop_dest_table }}",
            "TARGET_GCS_BUCKET": "{{ var.value.composer_bucket }}",
            "TARGET_GCS_PATH": "{{ var.json.nhtsa_traffic_fatalities.vision_2015_2020.target_gcs_path }}",
            "SCHEMA_PATH": "{{ var.json.nhtsa_traffic_fatalities.vision_2015_2020.schema_path }}",
            "INPUT_CSV_HEADERS": '[\n  "state_number",\n  "state_name",\n  "consecutive_number",\n  "vehicle_number",\n  "drivers_vision_obscured_by",\n  "drivers_vision_obscured_by_name"\n]',
            "INPUT_DTYPES": '{\n  "state_number": "str",\n  "state_name": "str",\n  "consecutive_number": "str",\n  "vehicle_number": "str",\n  "drivers_vision_obscured_by": "str",\n  "drivers_vision_obscured_by_name": "str"\n}',
            "RENAME_MAPPINGS_LIST": '{\n  "STATE": "state_number",\n  "STATENAME": "state_name",\n  "ST_CASE": "consecutive_number",\n  "VEH_NO": "vehicle_number",\n  "MVISOBSC": "drivers_vision_obscured_by",\n  "MVISOBSCNAME": "drivers_vision_obscured_by_name"\n}',
        },
        resources={
            "request_ephemeral_storage": "4G",
            "request_cpu": "1",
            "request_memory": "4G",
        },
    )

    # Run CSV transform within kubernetes pod for vsoe pipelines
    vsoe_2015_2020_transform_csv = kubernetes_pod.KubernetesPodOperator(
        task_id="vsoe_2015_2020_transform_csv",
        startup_timeout_seconds=600,
        name="vsoe",
        namespace="composer",
        service_account_name="datasets",
        image_pull_policy="Always",
        image="{{ var.json.nhtsa_traffic_fatalities.container_registry.run_csv_transform_kub }}",
        env_vars={
            "PIPELINE_NAME": "{{ var.json.nhtsa_traffic_fatalities.vsoe_2015_2020.pipeline_name }}",
            "SOURCE_URL": "{{ var.json.nhtsa_traffic_fatalities.vsoe_2015_2020.source_url }}",
            "CHUNKSIZE": "{{ var.json.nhtsa_traffic_fatalities.vsoe_2015_2020.chunksize }}",
            "SOURCE_ZIPFILE_EXTRACTED": "vsoe.csv",
            "SOURCE_FILE": "{{ var.json.nhtsa_traffic_fatalities.vsoe_2015_2020.source_file }}",
            "PROJECT_ID": "{{ var.value.gcp_project }}",
            "DATASET_ID": "{{ var.json.nhtsa_traffic_fatalities.vsoe_2015_2020.dataset_id }}",
            "TABLE_ID": "{{ var.json.nhtsa_traffic_fatalities.vsoe_2015_2020.destination_table }}",
            "START_YEAR": "{{ var.json.nhtsa_traffic_fatalities.vsoe_2015_2020.start_year }}",
            "END_YEAR": "{{ var.json.nhtsa_traffic_fatalities.vsoe_2015_2020.end_year }}",
            "DROP_DEST_TABLE": "{{ var.json.nhtsa_traffic_fatalities.vsoe_2015_2020.drop_dest_table }}",
            "TARGET_GCS_BUCKET": "{{ var.value.composer_bucket }}",
            "TARGET_GCS_PATH": "{{ var.json.nhtsa_traffic_fatalities.vsoe_2015_2020.target_gcs_path }}",
            "SCHEMA_PATH": "{{ var.json.nhtsa_traffic_fatalities.vsoe_2015_2020.schema_path }}",
            "INPUT_CSV_HEADERS": '[\n  "state_number",\n  "state_name",\n  "consecutive_number",\n  "vehicle_number",\n  "vehicle_event_number",\n  "sequence_of_events",\n  "sequence_of_events_name",\n  "area_of_Impact_associated_with_the_event",\n  "area_of_Impact_associated_with_the_event_name"\n]',
            "INPUT_DTYPES": '{\n  "state_number": "str",\n  "state_name": "str",\n  "consecutive_number": "str",\n  "vehicle_number": "str",\n  "vehicle_event_number": "str",\n  "sequence_of_events": "str",\n  "sequence_of_events_name": "str",\n  "area_of_Impact_associated_with_the_event": "str",\n  "area_of_Impact_associated_with_the_event_name": "str"\n}',
            "RENAME_MAPPINGS_LIST": '{\n  "STATE": "state_number",\n  "STATENAME": "state_name",\n  "ST_CASE": "consecutive_number",\n  "VEH_NO": "vehicle_number",\n  "VEVENTNUM": "vehicle_event_number",\n  "SOE": "sequence_of_events",\n  "SOENAME": "sequence_of_events_name",\n  "AOI": "area_of_Impact_associated_with_the_event",\n  "AOINAME": "area_of_Impact_associated_with_the_event_name"\n}',
        },
        resources={
            "request_ephemeral_storage": "4G",
            "request_cpu": "1",
            "request_memory": "4G",
        },
    )
    delete_cluster = kubernetes_engine.GKEDeleteClusterOperator(
        task_id="delete_cluster",
        project_id="{{ var.value.gcp_project }}",
        location="us-central1-c",
        name="nhtsa-traffic-fatalities",
    )

    (
        create_cluster
        >> [
            accident_2015_transform_csv,
            accident_2016_2019_transform_csv,
            accident_2020_transform_csv,
            cevent_2015_2020_transform_csv,
            damage_2015_2020_transform_csv,
            distract_2015_2020_transform_csv,
            drimpair_2015_2020_transform_csv,
            factor_2015_2020_transform_csv,
            maneuver_2015_2020_transform_csv,
            nmcrash_2015_2020_transform_csv,
            nmimpair_2015_2020_transform_csv,
            nmprior_2015_2020_transform_csv,
            parkwork_2015_transform_csv,
            parkwork_2016_2017_transform_csv,
            parkwork_2018_transform_csv,
            parkwork_2019_transform_csv,
            parkwork_2020_transform_csv,
            pbtype_transform_csv,
            person_2015_2017_transform_csv,
            person_2018_transform_csv,
            person_2019_transform_csv,
            person_2020_transform_csv,
            safetyeq_2015_2016_transform_csv,
            safetyeq_2017_2020_transform_csv,
            vehicle_2015_transform_csv,
            vehicle_2016_2017_transform_csv,
            vehicle_2018_2019_transform_csv,
            vehicle_2020_transform_csv,
            vevent_2015_2020_transform_csv,
            vindecode_2015_transform_csv,
            violatn_2015_2020_transform_csv,
            vision_2015_2020_transform_csv,
            vsoe_2015_2020_transform_csv,
        ]
        >> delete_cluster
    )
